@@ -31,6 +31,7 @@ module Annex (
 	overrideGitConfig,
 	changeGitRepo,
 	adjustGitRepo,
+	addGitConfigOverride,
 	getRemoteGitConfig,
 	withCurrentState,
 	changeDirectory,
@@ -43,7 +44,6 @@ import qualified Git
 import qualified Git.Config
 import qualified Git.Construct
 import Annex.Fixup
-import Git.CatFile
 import Git.HashObject
 import Git.CheckAttr
 import Git.CheckIgnore
@@ -67,9 +67,12 @@ import Types.DesktopNotify
 import Types.CleanupActions
 import Types.AdjustedBranch
 import Types.WorkerPool
+import Types.IndexFiles
+import Types.CatFileHandles
 import qualified Database.Keys.Handle as Keys
 import Utility.InodeCache
 import Utility.Url
+import Utility.ResourcePool
 
 import "mtl" Control.Monad.Reader
 import Control.Concurrent
@@ -108,7 +111,6 @@ data AnnexState = AnnexState
 	, gitremotes :: Maybe [Git.Repo]
 	, backend :: Maybe (BackendA Annex)
 	, remotes :: [Types.Remote.RemoteA Annex]
-	, remoteannexstate :: M.Map UUID AnnexState
 	, output :: MessageState
 	, concurrency :: Concurrency
 	, force :: Bool
@@ -116,10 +118,10 @@ data AnnexState = AnnexState
 	, daemon :: Bool
 	, branchstate :: BranchState
 	, repoqueue :: Maybe (Git.Queue.Queue Annex)
-	, catfilehandles :: M.Map FilePath CatFileHandle
+	, catfilehandles :: CatFileHandles
 	, hashobjecthandle :: Maybe HashObjectHandle
-	, checkattrhandle :: Maybe CheckAttrHandle
-	, checkignorehandle :: Maybe CheckIgnoreHandle
+	, checkattrhandle :: Maybe (ResourcePool CheckAttrHandle)
+	, checkignorehandle :: Maybe (ResourcePool CheckIgnoreHandle)
 	, forcebackend :: Maybe String
 	, globalnumcopies :: Maybe NumCopies
 	, forcenumcopies :: Maybe NumCopies
@@ -146,10 +148,11 @@ data AnnexState = AnnexState
 	, workers :: Maybe (TMVar (WorkerPool AnnexState))
 	, activekeys :: TVar (M.Map Key ThreadId)
 	, activeremotes :: MVar (M.Map (Types.Remote.RemoteA Annex) Integer)
-	, keysdbhandle :: Maybe Keys.DbHandle
+	, keysdbhandle :: Keys.DbHandle
 	, cachedcurrentbranch :: (Maybe (Maybe Git.Branch, Maybe Adjustment))
-	, cachedgitenv :: Maybe (FilePath, [(String, String)])
+	, cachedgitenv :: Maybe (AltIndexFile, FilePath, [(String, String)])
 	, urloptions :: Maybe UrlOptions
+	, insmudgecleanfilter :: Bool
 	}
 
 newState :: GitConfig -> Git.Repo -> IO AnnexState
@@ -158,6 +161,7 @@ newState c r = do
 	emptyactivekeys <- newTVarIO M.empty
 	o <- newMessageState
 	sc <- newTMVarIO False
+	kh <- Keys.newDbHandle
 	return $ AnnexState
 		{ repo = r
 		, repoadjustment = return
@@ -166,7 +170,6 @@ newState c r = do
 		, gitremotes = Nothing
 		, backend = Nothing
 		, remotes = []
-		, remoteannexstate = M.empty
 		, output = o
 		, concurrency = NonConcurrent
 		, force = False
@@ -174,7 +177,7 @@ newState c r = do
 		, daemon = False
 		, branchstate = startBranchState
 		, repoqueue = Nothing
-		, catfilehandles = M.empty
+		, catfilehandles = catFileHandlesNonConcurrent
 		, hashobjecthandle = Nothing
 		, checkattrhandle = Nothing
 		, checkignorehandle = Nothing
@@ -204,10 +207,11 @@ newState c r = do
 		, workers = Nothing
 		, activekeys = emptyactivekeys
 		, activeremotes = emptyactiveremotes
-		, keysdbhandle = Nothing
+		, keysdbhandle = kh
 		, cachedcurrentbranch = Nothing
 		, cachedgitenv = Nothing
 		, urloptions = Nothing
+		, insmudgecleanfilter = False
 		}
 
 {- Makes an Annex state object for the specified git repo.
@@ -232,7 +236,7 @@ run' mvar a = do
 	flush s'
 	return (r, s')
   where
-	flush = maybe noop Keys.flushDbQueue . keysdbhandle
+	flush = Keys.flushDbQueue . keysdbhandle
 
 {- Performs an action in the Annex monad from a starting state, 
  - and throws away the new state. -}
@@ -335,6 +339,21 @@ adjustGitRepo a = do
 	changeState $ \s -> s { repoadjustment = \r -> repoadjustment s r >>= a }
 	changeGitRepo =<< gitRepo
 
+{- Adds git config setting, like "foo=bar". It will be passed with -c
+ - to git processes. The config setting is also recorded in the repo,
+ - and the GitConfig is updated. -}
+addGitConfigOverride :: String -> Annex ()
+addGitConfigOverride v = adjustGitRepo $ \r ->
+	Git.Config.store (encodeBS' v) Git.Config.ConfigList $
+		r { Git.gitGlobalOpts = go (Git.gitGlobalOpts r) }
+  where
+	-- Remove any prior occurrance of the setting to avoid
+	-- building up many of them when the adjustment is run repeatedly,
+	-- and add the setting to the end.
+	go [] = [Param "-c", Param v]
+	go (Param "-c": Param v':rest) | v' == v = go rest
+	go (c:rest) = c : go rest
+
 {- Changing the git Repo data also involves re-extracting its GitConfig. -}
 changeGitRepo :: Git.Repo -> Annex ()
 changeGitRepo r = do
