@@ -5,6 +5,8 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Annex.AdjustedBranch.Name (
 	originalToAdjusted,
 	adjustedToOriginal,
@@ -18,14 +20,15 @@ import qualified Git.Ref
 import Utility.Misc
 
 import Control.Applicative
-import Data.List
+import Data.Char
+import qualified Data.ByteString as S
 
-adjustedBranchPrefix :: String
+adjustedBranchPrefix :: S.ByteString
 adjustedBranchPrefix = "refs/heads/adjusted/"
 
 class SerializeAdjustment t where
-	serializeAdjustment :: t -> String
-	deserializeAdjustment :: String -> Maybe t
+	serializeAdjustment :: t -> S.ByteString
+	deserializeAdjustment :: S.ByteString -> Maybe t
 
 instance SerializeAdjustment Adjustment where
 	serializeAdjustment (LinkAdjustment l) =
@@ -33,7 +36,7 @@ instance SerializeAdjustment Adjustment where
 	serializeAdjustment (PresenceAdjustment p Nothing) =
 		serializeAdjustment p
 	serializeAdjustment (PresenceAdjustment p (Just l)) = 
-		serializeAdjustment p ++ "-" ++ serializeAdjustment l
+		serializeAdjustment p <> "-" <> serializeAdjustment l
 	deserializeAdjustment s = 
 		(LinkAdjustment <$> deserializeAdjustment s)
 			<|>
@@ -41,7 +44,7 @@ instance SerializeAdjustment Adjustment where
 			<|>
 		(PresenceAdjustment <$> deserializeAdjustment s <*> pure Nothing)
 	  where
-		(s1, s2) = separate (== '-') s
+		(s1, s2) = separate' (== (fromIntegral (ord '-'))) s
 
 instance SerializeAdjustment LinkAdjustment where
 	serializeAdjustment UnlockAdjustment = "unlocked"
@@ -65,19 +68,21 @@ newtype AdjBranch = AdjBranch { adjBranch :: Branch }
 
 originalToAdjusted :: OrigBranch -> Adjustment -> AdjBranch
 originalToAdjusted orig adj = AdjBranch $ Ref $
-	adjustedBranchPrefix ++ base ++ '(' : serializeAdjustment adj ++ ")"
+	adjustedBranchPrefix <> base <> "(" <> serializeAdjustment adj <> ")"
   where
-	base = fromRef (Git.Ref.base orig)
+	base = fromRef' (Git.Ref.base orig)
 
 type OrigBranch = Branch
 
 adjustedToOriginal :: Branch -> Maybe (Adjustment, OrigBranch)
 adjustedToOriginal b
-	| adjustedBranchPrefix `isPrefixOf` bs = do
-		let (base, as) = separate (== '(') (drop prefixlen bs)
-		adj <- deserializeAdjustment (takeWhile (/= ')') as)
+	| adjustedBranchPrefix `S.isPrefixOf` bs = do
+		let (base, as) = separate' (== openparen) (S.drop prefixlen bs)
+		adj <- deserializeAdjustment (S.takeWhile (/= closeparen) as)
 		Just (adj, Git.Ref.branchRef (Ref base))
 	| otherwise = Nothing
   where
-	bs = fromRef b
-	prefixlen = length adjustedBranchPrefix
+	bs = fromRef' b
+	prefixlen = S.length adjustedBranchPrefix
+	openparen = fromIntegral (ord '(')
+	closeparen = fromIntegral (ord ')')
