@@ -1,6 +1,6 @@
 {- git-annex automatic merge conflict resolution
  -
- - Copyright 2012-2016 Joey Hess <id@joeyh.name>
+ - Copyright 2012-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -9,11 +9,13 @@
 
 module Annex.AutoMerge
 	( autoMergeFrom
+	, autoMergeFrom'
 	, resolveMerge
 	, commitResolvedMerge
 	) where
 
 import Annex.Common
+import qualified Annex
 import qualified Annex.Queue
 import Annex.CatFile
 import Annex.Link
@@ -36,6 +38,7 @@ import Utility.FileMode
 import qualified Data.Set as S
 import qualified Data.Map as M
 import qualified Data.ByteString.Lazy as L
+import qualified Utility.RawFilePath as R
 
 {- Merges from a branch into the current branch (which may not exist yet),
  - with automatic merge conflict resolution.
@@ -43,27 +46,49 @@ import qualified Data.ByteString.Lazy as L
  - Callers should use Git.Branch.changed first, to make sure that
  - there are changes from the current branch to the branch being merged in.
  -}
-autoMergeFrom :: Git.Ref -> Maybe Git.Ref -> [Git.Merge.MergeConfig] -> Annex Bool -> Git.Branch.CommitMode -> Annex Bool
-autoMergeFrom branch currbranch mergeconfig canresolvemerge commitmode = do
+autoMergeFrom :: Git.Ref -> Maybe Git.Ref -> [Git.Merge.MergeConfig] -> Git.Branch.CommitMode -> Bool -> Annex Bool
+autoMergeFrom branch currbranch mergeconfig commitmode canresolvemerge =
+	autoMergeFrom' branch currbranch mergeconfig commitmode canresolvemerge resolvemerge
+  where
+	resolvemerge old
+		| canresolvemerge = resolveMerge old branch False
+		| otherwise = return False 
+
+autoMergeFrom' :: Git.Ref -> Maybe Git.Ref -> [Git.Merge.MergeConfig] -> Git.Branch.CommitMode -> Bool -> (Maybe Git.Ref -> Annex Bool) -> Annex Bool
+autoMergeFrom' branch currbranch mergeconfig commitmode willresolvemerge toresolvemerge = do
 	showOutput
 	case currbranch of
 		Nothing -> go Nothing
 		Just b -> go =<< inRepo (Git.Ref.sha b)
   where
 	go old = do
-			r <- inRepo (Git.Merge.merge branch mergeconfig commitmode)
-				<||> (resolvemerge <&&> commitResolvedMerge commitmode)
+			-- merge.directoryRenames=conflict plus automatic
+			-- merge conflict resolution results in files in a
+			-- "renamed" directory getting variant names,
+			-- so is not a great combination. If the user has
+			-- explicitly set it, use it, but otherwise when
+			-- merge conflicts will be resolved, override
+			-- to merge.directoryRenames=false.
+			overridedirectoryrenames <- if willresolvemerge
+				then isNothing . mergeDirectoryRenames
+					<$> Annex.getGitConfig
+				else pure False
+			let f r 
+				| overridedirectoryrenames = r
+					{ Git.gitGlobalOpts = 
+						Param "-c"
+						: Param "merge.directoryRenames=false" 
+						: Git.gitGlobalOpts r
+					}
+				| otherwise = r
+			r <- inRepo (Git.Merge.merge branch mergeconfig commitmode . f)
+				<||> (toresolvemerge old <&&> commitResolvedMerge commitmode)
 			-- Merging can cause new associated files to appear
 			-- and the smudge filter will add them to the database.
 			-- To ensure that this process sees those changes,
 			-- close the database if it was open.
 			Database.Keys.closeDb
 			return r
-	  where
-		resolvemerge = ifM canresolvemerge
-			( resolveMerge old branch False
-			, return False 
-			)
 
 {- Resolves a conflicted merge. It's important that any conflicts be
  - resolved in a way that itself avoids later merge conflicts, since
@@ -120,7 +145,7 @@ resolveMerge us them inoverlay = do
 	void $ liftIO cleanup
 
 	unless inoverlay $ do
-		(deleted, cleanup2) <- inRepo (LsFiles.deleted [top])
+		(deleted, cleanup2) <- inRepo (LsFiles.deleted [] [top])
 		unless (null deleted) $
 			Annex.Queue.addCommand "rm"
 				[Param "--quiet", Param "-f", Param "--"]
@@ -130,7 +155,8 @@ resolveMerge us them inoverlay = do
 	when merged $ do
 		Annex.Queue.flush
 		unless inoverlay $ do
-			unstagedmap <- inodeMap $ inRepo $ LsFiles.notInRepo False [top]
+			unstagedmap <- inodeMap $ inRepo $
+				LsFiles.notInRepo [] False [top]
 			cleanConflictCruft mergedks' mergedfs' unstagedmap
 		showLongNote "Merge conflict was automatically resolved; you may want to examine the result."
 	return merged
@@ -144,8 +170,8 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 		-- Both sides of conflict are annexed files
 		(Just keyUs, Just keyThem)
 			| keyUs /= keyThem -> resolveby [keyUs, keyThem] $ do
-				makeannexlink keyUs LsFiles.valUs
-				makeannexlink keyThem LsFiles.valThem
+				makevariantannexlink keyUs LsFiles.valUs
+				makevariantannexlink keyThem LsFiles.valThem
 				-- cleanConflictCruft can't handle unlocked
 				-- files, so delete here.
 				unless inoverlay $
@@ -161,13 +187,15 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 					else makepointer keyUs file (combinedmodes)
 				return ([keyUs, keyThem], Just file)
 		-- Our side is annexed file, other side is not.
+		-- Make the annexed file into a variant file and graft in the
+		-- other file/directory as it was.
 		(Just keyUs, Nothing) -> resolveby [keyUs] $ do
 			graftin them file LsFiles.valThem LsFiles.valThem LsFiles.valUs
-			makeannexlink keyUs LsFiles.valUs 
+			makevariantannexlink keyUs LsFiles.valUs 
 		-- Our side is not annexed file, other side is.
 		(Nothing, Just keyThem) -> resolveby [keyThem] $ do
 			graftin us file LsFiles.valUs LsFiles.valUs LsFiles.valThem
-			makeannexlink keyThem LsFiles.valThem
+			makevariantannexlink keyThem LsFiles.valThem
 		-- Neither side is annexed file; cannot resolve.
 		(Nothing, Nothing) -> return ([], Nothing)
   where
@@ -176,7 +204,7 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 	getkey select = 
 		case select (LsFiles.unmergedSha u) of
 			Just sha -> catKey sha
-			Nothing -> return Nothing
+			Nothing -> pure Nothing
 	
 	islocked select = select (LsFiles.unmergedTreeItemType u) == Just TreeSymlink
 
@@ -189,7 +217,7 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 		theirmode = fromTreeItemType
 			<$> LsFiles.valThem (LsFiles.unmergedTreeItemType u)
 
-	makeannexlink key select
+	makevariantannexlink key select
 		| islocked select = makesymlink key dest
 		| otherwise = makepointer key dest destmode
 	  where
@@ -207,8 +235,8 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 		dest' <- toRawFilePath <$> stagefile dest
 		stageSymlink dest' =<< hashSymlink l
 
-	replacewithsymlink dest link = withworktree dest $ \f ->
-		replaceWorkTreeFile f $ makeGitLink link . toRawFilePath
+	replacewithsymlink dest link = replaceWorkTreeFile dest $
+		makeGitLink link . toRawFilePath
 
 	makepointer key dest destmode = do
 		unless inoverlay $ 
@@ -223,8 +251,6 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 			Database.Keys.addAssociatedFile key
 				=<< inRepo (toTopFilePath (toRawFilePath dest))
 
-	withworktree f a = a f
-
 	{- Stage a graft of a directory or file from a branch
 	 - and update the work tree. -}
 	graftin b item selectwant selectwant' selectunwant = do
@@ -245,7 +271,7 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 			-- And when grafting in anything else vs a symlink,
 			-- the work tree already contains what we want.
 			(_, Just TreeSymlink) -> noop
-			_ -> ifM (withworktree item (liftIO . doesDirectoryExist))
+			_ -> ifM (liftIO $ doesDirectoryExist item)
 				-- a conflict between a file and a directory
 				-- leaves the directory, so since a directory
 				-- is there, it must be what was wanted
@@ -255,10 +281,9 @@ resolveMerge' unstagedmap (Just us) them inoverlay u = do
 				-- file content
 				, case selectwant' (LsFiles.unmergedSha u) of
 					Nothing -> noop
-					Just sha -> withworktree item $ \f -> 
-						replaceWorkTreeFile f $ \tmp -> do
-							c <- catObject sha
-							liftIO $ L.writeFile tmp c
+					Just sha -> replaceWorkTreeFile item $ \tmp -> do
+						c <- catObject sha
+						liftIO $ L.writeFile tmp c
 				)
 	
 	resolveby ks a = do
@@ -291,7 +316,7 @@ cleanConflictCruft resolvedks resolvedfs unstagedmap = do
 	inks = maybe False (flip S.member ks)
 	matchesresolved is i f
 		| S.member f fs || S.member (conflictCruftBase f) fs = anyM id
-			[ pure (S.member i is)
+			[ pure $ either (const False) (`S.member` is) i
 			, inks <$> isAnnexLink (toRawFilePath f)
 			, inks <$> liftIO (isPointerFile (toRawFilePath f))
 			]
@@ -308,7 +333,7 @@ reuseOldFile :: InodeMap -> Key -> FilePath -> FilePath -> Annex Bool
 reuseOldFile srcmap key origfile destfile = do
 	is <- map (inodeCacheToKey Strongly)
 		<$> Database.Keys.getInodeCaches key
-	liftIO $ go $ mapMaybe (\i -> M.lookup i srcmap) is
+	liftIO $ go $ mapMaybe (\i -> M.lookup (Right i) srcmap) is
   where
 	go [] = return False
 	go (f:fs)
@@ -328,15 +353,19 @@ commitResolvedMerge commitmode = inRepo $ Git.Branch.commitCommand commitmode
 	, Param "git-annex automatic merge conflict fix"
 	]
 
-type InodeMap = M.Map InodeCacheKey FilePath
+type InodeMap = M.Map (Either FilePath InodeCacheKey) FilePath
 
 inodeMap :: Annex ([RawFilePath], IO Bool) -> Annex InodeMap
 inodeMap getfiles = do
 	(fs, cleanup) <- getfiles
 	fsis <- forM fs $ \f -> do
-		mi <- withTSDelta (liftIO . genInodeCache f)
-		return $ case mi of
-			Nothing -> Nothing
-			Just i -> Just (inodeCacheToKey Strongly i, fromRawFilePath f)
+		s <- liftIO $ R.getSymbolicLinkStatus f
+		let f' = fromRawFilePath f
+		if isSymbolicLink s
+			then pure $ Just (Left f', f')
+			else withTSDelta (\d -> liftIO $ toInodeCache d f' s)
+				>>= return . \case
+					Just i -> Just (Right (inodeCacheToKey Strongly i), f')
+					Nothing -> Nothing
 	void $ liftIO cleanup
 	return $ M.fromList $ catMaybes fsis
