@@ -1,10 +1,12 @@
 {- management of the git-annex branch
  -
- - Copyright 2011-2019 Joey Hess <id@joeyh.name>
+ - Copyright 2011-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Annex.Branch (
 	fullname,
 	name,
@@ -12,6 +14,7 @@ module Annex.Branch (
 	hasSibling,
 	siblingBranches,
 	create,
+	UpdateMade(..),
 	update,
 	forceUpdate,
 	updateTo,
@@ -29,7 +32,9 @@ module Annex.Branch (
 	withIndex,
 ) where
 
+import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as L
+import qualified Data.ByteString.Char8 as B8
 import qualified Data.Set as S
 import qualified Data.Map as M
 import Data.Function
@@ -38,6 +43,7 @@ import Data.ByteString.Builder
 import Control.Concurrent (threadDelay)
 
 import Annex.Common
+import Types.BranchState
 import Annex.BranchState
 import Annex.Journal
 import Annex.GitOverlay
@@ -55,7 +61,7 @@ import qualified Git.LsTree
 import Git.LsTree (lsTreeParams)
 import qualified Git.HashObject
 import Annex.HashObject
-import Git.Types (Ref(..), fromRef, RefDate, TreeItemType(..))
+import Git.Types (Ref(..), fromRef, fromRef', RefDate, TreeItemType(..))
 import Git.FilePath
 import Annex.CatFile
 import Annex.Perms
@@ -77,11 +83,11 @@ name = Git.Ref "git-annex"
 
 {- Fully qualified name of the branch. -}
 fullname :: Git.Ref
-fullname = Git.Ref $ "refs/heads/" ++ fromRef name
+fullname = Git.Ref $ "refs/heads/" <> fromRef' name
 
 {- Branch's name in origin. -}
 originname :: Git.Ref
-originname = Git.Ref $ "origin/" ++ fromRef name
+originname = Git.Ref $ "origin/" <> fromRef' name
 
 {- Does origin/git-annex exist? -}
 hasOrigin :: Annex Bool
@@ -119,13 +125,18 @@ getBranch = maybe (hasOrigin >>= go >>= use) return =<< branchsha
 
 {- Ensures that the branch and index are up-to-date; should be
  - called before data is read from it. Runs only once per git-annex run. -}
-update :: Annex ()
-update = runUpdateOnce $ void $ updateTo =<< siblingBranches
+update :: Annex BranchState
+update = runUpdateOnce $ journalClean <$$> updateTo =<< siblingBranches
 
 {- Forces an update even if one has already been run. -}
-forceUpdate :: Annex Bool
+forceUpdate :: Annex UpdateMade
 forceUpdate = updateTo =<< siblingBranches
 
+data UpdateMade = UpdateMade
+	{ refsWereMerged :: Bool
+	, journalClean :: Bool
+	}
+
 {- Merges the specified Refs into the index, if they have any changes not
  - already in it. The Branch names are only used in the commit message;
  - it's even possible that the provided Branches have not been updated to
@@ -144,13 +155,13 @@ forceUpdate = updateTo =<< siblingBranches
  -
  - Returns True if any refs were merged in, False otherwise.
  -}
-updateTo :: [(Git.Sha, Git.Branch)] -> Annex Bool
+updateTo :: [(Git.Sha, Git.Branch)] -> Annex UpdateMade
 updateTo pairs = ifM (annexMergeAnnexBranches <$> Annex.getGitConfig)
 	( updateTo' pairs
-	, return False
+	, return (UpdateMade False False)
 	)
 
-updateTo' :: [(Git.Sha, Git.Branch)] -> Annex Bool
+updateTo' :: [(Git.Sha, Git.Branch)] -> Annex UpdateMade
 updateTo' pairs = do
 	-- ensure branch exists, and get its current ref
 	branchref <- getBranch
@@ -162,20 +173,35 @@ updateTo' pairs = do
 		else do
 			mergedrefs <- getMergedRefs
 			filterM isnewer (excludeset mergedrefs unignoredrefs)
-	if null tomerge
+	journalclean <- if null tomerge
 		{- Even when no refs need to be merged, the index
 		 - may still be updated if the branch has gotten ahead 
-		 - of the index. -}
-		then do
-			whenM (needUpdateIndex branchref) $ lockJournal $ \jl -> do
+		 - of the index, or just if the journal is dirty. -}
+		then ifM (needUpdateIndex branchref)
+			( lockJournal $ \jl -> do
 				forceUpdateIndex jl branchref
 				{- When there are journalled changes
 				 - as well as the branch being updated,
 				 - a commit needs to be done. -}
 				when dirty $
-					go branchref True [] jl
-		else lockJournal $ go branchref dirty tomerge
-	return $ not $ null tomerge
+					go branchref dirty [] jl
+				return True
+			, if dirty
+				then ifM (annexAlwaysCommit <$> Annex.getGitConfig)
+					( do
+						lockJournal $ go branchref dirty []
+						return True
+					, return False
+					)
+				else return True
+			)
+		else do
+			lockJournal $ go branchref dirty tomerge
+			return True
+	return $ UpdateMade
+		{ refsWereMerged = not (null tomerge)
+		, journalClean = journalclean
+		}
   where
 	excludeset s = filter (\(r, _) -> S.notMember r s)
 	isnewer (r, _) = inRepo $ Git.Branch.changed fullname r
@@ -204,6 +230,7 @@ updateTo' pairs = do
 					else commitIndex jl branchref merge_desc commitrefs
 			)
 		addMergedRefs tomerge
+		invalidateCache
 	stagejournalwhen dirty jl a
 		| dirty = stageJournal jl a
 		| otherwise = withIndex a
@@ -216,9 +243,15 @@ updateTo' pairs = do
  -
  - Returns an empty string if the file doesn't exist yet. -}
 get :: RawFilePath -> Annex L.ByteString
-get file = do
-	update
-	getLocal file
+get file = getCache file >>= \case
+	Just content -> return content
+	Nothing -> do
+		st <- update
+		content <- if journalIgnorable st
+			then getRef fullname file
+			else getLocal file
+		setCache file content
+		return content
 
 {- Like get, but does not merge the branch, so the info returned may not
  - reflect changes in remotes.
@@ -270,7 +303,14 @@ maybeChange file f = lockJournal $ \jl -> do
 
 {- Records new content of a file into the journal -}
 set :: Journalable content => JournalLocked -> RawFilePath -> content -> Annex ()
-set = setJournalFile
+set jl f c = do
+	journalChanged
+	setJournalFile jl f c
+	-- Could cache the new content, but it would involve
+	-- evaluating a Journalable Builder twice, which is not very
+	-- efficient. Instead, assume that it's not common to need to read
+	-- a log file immediately after writing it.
+	invalidateCache
 
 {- Commit message used when making a commit of whatever data has changed
  - to the git-annex brach. -}
@@ -327,9 +367,9 @@ commitIndex' jl branchref message basemessage retrynum parents = do
   where
 	-- look for "parent ref" lines and return the refs
 	commitparents = map (Git.Ref . snd) . filter isparent .
-		map (toassoc . decodeBL) . L.split newline
+		map (toassoc . L.toStrict) . L.split newline
 	newline = fromIntegral (ord '\n')
-	toassoc = separate (== ' ')
+	toassoc = separate' (== (fromIntegral (ord ' ')))
 	isparent (k,_) = k == "parent"
 		
 	{- The race can be detected by checking the commit's
@@ -355,7 +395,7 @@ commitIndex' jl branchref message basemessage retrynum parents = do
  - that have not been committed yet. There may be duplicates in the list. -}
 files :: Annex [RawFilePath]
 files = do
-	update
+	_  <- update
 	-- ++ forces the content of the first list to be buffered in memory,
 	-- so use getJournalledFilesStale which should be much smaller most
 	-- of the time. branchFiles will stream as the list is consumed.
@@ -388,8 +428,8 @@ mergeIndex :: JournalLocked -> [Git.Ref] -> Annex ()
 mergeIndex jl branches = do
 	prepareModifyIndex jl
 	hashhandle <- hashObjectHandle
-	ch <- catFileHandle
-	inRepo $ \g -> Git.UnionMerge.mergeIndex hashhandle ch g branches
+	withCatFileHandle $ \ch ->
+		inRepo $ \g -> Git.UnionMerge.mergeIndex hashhandle ch g branches
 
 {- Removes any stale git lock file, to avoid git falling over when
  - updating the index.
@@ -410,14 +450,12 @@ prepareModifyIndex _jl = do
 withIndex :: Annex a -> Annex a
 withIndex = withIndex' False
 withIndex' :: Bool -> Annex a -> Annex a
-withIndex' bootstrapping a = do
-	f <- fromRepo gitAnnexIndex
-	withIndexFile f $ do
-		checkIndexOnce $ unlessM (liftIO $ doesFileExist f) $ do
-			unless bootstrapping create
-			createAnnexDirectory $ takeDirectory f
-			unless bootstrapping $ inRepo genIndex
-		a
+withIndex' bootstrapping a = withIndexFile AnnexIndexFile $ \f -> do
+	checkIndexOnce $ unlessM (liftIO $ doesFileExist f) $ do
+		unless bootstrapping create
+		createAnnexDirectory $ takeDirectory f
+		unless bootstrapping $ inRepo genIndex
+	a
 
 {- Updates the branch's index to reflect the current contents of the branch.
  - Any changes staged in the index will be preserved.
@@ -438,8 +476,8 @@ forceUpdateIndex jl branchref = do
 needUpdateIndex :: Git.Ref -> Annex Bool
 needUpdateIndex branchref = do
 	f <- fromRepo gitAnnexIndexStatus
-	committedref <- Git.Ref . firstLine <$>
-		liftIO (catchDefaultIO "" $ readFileStrict f)
+	committedref <- Git.Ref . firstLine' <$>
+		liftIO (catchDefaultIO mempty $ B.readFile f)
 	return (committedref /= branchref)
 
 {- Record that the branch's index has been updated to correspond to a
@@ -621,11 +659,12 @@ ignoreRefs rs = do
 		unlines $ map fromRef $ S.elems s
 
 getIgnoredRefs :: Annex (S.Set Git.Sha)
-getIgnoredRefs = S.fromList . mapMaybe Git.Sha.extractSha . lines <$> content
+getIgnoredRefs = 
+	S.fromList . mapMaybe Git.Sha.extractSha . B8.lines <$> content
   where
 	content = do
 		f <- fromRepo gitAnnexIgnoredRefs
-		liftIO $ catchDefaultIO "" $ readFile f
+		liftIO $ catchDefaultIO mempty $ B.readFile f
 
 addMergedRefs :: [(Git.Sha, Git.Branch)] -> Annex ()
 addMergedRefs [] = return ()
@@ -643,11 +682,11 @@ getMergedRefs = S.fromList . map fst <$> getMergedRefs'
 getMergedRefs' :: Annex [(Git.Sha, Git.Branch)]
 getMergedRefs' = do
 	f <- fromRepo gitAnnexMergedRefs
-	s <- liftIO $ catchDefaultIO "" $ readFile f
-	return $ map parse $ lines s
+	s <- liftIO $ catchDefaultIO mempty $ B.readFile f
+	return $ map parse $ B8.lines s
   where
 	parse l = 
-		let (s, b) = separate (== '\t') l
+		let (s, b) = separate' (== (fromIntegral (ord '\t'))) l
 		in (Ref s, Ref b)
 
 {- Grafts a treeish into the branch at the specified location,
