@@ -20,6 +20,7 @@ module Annex.Content (
 	getViaTmp,
 	getViaTmpFromDisk,
 	checkDiskSpaceToGet,
+	checkSecureHashes,
 	prepTmp,
 	withTmp,
 	checkDiskSpace,
@@ -74,6 +75,7 @@ import Git.FilePath
 import Annex.Perms
 import Annex.Link
 import Annex.LockPool
+import Annex.WorkerPool
 import Messages.Progress
 import Types.Remote (unVerified, Verification(..), RetrievalSecurityPolicy(..))
 import qualified Types.Remote
@@ -87,7 +89,6 @@ import Annex.InodeSentinal
 import Utility.InodeCache
 import Annex.Content.LowLevel
 import Annex.Content.PointerFile
-import Annex.Concurrent
 import Types.WorkerPool
 import qualified Utility.RawFilePath as R
 
@@ -193,13 +194,15 @@ contentLockFile key = Just <$> calcRepo (gitAnnexContentLock key)
  - rather than running the action.
  -}
 lockContentShared :: Key -> (VerifiedCopy -> Annex a) -> Annex a
-lockContentShared key a = lockContentUsing lock key $ ifM (inAnnex key)
-	( do
-		u <- getUUID
-		withVerifiedCopy LockedCopy u (return True) a
-	, giveup $ "failed to lock content: not present"
-	)
+lockContentShared key a = lockContentUsing lock key notpresent $
+	ifM (inAnnex key)
+		( do
+			u <- getUUID
+			withVerifiedCopy LockedCopy u (return True) a
+		, notpresent
+		)
   where
+	notpresent = giveup $ "failed to lock content: not present"
 #ifndef mingw32_HOST_OS
 	lock contentfile Nothing = tryLockShared Nothing contentfile
 	lock _ (Just lockfile) = posixLocker tryLockShared lockfile
@@ -211,9 +214,12 @@ lockContentShared key a = lockContentUsing lock key $ ifM (inAnnex key)
  - might remove it.
  -
  - If locking fails, throws an exception rather than running the action.
+ -
+ - But, if locking fails because the the content is not present, runs the
+ - fallback action instead.
  -}
-lockContentForRemoval :: Key -> (ContentRemovalLock -> Annex a) -> Annex a
-lockContentForRemoval key a = lockContentUsing lock key $ 
+lockContentForRemoval :: Key -> Annex a -> (ContentRemovalLock -> Annex a) -> Annex a
+lockContentForRemoval key fallback a = lockContentUsing lock key fallback $ 
 	a (ContentRemovalLock key)
   where
 #ifndef mingw32_HOST_OS
@@ -250,22 +256,31 @@ winLocker takelock _ (Just lockfile) = do
 winLocker _ _ Nothing = return Nothing
 #endif
 
-lockContentUsing :: ContentLocker -> Key -> Annex a -> Annex a
-lockContentUsing locker key a = do
+{- The fallback action is run if the ContentLocker throws an IO exception
+ - and the content is not present. It's not guaranteed to always run when
+ - the content is not present, because the content file is not always
+ - the file that is locked eg on Windows a different file is locked. -}
+lockContentUsing :: ContentLocker -> Key -> Annex a -> Annex a -> Annex a
+lockContentUsing locker key fallback a = do
 	contentfile <- fromRawFilePath <$> calcRepo (gitAnnexLocation key)
 	lockfile <- contentLockFile key
 	bracket
 		(lock contentfile lockfile)
-		(unlock lockfile)
-		(const a)
+		(either (const noop) (unlock lockfile))
+		go
   where
 	alreadylocked = giveup "content is locked"
 	failedtolock e = giveup $ "failed to lock content: " ++ show e
 
-	lock contentfile lockfile =
-		(maybe alreadylocked return 
-			=<< locker contentfile lockfile)
-		`catchIO` failedtolock
+	lock contentfile lockfile = tryIO $
+		maybe alreadylocked return 
+			=<< locker contentfile lockfile
+	
+	go (Right _) = a
+	go (Left e) = ifM (inAnnex key)
+		( failedtolock e
+		, fallback 
+		)
 
 #ifndef mingw32_HOST_OS
 	unlock mlockfile lck = do
@@ -334,12 +349,13 @@ getViaTmpFromDisk rsp v key action = checkallowed $ do
 	-- RetrievalSecurityPolicy would cause verification to always fail.
 	checkallowed a = case rsp of
 		RetrievalAllKeysSecure -> a
-		RetrievalVerifiableKeysSecure
-			| isVerifiable (fromKey keyVariety key) -> a
-			| otherwise -> ifM (annexAllowUnverifiedDownloads <$> Annex.getGitConfig)
+		RetrievalVerifiableKeysSecure -> ifM (Backend.isVerifiable key)
+			( a
+			, ifM (annexAllowUnverifiedDownloads <$> Annex.getGitConfig)
 				( a
 				, warnUnverifiableInsecure key >> return False
 				)
+			)
 
 {- Verifies that a file is the expected content of a key.
  -
@@ -358,12 +374,13 @@ getViaTmpFromDisk rsp v key action = checkallowed $ do
 verifyKeyContent :: RetrievalSecurityPolicy -> VerifyConfig -> Verification -> Key -> FilePath -> Annex Bool
 verifyKeyContent rsp v verification k f = case (rsp, verification) of
 	(_, Verified) -> return True
-	(RetrievalVerifiableKeysSecure, _)
-		| isVerifiable (fromKey keyVariety k) -> verify
-		| otherwise -> ifM (annexAllowUnverifiedDownloads <$> Annex.getGitConfig)
+	(RetrievalVerifiableKeysSecure, _) -> ifM (Backend.isVerifiable k)
+		( verify
+		, ifM (annexAllowUnverifiedDownloads <$> Annex.getGitConfig)
 			( verify
 			, warnUnverifiableInsecure k >> return False
 			)
+		)
 	(_, UnVerified) -> ifM (shouldVerify v)
 		( verify
 		, return True
@@ -376,9 +393,11 @@ verifyKeyContent rsp v verification k f = case (rsp, verification) of
 		Just size -> do
 			size' <- liftIO $ catchDefaultIO 0 $ getFileSize f
 			return (size' == size)
-	verifycontent = case Types.Backend.verifyKeyContent =<< Backend.maybeLookupBackendVariety (fromKey keyVariety k) of
+	verifycontent = Backend.maybeLookupBackendVariety (fromKey keyVariety k) >>= \case
 		Nothing -> return True
-		Just verifier -> verifier k f
+		Just b -> case Types.Backend.verifyKeyContent b of
+			Nothing -> return True
+			Just verifier -> verifier k f
 
 warnUnverifiableInsecure :: Key -> Annex ()
 warnUnverifiableInsecure k = warning $ unwords
@@ -473,7 +492,7 @@ withTmp key action = do
  - case. May also throw exceptions in some cases.
  -}
 moveAnnex :: Key -> FilePath -> Annex Bool
-moveAnnex key src = ifM (checkSecureHashes key)
+moveAnnex key src = ifM (checkSecureHashes' key)
 	( do
 		withObjectLoc key storeobject
 		return True
@@ -496,22 +515,28 @@ moveAnnex key src = ifM (checkSecureHashes key)
 		dest' = fromRawFilePath dest
 	alreadyhave = liftIO $ removeFile src
 
-checkSecureHashes :: Key -> Annex Bool
-checkSecureHashes key
-	| cryptographicallySecure (fromKey keyVariety key) = return True
-	| otherwise = ifM (annexSecureHashesOnly <$> Annex.getGitConfig)
-		( do
-			warning $ "annex.securehashesonly blocked adding " ++ decodeBS (formatKeyVariety (fromKey keyVariety key)) ++ " key to annex objects"
-			return False
-		, return True
+checkSecureHashes :: Key -> Annex (Maybe String)
+checkSecureHashes key = ifM (Backend.isCryptographicallySecure key)
+	( return Nothing
+	, ifM (annexSecureHashesOnly <$> Annex.getGitConfig)
+		( return $ Just $ "annex.securehashesonly blocked adding " ++ decodeBS (formatKeyVariety (fromKey keyVariety key)) ++ " key"
+		, return Nothing
 		)
+	)
+
+checkSecureHashes' :: Key -> Annex Bool
+checkSecureHashes' key = checkSecureHashes key >>= \case
+	Nothing -> return True
+	Just msg -> do
+		warning $ msg ++ "to annex objects"
+		return False
 
 data LinkAnnexResult = LinkAnnexOk | LinkAnnexFailed | LinkAnnexNoop
 
 {- Populates the annex object file by hard linking or copying a source
  - file to it. -}
 linkToAnnex :: Key -> FilePath -> Maybe InodeCache -> Annex LinkAnnexResult
-linkToAnnex key src srcic = ifM (checkSecureHashes key)
+linkToAnnex key src srcic = ifM (checkSecureHashes' key)
 	( do
 		dest <- fromRawFilePath <$> calcRepo (gitAnnexLocation key)
 		modifyContent dest $ linkAnnex To key src srcic dest Nothing
@@ -584,21 +609,19 @@ unlinkAnnex key = do
 {- Runs an action to transfer an object's content.
  -
  - In some cases, it's possible for the file to change as it's being sent.
- - If this happens, runs the rollback action and returns False. The
- - rollback action should remove the data that was transferred.
+ - If this happens, runs the rollback action and throws an exception.
+ - The rollback action should remove the data that was transferred.
  -}
-sendAnnex :: Key -> Annex () -> (FilePath -> Annex Bool) -> Annex Bool
+sendAnnex :: Key -> Annex () -> (FilePath -> Annex a) -> Annex a
 sendAnnex key rollback sendobject = go =<< prepSendAnnex key
   where
-	go Nothing = return False
 	go (Just (f, checksuccess)) = do
 		r <- sendobject f
-		ifM checksuccess
-			( return r
-			, do
-				rollback
-				return False
-			)
+		unlessM checksuccess $ do
+			rollback
+			giveup "content changed while it was being sent"
+		return r
+	go Nothing = giveup "content not available to send"
 
 {- Returns a file that contains an object's content,
  - and a check to run after the transfer is complete.
@@ -784,8 +807,15 @@ downloadUrl :: Key -> MeterUpdate -> [Url.URLString] -> FilePath -> Url.UrlOptio
 downloadUrl k p urls file uo = 
 	-- Poll the file to handle configurations where an external
 	-- download command is used.
-	meteredFile file (Just p) k $
-		anyM (\u -> Url.download p u file uo) urls
+	meteredFile file (Just p) k (go urls Nothing)
+  where
+  	-- Display only one error message, if all the urls fail to
+	-- download.
+	go [] (Just err) = warning err >> return False
+	go [] Nothing = return False
+	go (u:us) _ = Url.download' p u file uo >>= \case
+		Right () -> return True
+		Left err -> go us (Just err)
 
 {- Copies a key's content, when present, to a temp file.
  - This is used to speed up some rsyncs. -}
