@@ -1,6 +1,6 @@
 {- git-annex import from remotes
  -
- - Copyright 2019 Joey Hess <id@joeyh.name>
+ - Copyright 2019-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -12,7 +12,7 @@ module Annex.Import (
 	ImportCommitConfig(..),
 	buildImportCommit,
 	buildImportTrees,
-	downloadImport,
+	importKeys,
 	filterImportableContents,
 	makeImportMatcher,
 	listImportableContents,
@@ -34,6 +34,8 @@ import Annex.LockFile
 import Annex.Content
 import Annex.Export
 import Annex.RemoteTrackingBranch
+import Annex.HashObject
+import Annex.Transfer
 import Command
 import Backend
 import Types.Key
@@ -50,6 +52,7 @@ import Utility.Matcher (isEmpty)
 import qualified Database.Export as Export
 import qualified Database.ContentIdentifier as CIDDb
 import qualified Logs.ContentIdentifier as CIDLog
+import Backend.Utilities
 
 import Control.Concurrent.STM
 import qualified Data.Map.Strict as M
@@ -92,7 +95,7 @@ buildImportCommit
 	:: Remote
 	-> ImportTreeConfig
 	-> ImportCommitConfig
-	-> ImportableContents Key
+	-> ImportableContents (Either Sha Key)
 	-> Annex (Maybe Ref)
 buildImportCommit remote importtreeconfig importcommitconfig importable =
 	case importCommitTracking importcommitconfig of
@@ -122,7 +125,9 @@ buildImportCommit remote importtreeconfig importcommitconfig importable =
 			Nothing -> pure committedtree
 			Just dir -> 
 				let subtreeref = Ref $
-					fromRef committedtree ++ ":" ++ fromRawFilePath (getTopFilePath dir)
+					fromRef' committedtree 
+						<> ":"
+						<> getTopFilePath dir
 				in fromMaybe emptyTree
 					<$> inRepo (Git.Ref.tree subtreeref)
 		updateexportdb importedtree
@@ -243,7 +248,7 @@ buildImportCommit' remote importcommitconfig mtrackingcommit imported@(History t
 buildImportTrees
 	:: Ref
 	-> Maybe TopFilePath
-	-> ImportableContents Key
+	-> ImportableContents (Either Sha Key)
 	-> Annex (History Sha)
 buildImportTrees basetree msubdir importable = History
 	<$> (buildtree (importableContents importable) =<< Annex.gitRepo)
@@ -262,49 +267,71 @@ buildImportTrees basetree msubdir importable = History
 			Just subdir -> liftIO $ 
 				graftTree' importtree subdir basetree repo hdl
 	
-	mktreeitem (loc, k) = do
-		let lf = fromImportLocation loc
-		let treepath = asTopFilePath lf
-		let topf = asTopFilePath $
+	mktreeitem (loc, v) = case v of
+		Right k -> do
+			relf <- fromRepo $ fromTopFilePath topf
+			symlink <- calcRepo $ gitAnnexLink (fromRawFilePath relf) k
+			linksha <- hashSymlink symlink
+			return $ TreeItem treepath (fromTreeItemType TreeSymlink) linksha
+		Left sha -> 
+			return $ TreeItem treepath (fromTreeItemType TreeFile) sha
+	  where
+		lf = fromImportLocation loc
+		treepath = asTopFilePath lf
+		topf = asTopFilePath $
 			maybe lf (\sd -> getTopFilePath sd P.</> lf) msubdir
-		relf <- fromRepo $ fromTopFilePath topf
-		symlink <- calcRepo $ gitAnnexLink (fromRawFilePath relf) k
-		linksha <- hashSymlink symlink
-		return $ TreeItem treepath (fromTreeItemType TreeSymlink) linksha
 
-{- Downloads all new ContentIdentifiers as needed to generate Keys. 
+{- Downloads all new ContentIdentifiers, or when importcontent is False,
+ - generates Keys without downloading.
+ -
+ - Generates either a Key or a git Sha, depending on annex.largefiles.
+ - But when importcontent is False, it cannot match on annex.largefiles
+ - (or generate a git Sha), so always generates Keys.
+ -
  - Supports concurrency when enabled.
  -
- - If any download fails, the whole thing fails with Nothing, 
+ - If it fails on any file, the whole thing fails with Nothing, 
  - but it will resume where it left off.
+ -
+ - Note that, when a ContentIdentifier has been imported before,
+ - generates the same thing that was imported before, so annex.largefiles
+ - is not reapplied.
  -}
-downloadImport :: Remote -> ImportTreeConfig -> ImportableContents (ContentIdentifier, ByteSize) -> Annex (Maybe (ImportableContents Key))
-downloadImport remote importtreeconfig importablecontents = do
+importKeys
+	:: Remote
+	-> ImportTreeConfig
+	-> Bool
+	-> ImportableContents (ContentIdentifier, ByteSize)
+	-> Annex (Maybe (ImportableContents (Either Sha Key)))
+importKeys remote importtreeconfig importcontent importablecontents = do
+	when (not importcontent && isNothing (Remote.importKey ia)) $
+		giveup "This remote does not support importing without downloading content."
 	-- This map is used to remember content identifiers that
-	-- were just downloaded, before they have necessarily been
+	-- were just imported, before they have necessarily been
 	-- stored in the database. This way, if the same content
 	-- identifier appears multiple times in the
 	-- importablecontents (eg when it has a history), 
-	-- they will only be downloaded once.
+	-- they will only be imported once.
 	cidmap <- liftIO $ newTVarIO M.empty
 	-- When concurrency is enabled, this set is needed to
-	-- avoid two threads both downloading the same content identifier.
-	downloading <- liftIO $ newTVarIO S.empty
+	-- avoid two threads both importing the same content identifier.
+	importing <- liftIO $ newTVarIO S.empty
 	withExclusiveLock gitAnnexContentIdentifierLock $
 		bracket CIDDb.openDb CIDDb.closeDb $ \db -> do
 			CIDDb.needsUpdateFromLog db
 				>>= maybe noop (CIDDb.updateFromLog db)
-			go False cidmap downloading importablecontents db
+			go False cidmap importing importablecontents db
   where
-	go oldversion cidmap downloading (ImportableContents l h) db = do
+	go oldversion cidmap importing (ImportableContents l h) db = do
+		largematcher <- largeFilesMatcher
 		jobs <- forM l $ \i ->
-			startdownload cidmap downloading db i oldversion
+			startimport cidmap importing db i oldversion largematcher
 		l' <- liftIO $ forM jobs $
 			either pure (atomically . takeTMVar)
 		if any isNothing l'
 			then return Nothing
 			else do
-				h' <- mapM (\ic -> go True cidmap downloading ic db) h
+				h' <- mapM (\ic -> go True cidmap importing ic db) h
 				if any isNothing h'
 					then return Nothing
 					else return $ Just $
@@ -312,25 +339,35 @@ downloadImport remote importtreeconfig importablecontents = do
 							(catMaybes l')
 							(catMaybes h')
 	
-	waitstart downloading cid = liftIO $ atomically $ do
-		s <- readTVar downloading
+	waitstart importing cid = liftIO $ atomically $ do
+		s <- readTVar importing
 		if S.member cid s
 			then retry
-			else writeTVar downloading $ S.insert cid s
+			else writeTVar importing $ S.insert cid s
 	
-	signaldone downloading cid = liftIO $ atomically $ do
-		s <- readTVar downloading
-		writeTVar downloading $ S.delete cid s
+	signaldone importing cid = liftIO $ atomically $ do
+		s <- readTVar importing
+		writeTVar importing $ S.delete cid s
 	
-	startdownload cidmap downloading db i@(loc, (cid, _sz)) oldversion = getcidkey cidmap db cid >>= \case
-		(k:_) -> return $ Left $ Just (loc, k)
+	startimport cidmap importing db i@(loc, (cid, _sz)) oldversion largematcher = getcidkey cidmap db cid >>= \case
+		(k:ks) ->
+			-- If the same content was imported before
+			-- yeilding multiple different keys, it's not clear
+			-- which is best to use this time, so pick the
+			-- first in the list. But, if any of them is a
+			-- git sha, use it, because the content must
+			-- be included in the git repo then.
+			let v = case mapMaybe keyGitSha (k:ks) of
+				(sha:_) -> Left sha
+				[] -> Right k
+			in return $ Left $ Just (loc, v)
 		[] -> do
 			job <- liftIO $ newEmptyTMVarIO
 			let ai = ActionItemOther (Just (fromRawFilePath (fromImportLocation loc)))
-			let downloadaction = starting ("import " ++ Remote.name remote) ai $ do
+			let importaction = starting ("import " ++ Remote.name remote) ai $ do
 				when oldversion $
 					showNote "old version"
-				tryNonAsync (download cidmap db i) >>= \case
+				tryNonAsync (importordownload cidmap db i largematcher) >>= \case
 					Left e -> next $ do
 						warning (show e)
 						liftIO $ atomically $
@@ -341,44 +378,98 @@ downloadImport remote importtreeconfig importablecontents = do
 							putTMVar job r
 						return True
 			commandAction $ bracket_
-				(waitstart downloading cid)
-				(signaldone downloading cid)
-				downloadaction
+				(waitstart importing cid)
+				(signaldone importing cid)
+				importaction
 			return (Right job)
 	
-	download cidmap db (loc, (cid, sz)) = do
-		let rundownload tmpfile p = 
-			Remote.retrieveExportWithContentIdentifier ia loc cid tmpfile (mkkey loc tmpfile) p >>= \case
-				Just k -> tryNonAsync (moveAnnex k tmpfile) >>= \case
-					Right True -> do
+	importordownload
+		| not importcontent = doimport
+		| otherwise = dodownload
+
+	doimport cidmap db (loc, (cid, sz)) _largematcher =
+		case Remote.importKey ia of
+			Nothing -> error "internal" -- checked earlier
+			Just a -> do
+				let importer p = do
+					unsizedk <- a loc cid p
+					-- This avoids every remote needing
+					-- to add the size.
+					let k = alterKey unsizedk $ \kd -> kd
+						{ keySize = keySize kd <|> Just sz }
+					checkSecureHashes k >>= \case
+						Nothing -> do
+							recordcidkey cidmap db cid k
+							logChange k (Remote.uuid remote) InfoPresent
+							return (Right k)
+						Just msg -> giveup (msg ++ " to import")
+				let runimport p = tryNonAsync (importer p) >>= \case
+					Right k -> return $ Just (loc, k)
+					Left e -> do
+						warning (show e)
+						return Nothing
+				metered Nothing	sz $
+					const runimport
+	
+	dodownload cidmap db (loc, (cid, sz)) largematcher = do
+		f <- locworktreefile loc
+		let af = AssociatedFile (Just f)
+		let downloader tmpfile p = do
+			k <- Remote.retrieveExportWithContentIdentifier
+				ia loc cid tmpfile 
+				(mkkey f tmpfile largematcher)
+				p
+			case keyGitSha k of
+				Nothing -> do
+					ok <- moveAnnex k tmpfile
+					when ok $ do
 						recordcidkey cidmap db cid k
 						logStatus k InfoPresent
 						logChange k (Remote.uuid remote) InfoPresent
-						return $ Just (loc, k)
-					_ -> return Nothing
-				Nothing -> return Nothing
+					return (Right k, ok)
+				Just sha -> do
+					recordcidkey cidmap db cid k
+					return (Left sha, True)
+		let rundownload tmpfile p = tryNonAsync (downloader tmpfile p) >>= \case
+			Right (v, True) -> return $ Just (loc, v)
+			Right (_, False) -> return Nothing
+			Left e -> do
+				warning (show e)
+				return Nothing
 		checkDiskSpaceToGet tmpkey Nothing $
-			withTmp tmpkey $ \tmpfile ->
-				metered Nothing tmpkey $
-					const (rundownload tmpfile)
+			notifyTransfer Download af $
+				download (Remote.uuid remote) tmpkey af stdRetry $ \p ->
+					withTmp tmpkey $ \tmpfile ->
+						metered (Just p) tmpkey $
+							const (rundownload tmpfile)
 	  where
-		ia = Remote.importActions remote
 		tmpkey = importKey cid sz
 	
-	mkkey loc tmpfile = do
-		f <- fromRepo $ fromTopFilePath $ locworktreefilename loc
-		backend <- chooseBackend (fromRawFilePath f)
-		let ks = KeySource
-			{ keyFilename = f
-			, contentLocation = toRawFilePath tmpfile
-			, inodeCache = Nothing
+	ia = Remote.importActions remote
+	
+	mkkey f tmpfile largematcher = do
+		matcher <- largematcher (fromRawFilePath f)
+		let mi = MatchingFile FileInfo
+			{ matchFile = f
+			, currFile = toRawFilePath tmpfile
 			}
-		fmap fst <$> genKey ks nullMeterUpdate backend
+		islargefile <- checkMatcher' matcher mi mempty
+		if islargefile
+			then do
+				backend <- chooseBackend (fromRawFilePath f)
+				let ks = KeySource
+					{ keyFilename = f
+					, contentLocation = toRawFilePath tmpfile
+					, inodeCache = Nothing
+					}
+				fst <$> genKey ks nullMeterUpdate backend
+			else gitShaKey <$> hashFile tmpfile
 
-	locworktreefilename loc = asTopFilePath $ case importtreeconfig of
-		ImportTree -> fromImportLocation loc
-		ImportSubTree subdir _ ->
-			getTopFilePath subdir P.</> fromImportLocation loc
+	locworktreefile loc = fromRepo $ fromTopFilePath $ asTopFilePath $
+		case importtreeconfig of
+			ImportTree -> fromImportLocation loc
+			ImportSubTree subdir _ ->
+				getTopFilePath subdir P.</> fromImportLocation loc
 
 	getcidkey cidmap db cid = liftIO $
 		CIDDb.getContentIdentifierKeys db rs cid >>= \case
@@ -398,7 +489,7 @@ downloadImport remote importtreeconfig importablecontents = do
  - content, before generating its real key. -}
 importKey :: ContentIdentifier -> Integer -> Key
 importKey (ContentIdentifier cid) size = mkKey $ \k -> k
-	{ keyName = cid
+	{ keyName = genKeyName (decodeBS cid)
 	, keyVariety = OtherKey "CID"
 	, keySize = Just size
 	}
