@@ -30,6 +30,7 @@ import Git.FilePath
 import Git.Config
 import Annex.HashObject
 import Annex.InodeSentinal
+import Annex.PidLock
 import Utility.FileMode
 import Utility.InodeCache
 import Utility.Tmp.Dir
@@ -176,12 +177,18 @@ newtype Restage = Restage Bool
 restagePointerFile :: Restage -> RawFilePath -> InodeCache -> Annex ()
 restagePointerFile (Restage False) f _ =
 	toplevelWarning True $ unableToRestage $ Just $ fromRawFilePath f
-restagePointerFile (Restage True) f orig = withTSDelta $ \tsd -> do
-	-- update-index is documented as picky about "./file" and it
-	-- fails on "../../repo/path/file" when cwd is not in the repo 
-	-- being acted on. Avoid these problems with an absolute path.
-	absf <- liftIO $ absPath $ fromRawFilePath f
-	Annex.Queue.addInternalAction runner [(absf, isunmodified tsd)]
+restagePointerFile (Restage True) f orig = withTSDelta $ \tsd ->
+	-- Avoid refreshing the index if run by the
+	-- smudge clean filter, because git uses that when
+	-- it's already refreshing the index, probably because
+	-- this very action is running. Running it again would likely
+	-- deadlock.
+	unlessM (Annex.getState Annex.insmudgecleanfilter) $ do
+		-- update-index is documented as picky about "./file" and it
+		-- fails on "../../repo/path/file" when cwd is not in the repo 
+		-- being acted on. Avoid these problems with an absolute path.
+		absf <- liftIO $ absPath $ fromRawFilePath f
+		Annex.Queue.addInternalAction runner [(absf, isunmodified tsd)]
   where
 	isunmodified tsd = genInodeCache f tsd >>= return . \case
 		Nothing -> False
@@ -204,23 +211,24 @@ restagePointerFile (Restage True) f orig = withTSDelta $ \tsd -> do
 		    go (Just _) = withTmpDirIn (fromRawFilePath $ Git.localGitDir r) "annexindex" $ \tmpdir -> do
 			let tmpindex = tmpdir </> "index"
 			let updatetmpindex = do
-				r' <- Git.Env.addGitEnv r Git.Index.indexEnv 
+				r' <- liftIO $ Git.Env.addGitEnv r Git.Index.indexEnv 
 					=<< Git.Index.indexEnvVal tmpindex
 				-- Avoid git warning about CRLF munging.
 				let r'' = r' { gitGlobalOpts = gitGlobalOpts r' ++
 					[ Param "-c"
 					, Param $ "core.safecrlf=" ++ boolConfig False
 					] }
-				Git.UpdateIndex.refreshIndex r'' $ \feed ->
-					forM_ l $ \(f', checkunmodified) ->
-						whenM checkunmodified $
-							feed f'
+				runsGitAnnexChildProcessViaGit' r'' $ \r''' ->
+					liftIO $ Git.UpdateIndex.refreshIndex r''' $ \feed ->
+						forM_ l $ \(f', checkunmodified) ->
+							whenM checkunmodified $
+								feed f'
 			let replaceindex = catchBoolIO $ do
 				moveFile tmpindex realindex
 				return True
-			ok <- liftIO $ createLinkOrCopy realindex tmpindex
+			ok <- liftIO (createLinkOrCopy realindex tmpindex)
 				<&&> updatetmpindex
-				<&&> replaceindex
+				<&&> liftIO replaceindex
 			unless ok showwarning
 		bracket lockindex unlockindex go
 
