@@ -14,7 +14,6 @@ module Annex.SpecialRemote (
 
 import Annex.Common
 import Annex.SpecialRemote.Config
-import Remote (remoteTypes)
 import Types.Remote (RemoteConfig, SetupStage(..), typename, setup)
 import Types.GitConfig
 import Types.ProposedAccepted
@@ -22,7 +21,6 @@ import Config
 import Remote.List
 import Logs.Remote
 import Logs.Trust
-import qualified Git.Config
 import qualified Types.Remote as Remote
 import Git.Types (RemoteName)
 
@@ -99,7 +97,7 @@ autoEnable = do
 			_ -> return ()
   where
 	configured rc = fromMaybe False $
-		Git.Config.isTrueFalse . fromProposedAccepted
+		trueFalseParser' . fromProposedAccepted
 			=<< M.lookup autoEnableField rc
 	canenable u = (/= DeadTrusted) <$> lookupTrust u
 	getenabledremotes = M.fromList
