@@ -293,7 +293,7 @@ prop_view_roundtrips f metadata visible = or
 	, all hasfields (viewedFiles view viewedFileFromReference f metadata)
 	]
   where
-	view = View (Git.Ref "master") $
+	view = View (Git.Ref "foo") $
 		map (\(mf, mv) -> ViewComponent mf (FilterValues $ S.filter (not . B.null . fromMetaValue) mv) visible)
 			(fromMetaData metadata)
 	visiblefields = sort (map viewField $ filter viewVisible (viewComponents view))
@@ -345,16 +345,15 @@ narrowView = applyView' viewedFileReuse getViewedFileMetaData
 applyView' :: MkViewedFile -> (FilePath -> MetaData) -> View -> Annex Git.Branch
 applyView' mkviewedfile getfilemetadata view = do
 	top <- fromRepo Git.repoPath
-	(l, clean) <- inRepo $ Git.LsFiles.stagedDetails [top]
+	(l, clean) <- inRepo $ Git.LsFiles.inRepoDetails [] [top]
 	liftIO . nukeFile =<< fromRepo gitAnnexViewIndex
-	uh <- withViewIndex $ inRepo Git.UpdateIndex.startUpdateIndex
-	forM_ l $ \(f, sha, mode) -> do
-		topf <- inRepo (toTopFilePath f)
-		go uh topf sha (toTreeItemType =<< mode) =<< lookupFile f
-	liftIO $ do
-		void $ stopUpdateIndex uh
-		void clean
-	genViewBranch view
+	viewg <- withViewIndex gitRepo
+	withUpdateIndex viewg $ \uh -> do
+		forM_ l $ \(f, sha, mode) -> do
+			topf <- inRepo (toTopFilePath f)
+			go uh topf sha (toTreeItemType mode) =<< lookupKey f
+		liftIO $ void clean
+		genViewBranch view
   where
 	genviewedfiles = viewedFiles view mkviewedfile -- enables memoization
 
@@ -366,7 +365,7 @@ applyView' mkviewedfile getfilemetadata view = do
 			f' <- fromRawFilePath <$> 
 				fromRepo (fromTopFilePath $ asTopFilePath $ toRawFilePath fv)
 			stagesymlink uh f' =<< calcRepo (gitAnnexLink f' k)
-	go uh topf (Just sha) (Just treeitemtype) Nothing
+	go uh topf sha (Just treeitemtype) Nothing
 		| "." `B.isPrefixOf` getTopFilePath topf =
 			liftIO $ Git.UpdateIndex.streamUpdateIndex' uh $
 				pureStreamer $ updateIndexLine sha treeitemtype topf
@@ -412,9 +411,7 @@ withViewChanges addmeta removemeta = do
  - Note that the file does not necessarily exist, or can contain
  - info staged for an old view. -}
 withViewIndex :: Annex a -> Annex a
-withViewIndex a = do
-	f <- fromRepo gitAnnexViewIndex
-	withIndexFile f a
+withViewIndex = withIndexFile ViewIndexFile . const
 
 {- Generates a branch for a view, using the view index file
  - to make a commit to the view branch. The view branch is not
