@@ -65,7 +65,8 @@ onChange file
 	| ".lock" `isSuffixOf` file = noop
 	| isAnnexBranch file = do
 		branchChanged
-		diverged <- liftAnnex Annex.Branch.forceUpdate
+		diverged <- Annex.Branch.refsWereMerged
+			<$> liftAnnex Annex.Branch.forceUpdate
 		when diverged $ do
 			updateExportTreeFromLogAll
 			queueDeferredDownloads "retrying deferred download" Later
@@ -109,6 +110,6 @@ isAnnexBranch f = n `isSuffixOf` f
 	n = '/' : Git.fromRef Annex.Branch.name
 
 fileToBranch :: FilePath -> Git.Ref
-fileToBranch f = Git.Ref $ "refs" </> base
+fileToBranch f = Git.Ref $ encodeBS' $ "refs" </> base
   where
 	base = Prelude.last $ split "/refs/" f
