@@ -5,7 +5,7 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE CPP, QuasiQuotes, TemplateHaskell, OverloadedStrings #-}
+{-# LANGUAGE QuasiQuotes, TemplateHaskell, OverloadedStrings #-}
 {-# LANGUAGE FlexibleContexts #-}
 
 module Assistant.WebApp.Configurators.Edit where
@@ -19,13 +19,11 @@ import Assistant.ScanRemotes
 import Assistant.Sync
 import Assistant.Alert
 import qualified Assistant.WebApp.Configurators.AWS as AWS
-#ifdef WITH_S3
 import qualified Assistant.WebApp.Configurators.IA as IA
 import qualified Remote.S3 as S3
-#endif
 import qualified Remote
 import qualified Types.Remote as Remote
-import qualified Remote.List as Remote
+import Remote.List.Util
 import Logs.UUID
 import Logs.Group
 import Logs.PreferredContent
@@ -116,7 +114,7 @@ setRepoConfig uuid mremote oldc newc = do
 				, Param $ T.unpack $ repoName oldc
 				, Param name
 				]
-			void Remote.remoteListRefresh
+			remotesChanged
 		liftAssistant updateSyncRemotes
 	when associatedDirectoryChanged $ case repoAssociatedDirectory newc of
 		Nothing -> noop
@@ -256,14 +254,10 @@ checkAssociatedDirectory cfg (Just r) = do
 getRepoInfo :: Maybe Remote.Remote -> Remote.RemoteConfig -> Widget
 getRepoInfo (Just r) c = case fromProposedAccepted <$> M.lookup typeField c of
 	Just "S3" -> do
-#ifdef WITH_S3
 		pc <- liftAnnex $ parsedRemoteConfig S3.remote c
 		if S3.configIA pc
 			then IA.getRepoInfo c
 			else AWS.getRepoInfo c
-#else
-		AWS.getRepoInfo c
-#endif
 	Just t
 		| t /= "git" -> [whamlet|#{t} remote|]
 	_ -> getGitRepoInfo =<< liftAnnex (Remote.getRepo r)
@@ -309,7 +303,7 @@ getUpgradeRepositoryR r = go =<< liftAnnex (repoIdRemote r)
 			setConfig
 				(remoteAnnexConfig repo "ignore")
 				(Git.Config.boolConfig False)
-		liftAnnex $ void Remote.remoteListRefresh
+		liftAnnex remotesChanged
 		liftAssistant updateSyncRemotes
 		liftAssistant $ syncRemote rmt
 		redirect DashboardR
