@@ -5,7 +5,7 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE QuasiQuotes, TemplateHaskell, OverloadedStrings, CPP #-}
+{-# LANGUAGE QuasiQuotes, TemplateHaskell, OverloadedStrings #-}
 
 module Assistant.WebApp.RepoList where
 
@@ -14,7 +14,7 @@ import Assistant.DaemonStatus
 import Assistant.WebApp.Notifications
 import qualified Remote
 import qualified Types.Remote as Remote
-import Remote.List (remoteListRefresh)
+import Remote.List.Util
 import Annex.UUID (getUUID)
 import Logs.Remote
 import Logs.Trust
@@ -179,13 +179,9 @@ repoList reposelector
 	findinfo m g u = case fromProposedAccepted <$> getconfig (Accepted "type") of
 		Just "rsync" -> val True EnableRsyncR
 		Just "directory" -> val False EnableDirectoryR
-#ifdef WITH_S3
 		Just "S3" -> val True EnableS3R
-#endif
 		Just "glacier" -> val True EnableGlacierR
-#ifdef WITH_WEBDAV
 		Just "webdav" -> val True EnableWebDAVR
-#endif
 		Just "gcrypt" ->
 			-- Skip gcrypt repos on removable drives;
 			-- handled separately.
@@ -242,7 +238,7 @@ getRepositoriesReorderR = do
 			when (Remote.cost r /= newcost) $ do
 				repo <- Remote.getRepo r
 				setRemoteCost repo newcost
-		void remoteListRefresh
+		remotesChanged
 	fromjs = fromMaybe (RepoUUID NoUUID) . readish . T.unpack
 
 reorderCosts :: Remote -> [Remote] -> [(Remote, Cost)]
