@@ -66,10 +66,12 @@ main = do
 	changeWorkingDirectory repodir
 	updated <- catMaybes <$> mapM (getbuild repodir) autobuilds
 	state <- Annex.new =<< Git.Construct.fromPath "."
-	Annex.eval state $ do
+	ood <- Annex.eval state $ do
 		buildrpms topdir updated
 		makeinfos updated version
 	syncToArchiveOrg
+	unless (null ood) $
+		error $ "Some info files are out of date: " ++ show (map fst ood)
 
 -- Download a build from the autobuilder, virus check it, and return its
 -- version.
@@ -113,7 +115,7 @@ getbuild repodir (url, f) = do
 		return $ if null bv || any (not . versionchar) bv then Nothing else Just bv
 	versionchar c = isAlphaNum c || c == '.' || c == '-'
 
-makeinfos :: [(FilePath, Version)] -> Version -> Annex ()
+makeinfos :: [(FilePath, Version)] -> Version -> Annex [([Char], Maybe GitAnnexDistribution)]
 makeinfos updated version = do
 	mapM_ (\f -> inRepo $ runBool [Param "annex", Param "add", File f]) (map fst updated)
 	void $ inRepo $ runBool 
@@ -126,7 +128,7 @@ makeinfos updated version = do
 	now <- liftIO getCurrentTime
 	liftIO $ putStrLn $ "building info files"
 	forM_ updated $ \(f, bv) -> do
-		v <- lookupFile (toRawFilePath f)
+		v <- lookupKey (toRawFilePath f)
 		case v of
 			Nothing -> noop
 			Just k -> whenM (inAnnex k) $ do
@@ -167,8 +169,7 @@ makeinfos updated version = do
 	ds <- liftIO $ forM infos (readish <$$> readFile)
 	let dis = zip infos ds
 	let ood = filter outofdate dis
-	unless (null ood) $
-		error $ "Some info files are out of date: " ++ show (map fst ood)
+	return ood
   where
 	outofdate (_, md) = case md of
 		Nothing -> True
@@ -227,7 +228,7 @@ buildrpms topdir l = do
 		<$> liftIO (getDirectoryContents rpmrepo)
 	forM_ tarrpmarches $ \(tararch, rpmarch) ->
 		forM_ (filter (isstandalonetarball tararch . fst) l) $ \(tarball, v) -> do
-			liftIO $ mapM_ nukeFile (filter ((tararch ++ ".rpm") `isSuffixOf`) oldrpms)
+			liftIO $ mapM_ nukeFile (filter ((rpmarch ++ ".rpm") `isSuffixOf`) oldrpms)
 			void $ liftIO $ boolSystem script 
 				[ Param rpmarch
 				, File tarball
