@@ -5,9 +5,8 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-module Main where
+module Build.LinuxMkLibs (mklibs) where
 
-import System.Environment
 import Data.Maybe
 import System.FilePath
 import Control.Monad
@@ -25,29 +24,64 @@ import Utility.Path
 import Utility.FileMode
 import Utility.CopyFile
 
-main :: IO ()
-main = getArgs >>= go
-  where
-	go [] = error "specify LINUXSTANDALONE_DIST"
-	go (top:_) = mklibs top
-
-mklibs :: FilePath -> IO ()
-mklibs top = do
+mklibs :: FilePath -> a -> IO Bool
+mklibs top _installedbins = do
 	fs <- dirContentsRecursive top
 	exes <- filterM checkExe fs
 	libs <- parseLdd <$> readProcess "ldd" exes
+	
 	glibclibs <- glibcLibs
 	let libs' = nub $ libs ++ glibclibs
-	libdirs <- nub . catMaybes <$> mapM (installLib installFile top) libs'
+	let (linkers, otherlibs) = partition ("ld-linux" `isInfixOf`) libs'
+	libdirs <- nub . catMaybes <$> mapM (installLib installFile top) otherlibs
+	libdirs' <- consolidateUsrLib top libdirs
+
+	gconvlibs <- gconvLibs
+	mapM_ (installLib installFile top) gconvlibs
 
 	-- Various files used by runshell to set up env vars used by the
 	-- linker shims.
-	writeFile (top </> "libdirs") (unlines libdirs)
-	writeFile (top </> "gconvdir")
-		(parentDir $ Prelude.head $ filter ("/gconv/" `isInfixOf`) glibclibs)
+	writeFile (top </> "libdirs") (unlines libdirs')
+	writeFile (top </> "gconvdir") (parentDir $ Prelude.head gconvlibs)
 	
-	let linker = Prelude.head $ filter ("ld-linux" `isInfixOf`) libs'
+	mapM_ (installLib installFile top) linkers
+	let linker = Prelude.head linkers
 	mapM_ (installLinkerShim top linker) exes
+	
+	return (any hwcaplibdir libdirs)
+  where
+	-- hwcap lib dirs are things like foo/tls and foo/x86.
+	-- Hard to know if a directory is, so this is a heuristic
+	-- looking for things that are certianly not. If this heuristic
+	-- fails, a minor optimisation will not happen, but there will be
+	-- no bad results.
+	hwcaplibdir d = not $ or
+		[ "lib" == takeFileName d
+		-- eg, "lib/x86_64-linux-gnu"
+		, "-linux-" `isInfixOf` takeFileName d
+		]
+
+{- If there are two libdirs that are the same except one is in
+ - usr/lib and the other is in lib/, move the contents of the usr/lib one
+ - into the lib/ one. This reduces the number of directories the linker
+ - needs to look in, and so reduces the number of failed stats
+ - and improves startup time.
+ -}
+consolidateUsrLib :: FilePath -> [FilePath] -> IO [FilePath]
+consolidateUsrLib top libdirs = go [] libdirs
+  where
+	go c [] = return c
+	go c (x:rest) = case filter (\d -> ("/usr" ++ d) == x) libdirs of
+		(d:_) -> do
+			fs <- getDirectoryContents (inTop top x)
+			forM_ fs $ \f -> do
+				let src = inTop top (x </> f)
+				let dst = inTop top (d </> f)
+				unless (dirCruft f) $
+					unlessM (doesDirectoryExist src) $
+						renameFile src dst
+			go c rest
+		_ -> go (x:c) rest
 
 {- Installs a linker shim script around a binary.
  -
