@@ -1,6 +1,6 @@
 {- git-annex command-line actions and concurrency
  -
- - Copyright 2010-2019 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -17,7 +17,6 @@ import Types.Concurrency
 import Messages.Concurrent
 import Types.Messages
 import Types.WorkerPool
-import Remote.List
 
 import Control.Concurrent
 import Control.Concurrent.Async
@@ -137,6 +136,15 @@ finishCommandActions = Annex.getState Annex.workers >>= \case
 				else retry
 		mapM_ mergeState sts
 
+{- Waits for all worker threads that have been started so far to finish. -}
+waitForAllRunningCommandActions :: Annex ()
+waitForAllRunningCommandActions = Annex.getState Annex.workers >>= \case
+	Nothing -> noop
+	Just tv -> liftIO $ atomically $ do
+		pool <- readTMVar tv
+		unless (allIdle pool)
+			retry
+
 {- Like commandAction, but without the concurrency. -}
 includeCommandAction :: CommandStart -> CommandCleanup
 includeCommandAction start =
@@ -200,8 +208,7 @@ startConcurrency :: UsedStages -> Annex a -> Annex a
 startConcurrency usedstages a = do
 	fromcmdline <- Annex.getState Annex.concurrency
 	fromgitcfg <- annexJobs <$> Annex.getGitConfig
-	let usegitcfg = Annex.changeState $ 
-		\c -> c { Annex.concurrency = fromgitcfg }
+	let usegitcfg = setConcurrency fromgitcfg
 	case (fromcmdline, fromgitcfg) of
 		(NonConcurrent, NonConcurrent) -> a
 		(Concurrent n, _) ->
@@ -245,11 +252,6 @@ startConcurrency usedstages a = do
 			liftIO $ setNumCapabilities n
 	
 	initworkerpool n = do
-		-- Generate the remote list now, to avoid each thread
-		-- generating it, which would be more expensive and
-		-- could cause threads to contend over eg, calls to
-		-- setConfig.
-		_ <- remoteList
 		tv <- liftIO newEmptyTMVarIO
 		Annex.changeState $ \s -> s { Annex.workers = Just tv }
 		st <- dupState
