@@ -13,7 +13,7 @@ import Command
 import Logs.Config
 import Config
 import Types.GitConfig (globalConfigs)
-import Git.Types (ConfigKey(..), fromConfigValue)
+import Git.Types (fromConfigValue)
 
 import qualified Data.ByteString.Char8 as S8
 
@@ -69,8 +69,9 @@ seek (UnsetConfig ck@(ConfigKey name)) = checkIsGlobalConfig ck $ commandAction
 seek (GetConfig ck) = checkIsGlobalConfig ck $ commandAction $
 	startingCustomOutput (ActionItemOther Nothing) $ do
 		getGlobalConfig ck >>= \case
-			Nothing -> return ()
 			Just (ConfigValue v) -> liftIO $ S8.putStrLn v
+			Just NoConfigValue -> return ()
+			Nothing -> return ()
 		next $ return True
 
 checkIsGlobalConfig :: ConfigKey -> Annex a -> Annex a
