@@ -45,14 +45,24 @@ instance DeferredParseClass CopyOptions where
 
 seek :: CopyOptions -> CommandSeek
 seek o = startConcurrency commandStages $ do
-	let go = whenAnnexed $ start o
 	case batchOption o of
-		Batch fmt -> batchFilesMatching fmt (go . toRawFilePath)
 		NoBatch -> withKeyOptions
-			(keyOptions o) (autoMode o)
+			(keyOptions o) (autoMode o) seeker
 			(commandAction . Command.Move.startKey (fromToOptions o) Command.Move.RemoveNever)
-			(withFilesInGit $ commandAction . go)
-			=<< workTreeItems (copyFiles o)
+			(withFilesInGitAnnex ww seeker)
+			=<< workTreeItems ww (copyFiles o)
+		Batch fmt -> batchAnnexedFilesMatching fmt seeker
+  where
+	ww = WarnUnmatchLsFiles
+	
+	seeker = AnnexedFileSeeker
+		{ startAction = start o
+		, checkContentPresent = case fromToOptions o of
+			Right (FromRemote _) -> Just False
+			Right (ToRemote _) -> Just True
+			Left ToHere -> Just False
+		, usesLocationLog = True
+		}
 
 {- A copy is just a move that does not delete the source file.
  - However, auto mode avoids unnecessary copies, and avoids getting or
