@@ -35,7 +35,8 @@ seek o = do
 		giveup "dropkey can cause data loss; use --force if you're sure you want to do this"
 	withKeys (commandAction . start) (toDrop o)
 	case batchOption o of
-		Batch fmt -> batchInput fmt parsekey $ batchCommandAction . start
+		Batch fmt -> batchInput fmt (pure . parsekey) $
+			batchCommandAction . start
 		NoBatch -> noop
   where
 	parsekey = maybe (Left "bad key") Right . deserializeKey
@@ -46,7 +47,7 @@ start key = starting "dropkey" (mkActionItem key) $
 
 perform :: Key -> CommandPerform
 perform key = ifM (inAnnex key)
-	( lockContentForRemoval key $ \contentlock -> do
+	( lockContentForRemoval key (next $ cleanup key) $ \contentlock -> do
 		removeAnnex contentlock
 		next $ cleanup key
 	, next $ return True
