@@ -20,6 +20,7 @@ import qualified Git.Ref
 import Git.Types
 import Git.FilePath
 import Git.Sha
+import qualified Remote
 import Types.Remote
 import Types.Export
 import Annex.Export
@@ -37,14 +38,16 @@ import Utility.Tmp
 import Utility.Metered
 import Utility.Matcher
 
+import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Map as M
 import Control.Concurrent
 
 cmd :: Command
-cmd = command "export" SectionCommon
-	"export content to a remote"
-	paramTreeish (seek <$$> optParser)
+cmd = withGlobalOptions [jobsOption, jsonOptions, jsonProgressOption] $
+	command "export" SectionCommon
+		"export content to a remote"
+		paramTreeish (seek <$$> optParser)
 
 data ExportOptions = ExportOptions
 	{ exportTreeish :: Git.Ref
@@ -74,7 +77,7 @@ exportTempName ek = mkExportLocation $ toRawFilePath $
 	".git-annex-tmp-content-" ++ serializeKey (asKey (ek))
 
 seek :: ExportOptions -> CommandSeek
-seek o = do
+seek o = startConcurrency commandStages $ do
 	r <- getParsed (exportRemote o)
 	unlessM (isExportSupported r) $
 		giveup "That remote does not support exports."
@@ -112,7 +115,7 @@ getExportCommit r treeish
 		return (fmap (tb, ) commitsha)
 	| otherwise = return Nothing
   where
-	baseref = Ref $ takeWhile (/= ':') $ fromRef $ 
+	baseref = Ref $ S8.takeWhile (/= ':') $ fromRef' $ 
 		Git.Ref.removeBase refsheads treeish
 	refsheads = "refs/heads"
 
@@ -133,6 +136,7 @@ changeExport r db (PreferredFiltered new) = do
 			(Git.DiffTree.file diff)
 	forM_ (incompleteExportedTreeishes old) $ \incomplete ->
 		mapdiff recover incomplete new
+	waitForAllRunningCommandActions
 
 	-- Diff the old and new trees, and delete or rename to new name all
 	-- changed files in the export. After this, every file that remains
@@ -155,12 +159,14 @@ changeExport r db (PreferredFiltered new) = do
 					(Just oldf, Nothing) ->
 						startUnexport' r db oldf ek
 					_ -> stop
+			waitForAllRunningCommandActions
 			-- Rename from temp to new files.
 			seekdiffmap $ \(ek, (moldf, mnewf)) ->
 				case (moldf, mnewf) of
 					(Just _oldf, Just newf) ->
 						startMoveFromTempName r db ek newf
 					_ -> stop
+			waitForAllRunningCommandActions
 		ts -> do
 			warning "Resolving export conflict.."
 			forM_ ts $ \oldtreesha -> do
@@ -178,6 +184,7 @@ changeExport r db (PreferredFiltered new) = do
 					(\diff -> commandAction $ startUnexport r db (Git.DiffTree.file diff) (unexportboth diff))
 					oldtreesha new
 			updateExportTree db emptyTree new
+			waitForAllRunningCommandActions
 	liftIO $ recordExportTreeCurrent db new
 
 	-- Waiting until now to record the export guarantees that,
@@ -193,7 +200,7 @@ changeExport r db (PreferredFiltered new) = do
 	mapdiff a oldtreesha newtreesha = do
 		(diff, cleanup) <- inRepo $
 			Git.DiffTree.diffTreeRecursive oldtreesha newtreesha
-		seekActions $ pure $ map a diff
+		sequence_ $ map a diff
 		void $ liftIO cleanup
 
 -- Map of old and new filenames for each changed ExportKey in a diff.
@@ -236,6 +243,7 @@ fillExport r db (PreferredFiltered newtree) mtbcommitsha = do
 	allfilledvar <- liftIO $ newMVar (AllFilled True)
 	commandActions $ map (startExport r db cvar allfilledvar) l
 	void $ liftIO $ cleanup
+	waitForAllRunningCommandActions
 
 	case mtbcommitsha of
 		Nothing -> noop
@@ -273,13 +281,12 @@ performExport r db ek af contentsha loc allfilledvar = do
 	sent <- tryNonAsync $ case ek of
 		AnnexKey k -> ifM (inAnnex k)
 			( notifyTransfer Upload af $
-				-- Using noRetry here because interrupted
-				-- exports cannot be resumed.
-				upload (uuid r) k af noRetry $ \pm -> do
+				upload (uuid r) k af stdRetry $ \pm -> do
 					let rollback = void $
 						performUnexport r db [ek] loc
 					sendAnnex k rollback $ \f ->
-						storer f k loc pm
+						Remote.action $
+							storer f k loc pm
 			, do
 				showNote "not available"
 				return False
@@ -290,7 +297,8 @@ performExport r db ek af contentsha loc allfilledvar = do
 				b <- catObject contentsha
 				liftIO $ L.hPut h b
 				liftIO $ hClose h
-				storer tmp sha1k loc nullMeterUpdate
+				Remote.action $
+					storer tmp sha1k loc nullMeterUpdate
 	let failedsend = liftIO $ modifyMVar_ allfilledvar (pure . const (AllFilled False))
 	case sent of
 		Right True -> next $ cleanupExport r db ek loc True
@@ -333,10 +341,12 @@ startUnexport' r db f ek = starting ("unexport " ++ name r) (ActionItemOther (Ju
 -- not really remove the content, which must be accessible later on.
 performUnexport :: Remote -> ExportHandle -> [ExportKey] -> ExportLocation -> CommandPerform
 performUnexport r db eks loc = do
-	ifM (allM (\ek -> removeExport (exportActions r) (asKey ek) loc) eks)
+	ifM (allM rm eks)
 		( next $ cleanupUnexport r db eks loc
 		, stop
 		)
+  where
+	rm ek = Remote.action $ removeExport (exportActions r) (asKey ek) loc
 
 cleanupUnexport :: Remote -> ExportHandle -> [ExportKey] -> ExportLocation -> CommandCleanup
 cleanupUnexport r db eks loc = do
@@ -390,13 +400,13 @@ startMoveFromTempName r db ek f = do
 
 performRename :: Remote -> ExportHandle -> ExportKey -> ExportLocation -> ExportLocation -> CommandPerform
 performRename r db ek src dest =
-	renameExport (exportActions r) (asKey ek) src dest >>= \case
-		Just True -> next $ cleanupRename r db ek src dest
-		Just False -> do
-			warning "rename failed; deleting instead"
+	tryNonAsync (renameExport (exportActions r) (asKey ek) src dest) >>= \case
+		Right (Just ()) -> next $ cleanupRename r db ek src dest
+		Left err -> do
+			warning $ "rename failed (" ++ show err ++ "); deleting instead"
 			fallbackdelete
-		-- Remote does not support renaming, so don't warn about it.
-		Nothing -> fallbackdelete
+		-- remote does not support renaming
+		Right Nothing -> fallbackdelete
   where
 	fallbackdelete = performUnexport r db [ek] src
 
@@ -431,7 +441,7 @@ removeEmptyDirectories r db loc ks
   where
 	go removeexportdirectory d = 
 		ifM (liftIO $ isExportDirectoryEmpty db d)
-			( removeexportdirectory d
+			( Remote.action $ removeexportdirectory d
 			, return True
 			)
 
@@ -477,3 +487,4 @@ filterPreferredContent r tree = logExportExcluded (uuid r) $ \logwriter -> do
 					)
 			-- Always export non-annexed files.
 			Nothing -> return (Just ti)
+
