@@ -13,7 +13,6 @@ import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 
 import Command
-import Annex.Content
 import Limit
 import Types.Key
 import Git.FilePath
@@ -54,23 +53,30 @@ parseFormatOption =
 		)
 
 seek :: FindOptions -> CommandSeek
-seek o = case batchOption o of
-	NoBatch -> withKeyOptions (keyOptions o) False
-		(commandAction . startKeys o)
-		(withFilesInGit (commandAction . go))
-		=<< workTreeItems (findThese o)
-	Batch fmt -> batchFilesMatching fmt (go . toRawFilePath)
+seek o = do
+	islimited <- limited
+	let seeker = AnnexedFileSeeker
+		{ startAction = start o
+		-- only files with content present are shown, unless
+		-- the user has requested others via a limit
+		, checkContentPresent = if islimited
+			then Nothing
+			else Just True
+		, usesLocationLog = False
+		}
+	case batchOption o of
+		NoBatch -> withKeyOptions (keyOptions o) False seeker
+			(commandAction . startKeys o)
+			(withFilesInGitAnnex ww seeker)
+			=<< workTreeItems ww (findThese o)
+		Batch fmt -> batchAnnexedFilesMatching fmt seeker
   where
-	go = whenAnnexed $ start o
+	ww = WarnUnmatchLsFiles
 
--- only files inAnnex are shown, unless the user has requested
--- others via a limit
 start :: FindOptions -> RawFilePath -> Key -> CommandStart
-start o file key =
-	stopUnless (limited <||> inAnnex key) $
-		startingCustomOutput key $ do
-			showFormatted (formatOption o) file $ ("file", fromRawFilePath file) : keyVars key
-			next $ return True
+start o file key = startingCustomOutput key $ do
+	showFormatted (formatOption o) file $ ("file", fromRawFilePath file) : keyVars key
+	next $ return True
 
 startKeys :: FindOptions -> (Key, ActionItem) -> CommandStart
 startKeys o (key, ActionItemBranchFilePath (BranchFilePath _ topf) _) = 
