@@ -31,10 +31,15 @@ cmd = noCommit $ withGlobalOptions [annexedMatchingOptions] $
 		paramPaths (withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek ps = unlessM crippledFileSystem $ do 
-	withFilesInGit
-		(commandAction . (whenAnnexed $ start FixAll))
-		=<< workTreeItems ps
+seek ps = unlessM crippledFileSystem $
+	withFilesInGitAnnex ww seeker =<< workTreeItems ww ps
+  where
+	ww = WarnUnmatchLsFiles
+	seeker = AnnexedFileSeeker
+		{ startAction = start FixAll
+		, checkContentPresent = Nothing
+		, usesLocationLog = False
+		}
 
 data FixWhat = FixSymlinks | FixAll
 
