@@ -41,6 +41,7 @@ import Data.Time.Clock.POSIX
 import System.Posix.Types (EpochTime)
 import qualified Data.Set as S
 import qualified Data.Map as M
+import Data.Either
 
 cmd :: Command
 cmd = withGlobalOptions [jobsOption, jsonOptions, annexedMatchingOptions] $
@@ -80,7 +81,7 @@ optParser desc = FsckOptions
 			( long "more" <> short 'm'
 			<> help "continue an incremental fsck"
 			)
-		<|> (ScheduleIncrementalO <$> option (str >>= parseDuration)
+		<|> (ScheduleIncrementalO <$> option (eitherReader parseDuration)
 			( long "incremental-schedule" <> metavar paramTime
 			<> help "schedule incremental fscking"
 			))
@@ -91,12 +92,19 @@ seek o = startConcurrency commandStages $ do
 	u <- maybe getUUID (pure . Remote.uuid) from
 	checkDeadRepo u
 	i <- prepIncremental u (incrementalOpt o)
-	withKeyOptions (keyOptions o) False
+	let seeker = AnnexedFileSeeker
+		{ startAction = start from i
+		, checkContentPresent = Nothing
+		, usesLocationLog = True
+		}
+	withKeyOptions (keyOptions o) False seeker
 		(\kai -> commandAction . startKey from i kai =<< getNumCopies)
-		(withFilesInGit $ commandAction . (whenAnnexed (start from i)))
-		=<< workTreeItems (fsckFiles o)
+		(withFilesInGitAnnex ww seeker)
+		=<< workTreeItems ww (fsckFiles o)
 	cleanupIncremental i
 	void $ tryIO $ recordActivity Fsck u
+  where
+	ww = WarnUnmatchLsFiles
 
 checkDeadRepo :: UUID -> Annex ()
 checkDeadRepo u =
@@ -174,21 +182,24 @@ performRemote key afile backend numcopies remote =
 		cleanup
 		cleanup `after` a tmp
 	getfile tmp = ifM (checkDiskSpace (Just (takeDirectory tmp)) key 0 True)
-		( ifM (Remote.retrieveKeyFileCheap remote key afile tmp)
+		( ifM (getcheap tmp)
 			( return (Just True)
 			, ifM (Annex.getState Annex.fast)
 				( return Nothing
-				, Just . fst <$>
-					Remote.retrieveKeyFile remote key (AssociatedFile Nothing) tmp dummymeter
+				, Just . isRight <$> tryNonAsync (getfile' tmp)
 				)
 			)
 		, return (Just False)
 		)
+	getfile' tmp = Remote.retrieveKeyFile remote key (AssociatedFile Nothing) tmp dummymeter
 	dummymeter _ = noop
+	getcheap tmp = case Remote.retrieveKeyFileCheap remote of
+		Just a -> isRight <$> tryNonAsync (a key afile tmp)
+		Nothing -> return False
 
 startKey :: Maybe Remote -> Incremental -> (Key, ActionItem) -> NumCopies -> CommandStart
 startKey from inc (key, ai) numcopies =
-	case Backend.maybeLookupBackendVariety (fromKey keyVariety key) of
+	Backend.maybeLookupBackendVariety (fromKey keyVariety key) >>= \case
 		Nothing -> stop
 		Just backend -> runFsck inc ai key $
 			case from of
@@ -250,7 +261,7 @@ verifyLocationLog key keystatus ai = do
 	 - insecure hash is present. This should only be able to happen
 	 - if the repository already contained the content before the
 	 - config was set. -}
-	when (present && not (cryptographicallySecure (fromKey keyVariety key))) $
+	whenM (pure present <&&> (not <$> Backend.isCryptographicallySecure key)) $
 		whenM (annexSecureHashesOnly <$> Annex.getGitConfig) $
 			warning $ "** Despite annex.securehashesonly being set, " ++ obj ++ " has content present in the annex using an insecure " ++ decodeBS (formatKeyVariety (fromKey keyVariety key)) ++ " key"
 
@@ -535,14 +546,14 @@ badContentRemote remote localcopy key = do
 					)
 		)
 
-	dropped <- Remote.removeKey remote key
-	when dropped $
+	dropped <- tryNonAsync (Remote.removeKey remote key)
+	when (isRight dropped) $
 		Remote.logStatus remote key InfoMissing
 	return $ case (movedbad, dropped) of
-		(True, True) -> "moved from " ++ Remote.name remote ++
+		(True, Right ()) -> "moved from " ++ Remote.name remote ++
 			" to " ++ destbad
-		(False, True) -> "dropped from " ++ Remote.name remote
-		(_, False) -> "failed to drop from" ++ Remote.name remote
+		(False, Right ()) -> "dropped from " ++ Remote.name remote
+		(_, Left e) -> "failed to drop from" ++ Remote.name remote ++ ": " ++ show e
 
 runFsck :: Incremental -> ActionItem -> Key -> Annex Bool -> CommandStart
 runFsck inc ai key a = stopUnless (needFsck inc key) $
