@@ -1,11 +1,12 @@
 {- git-annex command
  -
- - Copyright 2013-2017 Joey Hess <id@joeyh.name>
+ - Copyright 2013-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE BangPatterns #-}
 
 module Command.ImportFeed where
 
@@ -16,8 +17,11 @@ import qualified Data.Set as S
 import qualified Data.Map as M
 import Data.Time.Clock
 import Data.Time.Format
+import Data.Time.Calendar
+import Data.Time.LocalTime
 import qualified Data.Text as T
 import System.Log.Logger
+import Control.Concurrent.Async
 
 import Command
 import qualified Annex
@@ -40,6 +44,11 @@ import Logs.MetaData
 import Annex.MetaData
 import Annex.FileMatcher
 import Command.AddUrl (addWorkTree)
+import Annex.UntrustedFilePath
+import qualified Git.Ref
+import qualified Annex.Branch
+import Logs
+import Git.CatFile (catObjectStream)
 
 cmd :: Command
 cmd = notBareRepo $
@@ -116,7 +125,7 @@ getCache opttemplate = ifM (Annex.getState Annex.force)
 	( ret S.empty S.empty
 	, do
 		showStart "importfeed" "checking known urls"
-		(is, us) <- unzip <$> (mapM knownItems =<< knownUrls)
+		(is, us) <- unzip <$> knownItems
 		showEndOk
 		ret (S.fromList us) (S.fromList (concat is))
 	)
@@ -124,13 +133,33 @@ getCache opttemplate = ifM (Annex.getState Annex.force)
 	tmpl = Utility.Format.gen $ fromMaybe defaultTemplate opttemplate
 	ret us is = return $ Cache us is tmpl
 
-knownItems :: (Key, URLString) -> Annex ([ItemId], URLString)
-knownItems (k, u) = do
-	itemids <- S.toList . S.filter (/= noneValue)
-		. S.map (decodeBS . fromMetaValue)
-		. currentMetaDataValues itemIdField 
-		<$> getCurrentMetaData k
-	return (itemids, u)
+knownItems :: Annex [([ItemId], URLString)]
+knownItems = do
+	g <- Annex.gitRepo
+	config <- Annex.getGitConfig
+	catObjectStream g $ \catfeeder catcloser catreader -> do
+		rt <- liftIO $ async $ reader catreader []
+		withKnownUrls (feeder config catfeeder catcloser)
+		liftIO (wait rt)
+  where
+	feeder config catfeeder catcloser urlreader = urlreader >>= \case
+		Just (k, us) -> do
+			forM_ us $ \u ->
+				let logf = metaDataLogFile config k
+				    ref = Git.Ref.branchFileRef Annex.Branch.fullname logf
+				in liftIO $ catfeeder (u, ref)
+			feeder config catfeeder catcloser urlreader
+		Nothing -> liftIO catcloser
+	
+	reader catreader c = catreader >>= \case
+		Just (u, Just mdc) ->
+			let !itemids = S.toList $ S.filter (/= noneValue) $
+				S.map (decodeBS . fromMetaValue) $
+					currentMetaDataValues itemIdField $
+						parseCurrentMetaData mdc
+			in reader catreader ((itemids,u):c)
+		Just (u, Nothing) -> reader catreader (([],u):c)
+		Nothing -> return c
 
 findDownloads :: URLString -> Feed -> [ToDownload]
 findDownloads u f = catMaybes $ map mk (feedItems f)
@@ -140,8 +169,8 @@ findDownloads u f = catMaybes $ map mk (feedItems f)
 			Just $ ToDownload f u i $ Enclosure $ 
 				T.unpack enclosureurl
 		Nothing -> case getItemLink i of
-			Just link -> Just $ ToDownload f u i $ 
-				MediaLink $ T.unpack link
+			Just l -> Just $ ToDownload f u i $ 
+				MediaLink $ T.unpack l
 			Nothing -> Nothing
 
 {- Feeds change, so a feed download cannot be resumed. -}
@@ -162,10 +191,6 @@ performDownload addunlockedmatcher opts cache todownload = case location todownl
 			r <- Remote.claimingUrl url
 			if Remote.uuid r == webUUID || rawOption (downloadOptions opts)
 				then do
-					urlinfo <- if relaxedOption (downloadOptions opts)
-						then pure Url.assumeUrlExists
-						else Url.withUrlOptions $
-							Url.getUrlInfo url
 					let dlopts = (downloadOptions opts)
 						-- force using the filename
 						-- chosen here
@@ -173,7 +198,14 @@ performDownload addunlockedmatcher opts cache todownload = case location todownl
 						-- don't use youtube-dl
 						, rawOption = True
 						}
-					maybeToList <$> addUrlFile addunlockedmatcher dlopts url urlinfo f
+					let go urlinfo = maybeToList <$> addUrlFile addunlockedmatcher dlopts url urlinfo f
+					if relaxedOption (downloadOptions opts)
+						then go Url.assumeUrlExists
+						else Url.withUrlOptions (Url.getUrlInfo url) >>= \case
+							Right urlinfo -> go urlinfo
+							Left err -> do
+								warning err
+								return []
 				else do
 					res <- tryNonAsync $ maybe
 						(error $ "unable to checkUrl of " ++ Remote.name r)
@@ -186,7 +218,7 @@ performDownload addunlockedmatcher opts cache todownload = case location todownl
 								downloadRemoteFile addunlockedmatcher r (downloadOptions opts) url f sz
 						Right (UrlMulti l) -> do
 							kl <- forM l $ \(url', sz, subf) ->
-								downloadRemoteFile addunlockedmatcher r (downloadOptions opts) url' (f </> fromSafeFilePath subf) sz
+								downloadRemoteFile addunlockedmatcher r (downloadOptions opts) url' (f </> sanitizeFilePath subf) sz
 							return $ if all isJust kl
 								then catMaybes kl
 								else []
@@ -306,18 +338,47 @@ defaultTemplate = "${feedtitle}/${itemtitle}${extension}"
 {- Generates a filename to use for a feed item by filling out the template.
  - The filename may not be unique. -}
 feedFile :: Utility.Format.Format -> ToDownload -> String -> FilePath
-feedFile tmpl i extension = Utility.Format.format tmpl $
-	M.map sanitizeFilePath $ M.fromList $ extractFields i ++
-		[ ("extension", extension)
-		, extractField "itempubdate" [pubdate $ item i]
-		]
+feedFile tmpl i extension = sanitizeLeadingFilePathCharacter $ 
+	Utility.Format.format tmpl $
+		M.map sanitizeFilePathComponent $ M.fromList $ extractFields i ++
+			[ ("extension", extension)
+			, extractField "itempubdate" [itempubdate]
+			, extractField "itempubyear" [itempubyear]
+			, extractField "itempubmonth" [itempubmonth]
+			, extractField "itempubday" [itempubday]
+			, extractField "itempubhour" [itempubhour]
+			, extractField "itempubminute" [itempubminute]
+			, extractField "itempubsecond" [itempubsecond]
+			]
   where
-	pubdate itm = case getItemPublishDate itm :: Maybe (Maybe UTCTime) of
-		Just (Just d) -> Just $
-			formatTime defaultTimeLocale "%F" d
+	itm = item i
+
+	pubdate = case getItemPublishDate itm :: Maybe (Maybe UTCTime) of
+		Just (Just d) -> Just d
+		_ -> Nothing
+	
+	itempubdate = case pubdate of
+		Just pd -> Just $
+			formatTime defaultTimeLocale "%F" pd
 		-- if date cannot be parsed, use the raw string
-		_ -> replace "/" "-" . T.unpack
+		Nothing-> replace "/" "-" . T.unpack
 			<$> getItemPublishDateString itm
+	
+	(itempubyear, itempubmonth, itempubday) = case pubdate of
+		Nothing -> (Nothing, Nothing, Nothing)
+		Just pd -> 
+			let (y, m, d) = toGregorian (utctDay pd)
+			in (Just (show y), Just (show m), Just (show d))
+	
+	(itempubhour, itempubminute, itempubsecond) = case pubdate of
+		Nothing -> (Nothing, Nothing, Nothing)
+		Just pd -> 
+			let tod = timeToTimeOfDay (utctDayTime pd)
+			in ( Just (show (todHour tod))
+			   , Just (show (todMin tod))
+			   -- avoid fractional seconds
+			   , Just (takeWhile (/= '.') (show (todSec tod)))
+			   )
 
 extractMetaData :: ToDownload -> MetaData
 extractMetaData i = case getItemPublishDate (item i) :: Maybe (Maybe UTCTime) of
