@@ -30,7 +30,6 @@ import Logs.UUID
 import Logs.Trust
 import Logs.Location
 import Annex.NumCopies
-import Remote
 import Git.Config (boolConfig)
 import qualified Git.LsTree as LsTree
 import Utility.Percentage
@@ -119,7 +118,7 @@ optParser desc = InfoOptions
 seek :: InfoOptions -> CommandSeek
 seek o = case batchOption o of
 	NoBatch -> withWords (commandAction . start o) (infoFor o)
-	Batch fmt -> batchInput fmt Right (itemInfo o)
+	Batch fmt -> batchInput fmt (pure . Right) (itemInfo o)
 
 start :: InfoOptions -> [String] -> CommandStart
 start o [] = do
@@ -152,9 +151,11 @@ itemInfo o p = ifM (isdir p)
 				v' <- Remote.nameToUUID' p
 				case v' of
 					Right u -> uuidInfo o u
-					Left _ -> ifAnnexed (toRawFilePath p)
-						(fileInfo o p)
-						(treeishInfo o p)
+					Left _ -> do
+						relp <- liftIO $ relPathCwdToFile p
+						ifAnnexed (toRawFilePath relp)
+							(fileInfo o relp)
+							(treeishInfo o p)
 	)
   where
 	isdir = liftIO . catchBoolIO . (isDirectory <$$> getFileStatus)
@@ -181,7 +182,7 @@ dirInfo o dir = showCustom (unwords ["info", dir]) $ do
 
 treeishInfo :: InfoOptions -> String -> Annex ()
 treeishInfo o t = do
-	mi <- getTreeStatInfo o (Git.Ref t)
+	mi <- getTreeStatInfo o (Git.Ref (encodeBS' t))
 	case mi of
 		Nothing -> noInfo t
 		Just i -> showCustom (unwords ["info", t]) $ do
@@ -317,11 +318,11 @@ showStat s = maybe noop calc =<< s
 repo_list :: TrustLevel -> Stat
 repo_list level = stat n $ nojson $ lift $ do
 	us <- filter (/= NoUUID) . M.keys 
-		<$> (M.union <$> (M.map fromUUIDDesc <$> uuidDescMap) <*> remoteMap Remote.name)
+		<$> (M.union <$> (M.map fromUUIDDesc <$> uuidDescMap) <*> Remote.remoteMap Remote.name)
 	rs <- fst <$> trustPartition level us
 	countRepoList (length rs)
 		-- This also handles json display.
-		<$> prettyPrintUUIDs n rs
+		<$> Remote.prettyPrintUUIDs n rs
   where
 	n = showTrustLevel level ++ " repositories"
 
@@ -495,9 +496,9 @@ reposizes_stats = stat desc $ nojson $ do
 		. M.toList
 		<$> cachedRepoData
 	let maxlen = maximum (map (length . snd) l)
-	descm <- lift uuidDescriptions
+	descm <- lift Remote.uuidDescriptions
 	-- This also handles json display.
-	s <- lift $ prettyPrintUUIDsWith (Just "size") desc descm (Just . show) $
+	s <- lift $ Remote.prettyPrintUUIDsWith (Just "size") desc descm (Just . show) $
 		map (\(u, sz) -> (u, Just $ mkdisp sz maxlen)) l
 	return $ countRepoList (length l) s
   where
