@@ -38,9 +38,15 @@ seek o = do
 			| otherwise -> commandAction stop
 		_ -> do
 			let s = S.fromList ts
-			withFilesInGit
-				(commandAction . (whenAnnexed (start s)))
-				=<< workTreeItems (inprogressFiles o)
+			let seeker = AnnexedFileSeeker
+				{ startAction = start s
+				, checkContentPresent = Nothing
+				, usesLocationLog = False
+				}
+			withFilesInGitAnnex ww seeker
+				=<< workTreeItems ww (inprogressFiles o)
+  where
+	ww = WarnUnmatchLsFiles
 
 start :: S.Set Key -> RawFilePath -> Key -> CommandStart
 start s _file k
