@@ -44,9 +44,14 @@ seek :: ListOptions -> CommandSeek
 seek o = do
 	list <- getList o
 	printHeader list
-	withFilesInGit
-		(commandAction . (whenAnnexed $ start list))
-		=<< workTreeItems (listThese o)
+	let seeker = AnnexedFileSeeker
+		{ startAction = start list
+		, checkContentPresent = Nothing
+		, usesLocationLog = True
+		}
+	withFilesInGitAnnex ww seeker =<< workTreeItems ww (listThese o)
+  where
+	ww = WarnUnmatchLsFiles
 
 getList :: ListOptions -> Annex [(UUID, RemoteName, TrustLevel)]
 getList o
