@@ -85,12 +85,20 @@ seek o = do
 	m <- Remote.uuidDescriptions
 	zone <- liftIO getCurrentTimeZone
 	let outputter = mkOutputter m zone o
+	let seeker = AnnexedFileSeeker
+		{ startAction = start o outputter
+		, checkContentPresent = Nothing
+		-- the way this uses the location log would not be helped
+		-- by precaching the current value
+		, usesLocationLog = False
+		}
 	case (logFiles o, allOption o) of
-		(fs, False) -> withFilesInGit
-			(commandAction . (whenAnnexed $ start o outputter)) 
-			=<< workTreeItems fs
+		(fs, False) -> withFilesInGitAnnex ww seeker
+			=<< workTreeItems ww fs
 		([], True) -> commandAction (startAll o outputter)
 		(_, True) -> giveup "Cannot specify both files and --all"
+  where
+	ww = WarnUnmatchLsFiles
 
 start :: LogOptions -> (FilePath -> Outputter) -> RawFilePath -> Key -> CommandStart
 start o outputter file key = do
@@ -264,7 +272,8 @@ parseGitRawLog config = parse epoch
 parseRawChangeLine :: String -> Maybe (Git.Ref, Git.Ref)
 parseRawChangeLine = go . words
   where
-	go (_:_:oldsha:newsha:_) = Just (Git.Ref oldsha, Git.Ref newsha)
+	go (_:_:oldsha:newsha:_) = 
+		Just (Git.Ref (encodeBS oldsha), Git.Ref (encodeBS newsha))
 	go _ = Nothing
 
 parseTimeStamp :: String -> POSIXTime
