@@ -222,18 +222,24 @@ tryScan r
 		Nothing -> return $ Just r
 	| otherwise = liftIO $ safely $ Git.Config.read r
   where
-	pipedconfig pcmd params = liftIO $ safely $
-		withHandle StdoutHandle createProcessSuccess p $
-			Git.Config.hRead r
+	pipedconfig st pcmd params = liftIO $ safely $
+		withCreateProcess p (pipedconfig' st p)
 	  where
-		p = proc pcmd $ toCommand params
+		p = (proc pcmd $ toCommand params)
+			{ std_out = CreatePipe }
+
+	pipedconfig' st p _ (Just h) _ pid = 
+		forceSuccessProcess p pid
+			`after`
+		Git.Config.hRead r st h
+	pipedconfig' _ _ _ _ _ _ = error "internal"
 
 	configlist = Ssh.onRemote NoConsumeStdin r
-		(pipedconfig, return Nothing) "configlist" [] []
+		(pipedconfig Git.Config.ConfigList, return Nothing) "configlist" [] []
 	manualconfiglist = do
 		gc <- Annex.getRemoteGitConfig r
 		(sshcmd, sshparams) <- Ssh.toRepo NoConsumeStdin r gc remotecmd
-		liftIO $ pipedconfig sshcmd sshparams
+		liftIO $ pipedconfig Git.Config.ConfigNullList sshcmd sshparams
 	  where
 		remotecmd = "sh -c " ++ shellEscape
 			(cddir ++ " && " ++ "git config --null --list")
