@@ -26,11 +26,11 @@ seek [] = do
 	commandAction mergeSyncedBranch
 seek bs = do
 	prepMerge
-	forM_ bs (commandAction . mergeBranch . Git.Ref)
+	forM_ bs (commandAction . mergeBranch . Git.Ref . encodeBS')
 
 mergeAnnexBranch :: CommandStart
 mergeAnnexBranch = starting "merge" (ActionItemOther (Just "git-annex")) $ do
-	Annex.Branch.update
+	_ <- Annex.Branch.update
 	-- commit explicitly, in case no remote branches were merged
 	Annex.Branch.commit =<< Annex.Branch.commitMessage
 	next $ return True
