@@ -75,15 +75,21 @@ seek :: MetaDataOptions -> CommandSeek
 seek o = case batchOption o of
 	NoBatch -> do
 		c <- liftIO currentVectorClock
-		let seeker = case getSet o of
-			Get _ -> withFilesInGit
-			GetAll -> withFilesInGit
-			Set _ -> withFilesInGitNonRecursive
+		let ww = WarnUnmatchLsFiles
+		let seeker = AnnexedFileSeeker
+			{ startAction = start c o
+			, checkContentPresent = Nothing
+			, usesLocationLog = False
+			}
+		let seekaction = case getSet o of
+			Get _ -> withFilesInGitAnnex ww
+			GetAll -> withFilesInGitAnnex ww
+			Set _ -> withFilesInGitAnnexNonRecursive ww
 				"Not recursively setting metadata. Use --force to do that."
-		withKeyOptions (keyOptions o) False
+		withKeyOptions (keyOptions o) False seeker
 			(commandAction . startKeys c o)
-			(seeker (commandAction . (whenAnnexed (start c o))))
-			=<< workTreeItems (forFiles o)
+			(seekaction seeker)
+			=<< workTreeItems ww (forFiles o)
 	Batch fmt -> withMessageState $ \s -> case outputType s of
 		JSONOutput _ -> ifM limited
 			( giveup "combining --batch with file matching options is not currently supported"
@@ -119,8 +125,9 @@ perform c o k = case getSet o of
 cleanup :: Key -> CommandCleanup
 cleanup k = do
 	m <- getCurrentMetaData k
-	let Object o = toJSON' (MetaDataFields m)
-	maybeShowJSON $ AesonObject o
+	case toJSON' (MetaDataFields m) of
+		Object o -> maybeShowJSON $ AesonObject o
+		_ -> noop
 	showLongNote $ unlines $ concatMap showmeta $
 		map unwrapmeta (fromMetaData m)
 	return True
@@ -147,21 +154,26 @@ instance FromJSON MetaDataFields where
 fieldsField :: T.Text
 fieldsField = T.pack "fields"
 
-parseJSONInput :: String -> Either String (Either RawFilePath Key, MetaData)
-parseJSONInput i = do
-	v <- eitherDecode (BU.fromString i)
-	let m = case itemAdded v of
-		Nothing -> emptyMetaData
-		Just (MetaDataFields m') -> m'
-	case (itemKey v, itemFile v) of
-		(Just k, _) -> Right (Right k, m)
-		(Nothing, Just f) -> Right (Left (toRawFilePath f), m)
-		(Nothing, Nothing) -> Left "JSON input is missing either file or key"
+parseJSONInput :: String -> Annex (Either String (Either RawFilePath Key, MetaData))
+parseJSONInput i = case eitherDecode (BU.fromString i) of
+	Left e -> return (Left e)
+	Right v -> do
+		let m = case itemAdded v of
+			Nothing -> emptyMetaData
+			Just (MetaDataFields m') -> m'
+		case (itemKey v, itemFile v) of
+			(Just k, _) -> return $
+				Right (Right k, m)
+			(Nothing, Just f) -> do
+				f' <- liftIO $ relPathCwdToFile f
+				return $ Right (Left (toRawFilePath f'), m)
+			(Nothing, Nothing) -> return $ 
+				Left "JSON input is missing either file or key"
 
 startBatch :: (Either RawFilePath Key, MetaData) -> CommandStart
 startBatch (i, (MetaData m)) = case i of
 	Left f -> do
-		mk <- lookupFile f
+		mk <- lookupKey f
 		case mk of
 			Just k -> go k (mkActionItem (k, AssociatedFile (Just f)))
 			Nothing -> giveup $ "not an annexed file: " ++ fromRawFilePath f
