@@ -41,11 +41,21 @@ instance DeferredParseClass MirrorOptions where
 		<*> pure (keyOptions v)
 
 seek :: MirrorOptions -> CommandSeek
-seek o = startConcurrency transferStages $ 
-	withKeyOptions (keyOptions o) False
+seek o = startConcurrency stages $ 
+	withKeyOptions (keyOptions o) False seeker
 		(commandAction . startKey o (AssociatedFile Nothing))
-		(withFilesInGit (commandAction . (whenAnnexed $ start o)))
-		=<< workTreeItems (mirrorFiles o)
+		(withFilesInGitAnnex ww seeker)
+		=<< workTreeItems ww (mirrorFiles o)
+  where
+	stages = case fromToOptions o of
+		FromRemote _ -> downloadStages
+		ToRemote _ -> commandStages
+	ww = WarnUnmatchLsFiles
+	seeker = AnnexedFileSeeker
+		{ startAction = start o
+		, checkContentPresent = Nothing
+		, usesLocationLog = True
+		}
 
 start :: MirrorOptions -> RawFilePath -> Key -> CommandStart
 start o file k = startKey o afile (k, ai)
@@ -65,7 +75,10 @@ startKey o afile (key, ai) = case fromToOptions o of
 		haskey <- flip Remote.hasKey key =<< getParsed r
 		case haskey of
 			Left _ -> stop
-			Right True -> Command.Get.start' (return True) Nothing key afile ai
+			Right True -> ifM (inAnnex key)
+				( stop
+				, Command.Get.start' (return True) Nothing key afile ai
+				)
 			Right False -> ifM (inAnnex key)
 				( do
 					numcopies <- getnumcopies
