@@ -16,7 +16,6 @@ import Annex.Hook
 import Annex.Link
 import Annex.View
 import Annex.View.ViewedFile
-import Annex.LockFile
 import Logs.View
 import Logs.MetaData
 import Types.View
@@ -33,7 +32,8 @@ cmd = command "pre-commit" SectionPlumbing
 
 seek :: CmdParams -> CommandSeek
 seek ps = do
-	l <- workTreeItems ps
+	let ww = WarnUnmatchWorkTreeItems
+	l <- workTreeItems ww ps
 	-- fix symlinks to files being committed
 	flip withFilesToBeCommitted l $ \f -> commandAction $
 		maybe stop (Command.Fix.start Command.Fix.FixSymlinks f)
