@@ -88,18 +88,23 @@ clean file = do
 	b <- liftIO $ L.hGetContents stdin
 	ifM fileoutsiderepo
 		( liftIO $ L.hPut stdout b
-		, case parseLinkTargetOrPointerLazy b of
-			Just k -> do
-				getMoveRaceRecovery k (toRawFilePath file)
-				liftIO $ L.hPut stdout b
-			Nothing -> do
-				let fileref = Git.Ref.fileRef (toRawFilePath file)
-				indexmeta <- catObjectMetaData fileref
-				go b indexmeta =<< catKey' fileref indexmeta
+		, do
+			-- Avoid a potential deadlock.
+			Annex.changeState $ \s -> s
+				{ Annex.insmudgecleanfilter = True }
+			go b
 		)
 	stop
   where
-	go b indexmeta oldkey = ifM (shouldAnnex file indexmeta oldkey)
+	go b = case parseLinkTargetOrPointerLazy b of
+		Just k -> do
+			getMoveRaceRecovery k (toRawFilePath file)
+			liftIO $ L.hPut stdout b
+		Nothing -> do
+			let fileref = Git.Ref.fileRef (toRawFilePath file)
+			indexmeta <- catObjectMetaData fileref
+			go' b indexmeta =<< catKey' fileref indexmeta
+	go' b indexmeta oldkey = ifM (shouldAnnex file indexmeta oldkey)
 		( do
 			-- Before git 2.5, failing to consume all stdin here
 			-- would cause a SIGPIPE and crash it.
@@ -126,7 +131,10 @@ clean file = do
 		-- Look up the backend that was used for this file
 		-- before, so that when git re-cleans a file its
 		-- backend does not change.
-		let oldbackend = maybe Nothing (maybeLookupBackendVariety . fromKey keyVariety) oldkey
+		oldbackend <- maybe
+			(pure Nothing)
+			(maybeLookupBackendVariety . fromKey keyVariety)
+			oldkey
 		-- Can't restage associated files because git add
 		-- runs this and has the index locked.
 		let norestage = Restage False
