@@ -46,7 +46,7 @@ import Config
 import Config.GitConfig
 import Annex.SpecialRemote.Config
 import Config.DynamicConfig
-import Config.Files
+import Annex.Path
 import Annex.Wanted
 import Annex.Content
 import Command.Get (getKey')
@@ -69,9 +69,12 @@ import qualified Database.Export as Export
 import Utility.Bloom
 import Utility.OptParse
 import Utility.Process.Transcript
+import Utility.Tuple
 
 import Control.Concurrent.MVar
 import qualified Data.Map as M
+import qualified Data.ByteString as S
+import Data.Char
 
 cmd :: Command
 cmd = withGlobalOptions [jobsOption] $
@@ -192,7 +195,7 @@ instance DeferredParseClass SyncOptions where
 seek :: SyncOptions -> CommandSeek
 seek o = do
 	prepMerge
-	startConcurrency transferStages (seek' o)
+	startConcurrency downloadStages (seek' o)
 	
 seek' :: SyncOptions -> CommandSeek
 seek' o = do
@@ -265,13 +268,13 @@ mergeConfig =
 	]
 
 merge :: CurrBranch -> [Git.Merge.MergeConfig] -> SyncOptions -> Git.Branch.CommitMode -> Git.Branch -> Annex Bool
-merge currbranch mergeconfig o commitmode tomerge = case currbranch of
-	(Just b, Just adj) -> mergeToAdjustedBranch tomerge (b, adj) mergeconfig canresolvemerge commitmode
-	(b, _) -> autoMergeFrom tomerge b mergeconfig canresolvemerge commitmode
-  where
-	canresolvemerge = if resolveMergeOverride o
+merge currbranch mergeconfig o commitmode tomerge = do
+	canresolvemerge <- if resolveMergeOverride o
 		then getGitConfigVal annexResolveMerge
 		else return False
+	case currbranch of
+		(Just b, Just adj) -> mergeToAdjustedBranch tomerge (b, adj) mergeconfig canresolvemerge commitmode
+		(b, _) -> autoMergeFrom tomerge b mergeconfig commitmode canresolvemerge
 
 syncBranch :: Git.Branch -> Git.Branch
 syncBranch = Git.Ref.underBase "refs/heads/synced" . fromAdjustedBranch
@@ -289,10 +292,8 @@ syncRemotes ps = do
 
 syncRemotes' :: [String] -> [Remote] -> Annex [Remote]
 syncRemotes' ps available = 
-	ifM (Annex.getState Annex.fast) ( nub <$> pickfast , wanted )
+	ifM (Annex.getState Annex.fast) ( fastest <$> wanted , wanted )
   where
-	pickfast = (++) <$> listed <*> (filterM good (fastest available))
-	
 	wanted
 		| null ps = filterM good (concat $ Remote.byCost available)
 		| otherwise = listed
@@ -346,36 +347,41 @@ mergeLocal mergeconfig o currbranch = stopUnless (notOnlyAnnex o) $
 	mergeLocal' mergeconfig o currbranch
 
 mergeLocal' :: [Git.Merge.MergeConfig] -> SyncOptions -> CurrBranch -> CommandStart
-mergeLocal' mergeconfig o currbranch@(Just _, _) =
-	needMerge currbranch >>= \case
+mergeLocal' mergeconfig o currbranch@(Just branch, _) =
+	needMerge currbranch branch >>= \case
 		Nothing -> stop
 		Just syncbranch ->
 			starting "merge" (ActionItemOther (Just $ Git.Ref.describe syncbranch)) $
 				next $ merge currbranch mergeconfig o Git.Branch.ManualCommit syncbranch
-mergeLocal' _ _ (Nothing, madj) = do
-	b <- inRepo Git.Branch.currentUnsafe
-	needMerge (b, madj) >>= \case
+mergeLocal' _ _ currbranch@(Nothing, _) = inRepo Git.Branch.currentUnsafe >>= \case
+	Just branch -> needMerge currbranch branch >>= \case
 		Nothing -> stop
 		Just syncbranch ->
 			starting "merge" (ActionItemOther (Just $ Git.Ref.describe syncbranch)) $ do
-				warning $ "There are no commits yet in the currently checked out branch, so cannot merge any remote changes into it."
+				warning $ "There are no commits yet to branch " ++ Git.fromRef branch ++ ", so cannot merge " ++ Git.fromRef syncbranch ++ " into it."
 				next $ return False
+	Nothing -> stop
 
 -- Returns the branch that should be merged, if any.
-needMerge :: CurrBranch -> Annex (Maybe Git.Branch)
-needMerge (Nothing, _) = return Nothing
-needMerge (Just branch, madj) = ifM (allM id checks)
+needMerge :: CurrBranch -> Git.Branch -> Annex (Maybe Git.Branch)
+needMerge currbranch headbranch = ifM (allM id checks)
 	( return (Just syncbranch)
 	, return Nothing
 	)
   where
-	checks =
-		[ not <$> isBareRepo
-		, inRepo (Git.Ref.exists syncbranch)
-		, inRepo (Git.Branch.changed branch' syncbranch)
-		]
-	syncbranch = syncBranch branch
-	branch' = maybe branch (adjBranch . originalToAdjusted branch) madj
+	syncbranch = syncBranch headbranch
+	checks = case currbranch of
+		(Just _, madj) -> 
+			let branch' = maybe headbranch (adjBranch . originalToAdjusted headbranch) madj
+			in 
+				[ not <$> isBareRepo
+				, inRepo (Git.Ref.exists syncbranch)
+				, inRepo (Git.Branch.changed branch' syncbranch)
+				]
+		(Nothing, _) ->
+			[ not <$> isBareRepo
+			, inRepo (Git.Ref.exists syncbranch)
+			]
 
 pushLocal :: SyncOptions -> CurrBranch -> CommandStart
 pushLocal o b = stopUnless (notOnlyAnnex o) $ do
@@ -444,12 +450,12 @@ importRemote o mergeconfig remote currbranch
 	| otherwise = case remoteAnnexTrackingBranch (Remote.gitconfig remote) of
 		Nothing -> noop
 		Just tb -> do
-			let (b, s) = separate (== ':') (Git.fromRef tb)
+			let (b, p) = separate' (== (fromIntegral (ord ':'))) (Git.fromRef' tb)
 			let branch = Git.Ref b
-			let subdir = if null s
+			let subdir = if S.null p
 				then Nothing
-				else Just (asTopFilePath (toRawFilePath s))
-			Command.Import.seekRemote remote branch subdir
+				else Just (asTopFilePath p)
+			Command.Import.seekRemote remote branch subdir True
 			void $ mergeRemote remote currbranch mergeconfig o
   where
 	wantpull = remoteAnnexPull (Remote.gitconfig remote)
@@ -508,14 +514,16 @@ pushRemote o remote (Just branch, _) = do
 	postpushupdate repo = case Git.repoWorkTree repo of
 		Nothing -> return True
 		Just wt -> ifM needemulation
-			( liftIO $ do
-				p <- readProgramFile
-				boolSystem' p [Param "post-receive"]
-					(\cp -> cp { cwd = Just (fromRawFilePath wt) })
+			( gitAnnexChildProcess ["post-receive"]
+				(\cp -> cp { cwd = Just (fromRawFilePath wt) })
+				(\_ _ _ pid -> waitForProcess pid >>= return . \case
+					ExitSuccess -> True
+					_ -> False
+				)
 			, return True
 			)
 	  where
-		needemulation = Remote.Git.onLocal repo remote $
+		needemulation = Remote.Git.onLocalRepo repo $
 			(annexCrippledFileSystem <$> Annex.getGitConfig)
 				<&&>
 			needUpdateInsteadEmulation			
@@ -628,43 +636,49 @@ seekSyncContent _ [] _ = return False
 seekSyncContent o rs currbranch = do
 	mvar <- liftIO newEmptyMVar
 	bloom <- case keyOptions o of
-		Just WantAllKeys -> Just <$> genBloomFilter (seekworktree mvar [])
+		Just WantAllKeys -> Just <$> genBloomFilter (seekworktree mvar (WorkTreeItems []))
 		_ -> case currbranch of
                 	(Just origbranch, Just adj) | adjustmentHidesFiles adj -> do
-				l <- workTreeItems' (AllowHidden True) (contentOfOption o)
+				l <- workTreeItems' (AllowHidden True) ww (contentOfOption o)
 				seekincludinghidden origbranch mvar l (const noop)
 				pure Nothing
 			_ -> do
-				l <- workTreeItems (contentOfOption o)
+				l <- workTreeItems ww (contentOfOption o)
 				seekworktree mvar l (const noop)
 				pure Nothing
 	withKeyOptions' (keyOptions o) False
-		(return (gokey mvar bloom))
+		(return (commandAction . gokey mvar bloom))
 		(const noop)
-		[]
-	finishCommandActions
+		(WorkTreeItems [])
+	waitForAllRunningCommandActions
 	liftIO $ not <$> isEmptyMVar mvar
   where
-	seekworktree mvar l bloomfeeder = 
-		seekHelper LsFiles.inRepo l
-			>>= gofiles bloomfeeder mvar
+	seekworktree mvar l bloomfeeder = do
+		let seeker = AnnexedFileSeeker
+			{ startAction = gofile bloomfeeder mvar
+			, checkContentPresent = Nothing
+			, usesLocationLog = True
+			}
+		seekFilteredKeys seeker $
+			seekHelper fst3 ww LsFiles.inRepoDetails l
 
-	seekincludinghidden origbranch mvar l bloomfeeder = 
-		seekHelper (LsFiles.inRepoOrBranch origbranch) l 
-			>>= gofiles bloomfeeder mvar
+	seekincludinghidden origbranch mvar l bloomfeeder =
+		seekFiltered (\f -> ifAnnexed f (commandAction . gofile bloomfeeder mvar f) noop) $
+			seekHelper id ww (LsFiles.inRepoOrBranch origbranch) l 
 
-	gofiles bloomfeeder mvar = mapM_ $ \f ->
-		ifAnnexed f
-			(go (Right bloomfeeder) mvar (AssociatedFile (Just f)))
-			noop
+	ww = WarnUnmatchLsFiles
+
+	gofile bloom mvar f k = 
+		go (Right bloom) mvar (AssociatedFile (Just f)) k
 	
-	gokey mvar bloom (k, _) = go (Left bloom) mvar (AssociatedFile Nothing) k
+	gokey mvar bloom (k, _) =
+		go (Left bloom) mvar (AssociatedFile Nothing) k
 
 	go ebloom mvar af k = do
 		-- Run syncFile as a command action so file transfers run
 		-- concurrently.
 		let ai = OnlyActionOn k (ActionItemKey k)
-		commandAction $ startingNoMessage ai $ do
+		startingNoMessage ai $ do
 			whenM (syncFile ebloom rs af k) $
 				void $ liftIO $ tryPutMVar mvar ()
 			next $ return True
