@@ -9,7 +9,6 @@ module Command.Unannex where
 
 import Command
 import qualified Annex
-import Annex.Content
 import Annex.Perms
 import qualified Git.Command
 import Utility.CopyFile
@@ -23,10 +22,19 @@ cmd = withGlobalOptions [annexedMatchingOptions] $
 		paramPaths (withParams seek)
 
 seek :: CmdParams -> CommandSeek
-seek ps = (withFilesInGit $ commandAction . whenAnnexed start) =<< workTreeItems ps
+seek ps = withFilesInGitAnnex ww seeker =<< workTreeItems ww ps
+  where
+	ww = WarnUnmatchLsFiles
+
+seeker :: AnnexedFileSeeker
+seeker = AnnexedFileSeeker
+	{ startAction = start
+	, checkContentPresent = Just True
+	, usesLocationLog = False
+	}
 
 start :: RawFilePath -> Key -> CommandStart
-start file key = stopUnless (inAnnex key) $
+start file key = 
 	starting "unannex" (mkActionItem (key, file)) $
 		perform file key
 
