@@ -29,12 +29,12 @@ import Annex.SpecialRemote.Config
 import Annex.Perms
 import Utility.FileMode
 import Crypto
-import Types.Remote (RemoteConfig, RemoteConfigField)
 import Types.ProposedAccepted
-import Remote.Helper.Encryptable (remoteCipher, remoteCipher', embedCreds, EncryptionIsSetup, extractCipher, parseEncryptionConfig)
+import Remote.Helper.Encryptable (remoteCipher, remoteCipher', embedCreds, EncryptionIsSetup, extractCipher)
 import Utility.Env (getEnv)
 
 import qualified Data.ByteString.Lazy.Char8 as L
+import qualified Data.ByteString.Char8 as S
 import qualified Data.Map as M
 import Utility.Base64
 
@@ -56,22 +56,25 @@ data CredPairStorage = CredPairStorage
  - if that's going to be done, so that the creds can be encrypted using the
  - cipher. The EncryptionIsSetup is witness to that being the case.
  -}
-setRemoteCredPair :: EncryptionIsSetup -> RemoteConfig -> RemoteGitConfig -> CredPairStorage -> Maybe CredPair -> Annex RemoteConfig
-setRemoteCredPair = setRemoteCredPair' id go
-  where
-	go c = either (const (ParsedRemoteConfig mempty c)) id (parseEncryptionConfig c)
-
-setRemoteCredPair'
-	:: (ProposedAccepted String -> a)
-	-> (M.Map RemoteConfigField a -> ParsedRemoteConfig)
-	-> EncryptionIsSetup
-	-> M.Map RemoteConfigField a
+setRemoteCredPair
+	:: EncryptionIsSetup
+	-> ParsedRemoteConfig
 	-> RemoteGitConfig
 	-> CredPairStorage
 	-> Maybe CredPair
-	-> Annex (M.Map RemoteConfigField a)
-setRemoteCredPair' mkval parseconfig encsetup c gc storage mcreds = case mcreds of
-	Nothing -> maybe (return c) (setRemoteCredPair' mkval parseconfig encsetup c gc storage . Just)
+	-> Annex RemoteConfig
+setRemoteCredPair encsetup pc gc storage mcreds = unparsedRemoteConfig <$>
+	setRemoteCredPair' pc encsetup gc storage mcreds
+
+setRemoteCredPair'
+	:: ParsedRemoteConfig
+	-> EncryptionIsSetup
+	-> RemoteGitConfig
+	-> CredPairStorage
+	-> Maybe CredPair
+	-> Annex ParsedRemoteConfig
+setRemoteCredPair' pc encsetup gc storage mcreds = case mcreds of
+	Nothing -> maybe (return pc) (setRemoteCredPair' pc encsetup gc storage . Just)
 		=<< getRemoteCredPair pc gc storage
 	Just creds
 		| embedCreds pc -> do
@@ -80,7 +83,7 @@ setRemoteCredPair' mkval parseconfig encsetup c gc storage mcreds = case mcreds
 			storeconfig creds key =<< remoteCipher pc gc
 		| otherwise -> do
 			localcache creds
-			return c
+			return pc
   where
 	localcache creds = writeCacheCredPair creds storage
 
@@ -88,12 +91,15 @@ setRemoteCredPair' mkval parseconfig encsetup c gc storage mcreds = case mcreds
 		cmd <- gpgCmd <$> Annex.getGitConfig
 		s <- liftIO $ encrypt cmd (pc, gc) cipher
 			(feedBytes $ L.pack $ encodeCredPair creds)
-			(readBytes $ return . L.unpack)
-		return $ M.insert key (mkval (Accepted (toB64 s))) c
+			(readBytesStrictly $ return . S.unpack)
+		storeconfig' key (Accepted (toB64 s))
 	storeconfig creds key Nothing =
-		return $ M.insert key (mkval (Accepted (toB64 $ encodeCredPair creds))) c
+		storeconfig' key (Accepted (toB64 $ encodeCredPair creds))
 	
-	pc = parseconfig c
+	storeconfig' key val = return $ pc
+		{ parsedRemoteConfigMap = M.insert key (RemoteConfigValue val) (parsedRemoteConfigMap pc)
+		, unparsedRemoteConfig = M.insert key val (unparsedRemoteConfig pc)
+		}
 
 {- Gets a remote's credpair, from the environment if set, otherwise
  - from the cache in gitAnnexCredsDir, or failing that, from the
@@ -106,7 +112,12 @@ getRemoteCredPair c gc storage = maybe fromcache (return . Just) =<< fromenv
 	fromconfig = do
 		let key = credPairRemoteField storage
 		mcipher <- remoteCipher' c gc
-		case (fromProposedAccepted <$> getRemoteConfigValue key c, mcipher) of
+		-- The RemoteConfig value may be passed through.
+		-- Check for those first, because getRemoteConfigValue
+		-- will throw an error if it does not find it.
+		let getval = M.lookup key (getRemoteConfigPassedThrough c)
+			<|> getRemoteConfigValue key c			
+		case (getval, mcipher) of
 			(Nothing, _) -> return Nothing
 			(Just enccreds, Just (cipher, storablecipher)) ->
 				fromenccreds enccreds cipher storablecipher
@@ -116,7 +127,7 @@ getRemoteCredPair c gc storage = maybe fromcache (return . Just) =<< fromenv
 		cmd <- gpgCmd <$> Annex.getGitConfig
 		mcreds <- liftIO $ catchMaybeIO $ decrypt cmd (c, gc) cipher
 			(feedBytes $ L.pack $ fromB64 enccreds)
-			(readBytes $ return . L.unpack)
+			(readBytesStrictly $ return . S.unpack)
 		case mcreds of
 			Just creds -> fromcreds creds
 			Nothing -> do
