@@ -6,6 +6,7 @@
  -}
 
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Database.Keys (
 	DbHandle,
@@ -44,6 +45,7 @@ import Git.Types
 import Git.Index
 
 import qualified Data.ByteString as S
+import qualified Data.ByteString.Char8 as S8
 import qualified System.FilePath.ByteString as P
 
 {- Runs an action that reads from the database.
@@ -59,7 +61,7 @@ import qualified System.FilePath.ByteString as P
  -}
 runReader :: Monoid v => (SQL.ReadHandle -> Annex v) -> Annex v
 runReader a = do
-	h <- getDbHandle
+	h <- Annex.getState Annex.keysdbhandle
 	withDbState h go
   where
 	go DbUnavailable = return (mempty, DbUnavailable)
@@ -83,7 +85,7 @@ runReaderIO a = runReader (liftIO . a)
  - The database is created if it doesn't exist yet. -}
 runWriter :: (SQL.WriteHandle -> Annex ()) -> Annex ()
 runWriter a = do
-	h <- getDbHandle
+	h <- Annex.getState Annex.keysdbhandle
 	withDbState h go
   where
 	go st@(DbOpen qh) = do
@@ -99,17 +101,6 @@ runWriter a = do
 runWriterIO :: (SQL.WriteHandle -> IO ()) -> Annex ()
 runWriterIO a = runWriter (liftIO . a)
 
-{- Gets the handle cached in Annex state; creates a new one if it's not yet
- - available, but doesn't open the database. -}
-getDbHandle :: Annex DbHandle
-getDbHandle = go =<< Annex.getState Annex.keysdbhandle
-  where
-	go (Just h) = pure h
-	go Nothing = do
-		h <- liftIO newDbHandle
-		Annex.changeState $ \s -> s { Annex.keysdbhandle = Just h }
-		return h
-
 {- Opens the database, perhaps creating it if it doesn't exist yet.
  -
  - Multiple readers and writers can have the database open at the same
@@ -151,9 +142,7 @@ openDb createdb _ = catchPermissionDenied permerr $ withExclusiveLock gitAnnexKe
  - data to it.
  -}
 closeDb :: Annex ()
-closeDb = Annex.getState Annex.keysdbhandle >>= \case
-	Nothing -> return ()
-	Just h -> liftIO (closeDbHandle h)
+closeDb = liftIO . closeDbHandle =<< Annex.getState Annex.keysdbhandle
 
 addAssociatedFile :: Key -> TopFilePath -> Annex ()
 addAssociatedFile k f = runWriterIO $ SQL.addAssociatedFile k f
@@ -237,8 +226,8 @@ reconcileStaged qh = do
 		Nothing -> noop
   where
 	go cur indexcache = do
-		(l, cleanup) <- inRepo $ pipeNullSplit diff
-		changed <- procdiff (map decodeBL' l) False
+		(l, cleanup) <- inRepo $ pipeNullSplit' diff
+		changed <- procdiff l False
 		void $ liftIO cleanup
 		-- Flush database changes immediately
 		-- so other processes can see them.
@@ -278,15 +267,16 @@ reconcileStaged qh = do
 		, Param "--no-ext-diff"
 		]
 	
-	procdiff (info:file:rest) changed = case words info of
-		((':':_srcmode):dstmode:_srcsha:dstsha:_change:[])
-			-- Only want files, not symlinks
-			| dstmode /= decodeBS' (fmtTreeItemType TreeSymlink) -> do
-				maybe noop (reconcile (asTopFilePath (toRawFilePath file)))
-					=<< catKey (Ref dstsha)
-				procdiff rest True
-			| otherwise -> procdiff rest changed
-		_ -> return changed -- parse failed
+	procdiff (info:file:rest) changed
+		| ":" `S.isPrefixOf` info = case S8.words info of
+			(_colonsrcmode:dstmode:_srcsha:dstsha:_change:[])
+				-- Only want files, not symlinks
+				| dstmode /= fmtTreeItemType TreeSymlink -> do
+					maybe noop (reconcile (asTopFilePath file))
+						=<< catKey (Ref dstsha)
+					procdiff rest True
+				| otherwise -> procdiff rest changed
+			_ -> return changed -- parse failed
 	procdiff _ changed = return changed
 
 	-- Note that database writes done in here will not necessarily 
