@@ -43,7 +43,7 @@ readFsckResults u = do
 	deserialize ("truncated":ls) = deserialize' ls True
 	deserialize ls = deserialize' ls False
 	deserialize' ls t =
-		let s = S.fromList $ map Ref ls
+		let s = S.fromList $ map (Ref . encodeBS') ls
 		in if S.null s then FsckFailed else FsckFoundMissing s t
 
 clearFsckResults :: UUID -> Annex ()
