@@ -11,6 +11,8 @@ module Remote (
 	Remote,
 	uuid,
 	name,
+	action,
+	verifiedAction,
 	storeKey,
 	retrieveKeyFile,
 	retrieveKeyFileCheap,
@@ -50,6 +52,7 @@ module Remote (
 	nameToUUID,
 	nameToUUID',
 	showTriedRemotes,
+	listRemoteNames,
 	showLocations,
 	forceTrust,
 	logStatus,
@@ -72,11 +75,27 @@ import Logs.Trust
 import Logs.Location hiding (logStatus)
 import Logs.Web
 import Remote.List
+import Remote.List.Util
 import Config
 import Config.DynamicConfig
 import Git.Types (RemoteName, ConfigKey(..), fromConfigValue)
 import Utility.Aeson
 
+{- Runs an action that may throw exceptions, catching and displaying them. -}
+action :: Annex () -> Annex Bool
+action a = tryNonAsync a >>= \case
+	Right () -> return True
+	Left e -> do
+		warning (show e)
+		return False
+
+verifiedAction :: Annex Verification -> Annex (Bool, Verification)
+verifiedAction a = tryNonAsync a >>= \case
+	Right v -> return (True, v)
+	Left e -> do
+		warning (show e)
+		return (False, UnVerified)
+
 {- Map from UUIDs of Remotes to a calculated value. -}
 remoteMap :: (Remote -> v) -> Annex (M.Map UUID v)
 remoteMap mkv = remoteMap' mkv (pure . mkk)
@@ -269,7 +288,7 @@ remoteFromUUID u = ifM ((==) u <$> getUUID)
 	findinmap = M.lookup u <$> remoteMap id
 	{- Re-read remote list in case a new remote has popped up. -}
 	tryharder = do
-		void remoteListRefresh
+		remotesChanged
 		findinmap
 
 {- Filters a list of remotes to ones that have the listed uuids. -}
@@ -347,8 +366,11 @@ showLocations separateuntrusted key exclude nolocmsg = do
 showTriedRemotes :: [Remote] -> Annex ()
 showTriedRemotes [] = noop
 showTriedRemotes remotes =
-	showLongNote $ "Unable to access these remotes: " ++
-		intercalate ", " (map name remotes)
+	showLongNote $ "Unable to access these remotes: "
+		++ listRemoteNames remotes
+
+listRemoteNames :: [Remote] -> String
+listRemoteNames remotes = intercalate ", " (map name remotes)
 
 forceTrust :: TrustLevel -> String -> Annex ()
 forceTrust level remotename = do
