@@ -1,6 +1,6 @@
 {- Standard git remotes.
  -
- - Copyright 2011-2019 Joey Hess <id@joeyh.name>
+ - Copyright 2011-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -12,7 +12,7 @@ module Remote.Git (
 	remote,
 	configRead,
 	repoAvail,
-	onLocal,
+	onLocalRepo,
 ) where
 
 import Annex.Common
@@ -149,15 +149,15 @@ gitSetup (Enable _) Nothing _ _ _ = error "unable to enable git remote with no s
 configRead :: Bool -> Git.Repo -> Annex Git.Repo
 configRead autoinit r = do
 	gc <- Annex.getRemoteGitConfig r
-	u <- getRepoUUID r
+	hasuuid <- (/= NoUUID) <$> getRepoUUID r
 	annexignore <- liftIO $ getDynamicConfig (remoteAnnexIgnore gc)
-	case (repoCheap r, annexignore, u) of
+	case (repoCheap r, annexignore, hasuuid) of
 		(_, True, _) -> return r
 		(True, _, _)
-			| remoteAnnexCheckUUID gc -> tryGitConfigRead autoinit r
+			| remoteAnnexCheckUUID gc -> tryGitConfigRead autoinit r hasuuid
 			| otherwise -> return r
-		(False, _, NoUUID) -> configSpecialGitRemotes r >>= \case
-			Nothing -> tryGitConfigRead autoinit r
+		(False, _, False) -> configSpecialGitRemotes r >>= \case
+			Nothing -> tryGitConfigRead autoinit r False
 			Just r' -> return r'
 		_ -> return r
 
@@ -183,7 +183,7 @@ gen r u rc gc rs
 			, name = Git.repoDescribe r
 			, storeKey = copyToRemote new st
 			, retrieveKeyFile = copyFromRemote new st
-			, retrieveKeyFileCheap = copyFromRemoteCheap new st
+			, retrieveKeyFileCheap = copyFromRemoteCheap new st r
 			, retrievalSecurityPolicy = RetrievalAllKeysSecure
 			, removeKey = dropKey new st
 			, lockContent = Just (lockKey new st)
@@ -244,12 +244,12 @@ repoAvail r
 
 {- Tries to read the config for a specified remote, updates state, and
  - returns the updated repo. -}
-tryGitConfigRead :: Bool -> Git.Repo -> Annex Git.Repo
-tryGitConfigRead autoinit r 
+tryGitConfigRead :: Bool -> Git.Repo -> Bool -> Annex Git.Repo
+tryGitConfigRead autoinit r hasuuid
 	| haveconfig r = return r -- already read
 	| Git.repoIsSsh r = storeUpdatedRemote $ do
 		v <- Ssh.onRemote NoConsumeStdin r
-			(pipedconfig autoinit (Git.repoDescribe r), return (Left $ giveup "configlist failed"))
+			(pipedconfig Git.Config.ConfigList autoinit (Git.repoDescribe r), return (Left $ giveup "configlist failed"))
 			"configlist" [] configlistfields
 		case v of
 			Right r'
@@ -258,14 +258,17 @@ tryGitConfigRead autoinit r
 			Left _ -> configlist_failed
 	| Git.repoIsHttp r = storeUpdatedRemote geturlconfig
 	| Git.GCrypt.isEncrypted r = handlegcrypt =<< getConfigMaybe (remoteAnnexConfig r "uuid")
-	| Git.repoIsUrl r = return r
-	| otherwise = storeUpdatedRemote $ liftIO $ 
-		readlocalannexconfig `catchNonAsync` (const $ return r)
+	| Git.repoIsUrl r = do
+		set_ignore "uses a protocol not supported by git-annex" False
+		return r
+	| otherwise = storeUpdatedRemote $
+		liftIO readlocalannexconfig
+			`catchNonAsync` const failedreadlocalconfig
   where
 	haveconfig = not . M.null . Git.config
 
-	pipedconfig mustincludeuuuid configloc cmd params = do
-		v <- liftIO $ Git.Config.fromPipe r cmd params
+	pipedconfig st mustincludeuuuid configloc cmd params = do
+		v <- liftIO $ Git.Config.fromPipe r cmd params st
 		case v of
 			Right (r', val, _err) -> do
 				unless (isUUIDConfigured r' || S.null val || not mustincludeuuuid) $ do
@@ -282,7 +285,7 @@ tryGitConfigRead autoinit r
 			liftIO $ hClose h
 			let url = Git.repoLocation r ++ "/config"
 			ifM (liftIO $ Url.downloadQuiet nullMeterUpdate url tmpfile uo)
-				( Just <$> pipedconfig False url "git" [Param "config", Param "--null", Param "--list", Param "--file", File tmpfile]
+				( Just <$> pipedconfig Git.Config.ConfigNullList False url "git" [Param "config", Param "--null", Param "--list", Param "--file", File tmpfile]
 				, return Nothing
 				)
 		case v of
@@ -334,11 +337,20 @@ tryGitConfigRead autoinit r
 	readlocalannexconfig = do
 		let check = do
 			Annex.BranchState.disableUpdate
-			catchNonAsync ensureInitialized (warning . show)
+			catchNonAsync autoInitialize $ \e ->
+				warning $ "remote " ++ Git.repoDescribe r ++
+					" :"  ++ show e
 			Annex.getState Annex.repo
 		s <- Annex.new r
 		Annex.eval s $ check `finally` stopCoProcesses
 		
+	failedreadlocalconfig = do
+		unless hasuuid $ case Git.remoteName r of
+			Nothing -> noop
+			Just n -> do
+				warning $ "Remote " ++ n ++ " cannot currently be accessed."
+		return r
+		
 	configlistfields = if autoinit
 		then [(Fields.autoInit, "1")]
 		else []
@@ -375,7 +387,7 @@ inAnnex rmt st key = do
 	inAnnex' repo rmt st key
 
 inAnnex' :: Git.Repo -> Remote -> State -> Key -> Annex Bool
-inAnnex' repo rmt (State connpool duc _ _) key
+inAnnex' repo rmt st@(State connpool duc _ _ _) key
 	| Git.repoIsHttp repo = checkhttp
 	| Git.repoIsUrl repo = checkremote
 	| otherwise = checklocal
@@ -393,7 +405,7 @@ inAnnex' repo rmt (State connpool duc _ _) key
 	checklocal = ifM duc
 		( guardUsable repo (cantCheck repo) $
 			maybe (cantCheck repo) return
-				=<< onLocalFast repo rmt (Annex.Content.inAnnexSafe key)
+				=<< onLocalFast st (Annex.Content.inAnnexSafe key)
 		, cantCheck repo
 		)
 
@@ -413,31 +425,27 @@ keyUrls gc repo r key = map tourl locs'
 #endif
 	remoteconfig = gitconfig r
 
-dropKey :: Remote -> State -> Key -> Annex Bool
+dropKey :: Remote -> State -> Key -> Annex ()
 dropKey r st key = do
 	repo <- getRepo r
-	catchNonAsync
-		(dropKey' repo r st key)
-		(\e -> warning (show e) >> return False)
+	dropKey' repo r st key
 
-dropKey' :: Git.Repo -> Remote -> State -> Key -> Annex Bool
-dropKey' repo r (State connpool duc _ _) key
+dropKey' :: Git.Repo -> Remote -> State -> Key -> Annex ()
+dropKey' repo r st@(State connpool duc _ _ _) key
 	| not $ Git.repoIsUrl repo = ifM duc
-		( guardUsable repo (return False) $
-			commitOnCleanup repo r $ onLocalFast repo r $ do
+		( guardUsable repo (giveup "cannot access remote") $
+			commitOnCleanup repo r st $ onLocalFast st $ do
 				whenM (Annex.Content.inAnnex key) $ do
-					Annex.Content.lockContentForRemoval key $ \lock -> do
+					let cleanup = logStatus key InfoMissing
+					Annex.Content.lockContentForRemoval key cleanup $ \lock -> do
 						Annex.Content.removeAnnex lock
-						logStatus key InfoMissing
+						cleanup
 					Annex.Content.saveState True
-				return True
-		, return False
+		, giveup "remote does not have expected annex.uuid value"
 		)
-	| Git.repoIsHttp repo = do
-		warning "dropping from http remote not supported"
-		return False
-	| otherwise = commitOnCleanup repo r $ do
-		let fallback = Ssh.dropKey repo key
+	| Git.repoIsHttp repo = giveup "dropping from http remote not supported"
+	| otherwise = commitOnCleanup repo r st $ do
+		let fallback = Ssh.dropKey' repo key
 		P2PHelper.remove (Ssh.runProto r connpool (return False) fallback) key
 
 lockKey :: Remote -> State -> Key -> (VerifiedCopy -> Annex r) -> Annex r
@@ -446,14 +454,14 @@ lockKey r st key callback = do
 	lockKey' repo r st key callback
 
 lockKey' :: Git.Repo -> Remote -> State -> Key -> (VerifiedCopy -> Annex r) -> Annex r
-lockKey' repo r (State connpool duc _ _) key callback
+lockKey' repo r st@(State connpool duc _ _ _) key callback
 	| not $ Git.repoIsUrl repo = ifM duc
 		( guardUsable repo failedlock $ do
 			inorigrepo <- Annex.makeRunner
 			-- Lock content from perspective of remote,
 			-- and then run the callback in the original
 			-- annex monad, not the remote's.
-			onLocalFast repo r $ 
+			onLocalFast st $ 
 				Annex.Content.lockContentShared key $
 					liftIO . inorigrepo . callback
 		, failedlock
@@ -464,18 +472,18 @@ lockKey' repo r (State connpool duc _ _) key callback
 		P2PHelper.lock withconn Ssh.runProtoConn (uuid r) key callback
 	| otherwise = failedlock
   where
-	fallback = do
+	fallback = withNullHandle $ \nullh -> do
 		Just (cmd, params) <- Ssh.git_annex_shell ConsumeStdin
 			repo "lockcontent"
 			[Param $ serializeKey key] []
-		(Just hin, Just hout, Nothing, p) <- liftIO $ 
-			withFile devNull WriteMode $ \nullh ->
-				createProcess $
-					 (proc cmd (toCommand params))
-						{ std_in = CreatePipe
-						, std_out = CreatePipe
-						, std_err = UseHandle nullh
-						}
+		let p = (proc cmd (toCommand params))
+			{ std_in = CreatePipe
+			, std_out = CreatePipe
+			, std_err = UseHandle nullh
+			}
+		bracketIO (createProcess p) cleanupProcess fallback'
+
+	fallback' (Just hin, Just hout, Nothing, p) = do
 		v <- liftIO $ tryIO $ getProtocolLine hout
 		let signaldone = void $ tryNonAsync $ liftIO $ mapM_ tryNonAsync
 			[ hPutStrLn hout ""
@@ -502,51 +510,60 @@ lockKey' repo r (State connpool duc _ _) key callback
 					showNote "lockcontent failed"
 					signaldone
 					failedlock
+	fallback' _ = error "internal"
+
 	failedlock = giveup "can't lock content"
 
 {- Tries to copy a key's content from a remote's annex to a file. -}
-copyFromRemote :: Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex (Bool, Verification)
+copyFromRemote :: Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex Verification
 copyFromRemote = copyFromRemote' False
 
-copyFromRemote' :: Bool -> Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex (Bool, Verification)
+copyFromRemote' :: Bool -> Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex Verification
 copyFromRemote' forcersync r st key file dest meterupdate = do
 	repo <- getRepo r
 	copyFromRemote'' repo forcersync r st key file dest meterupdate
 
-copyFromRemote'' :: Git.Repo -> Bool -> Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex (Bool, Verification)
-copyFromRemote'' repo forcersync r st@(State connpool _ _ _) key file dest meterupdate
-	| Git.repoIsHttp repo = unVerified $ do
+copyFromRemote'' :: Git.Repo -> Bool -> Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> Annex Verification
+copyFromRemote'' repo forcersync r st@(State connpool _ _ _ _) key file dest meterupdate
+	| Git.repoIsHttp repo = do
 		gc <- Annex.getGitConfig
-		Url.withUrlOptionsPromptingCreds $
+		ok <- Url.withUrlOptionsPromptingCreds $
 			Annex.Content.downloadUrl key meterupdate (keyUrls gc repo r key) dest
-	| not $ Git.repoIsUrl repo = guardUsable repo (unVerified (return False)) $ do
+		unless ok $
+			giveup "failed to download content"
+		return UnVerified
+	| not $ Git.repoIsUrl repo = guardUsable repo (giveup "cannot access remote") $ do
 		params <- Ssh.rsyncParams r Download
 		u <- getUUID
 		hardlink <- wantHardLink
 		-- run copy from perspective of remote
-		onLocalFast repo r $ do
-			v <- Annex.Content.prepSendAnnex key
-			case v of
-				Nothing -> return (False, UnVerified)
-				Just (object, checksuccess) -> do
-					copier <- mkCopier hardlink st params
-					runTransfer (Transfer Download u (fromKey id key))
-						file stdRetry $ \p ->
-							metered (Just (combineMeterUpdate p meterupdate)) key $ \_ p' -> 
-								copier object dest p' checksuccess
+		onLocalFast st $ Annex.Content.prepSendAnnex key >>= \case
+			Just (object, checksuccess) -> do
+				copier <- mkCopier hardlink st params
+				(ok, v) <- runTransfer (Transfer Download u (fromKey id key))
+					file stdRetry $ \p ->
+						metered (Just (combineMeterUpdate p meterupdate)) key $ \_ p' -> 
+							copier object dest p' checksuccess
+				if ok
+					then return v
+					else giveup "failed to retrieve content from remote"
+			Nothing -> giveup "content is not present in remote"
 	| Git.repoIsSsh repo = if forcersync
-		then fallback meterupdate
+		then do
+			(ok, v) <- fallback meterupdate
+			if ok
+				then return v
+				else giveup "failed to retrieve content from remote"
 		else P2PHelper.retrieve
 			(\p -> Ssh.runProto r connpool (return (False, UnVerified)) (fallback p))
 			key file dest meterupdate
-	| otherwise = do
-		warning "copying from non-ssh, non-http remote not supported"
-		unVerified (return False)
+	| otherwise = giveup "copying from non-ssh, non-http remote not supported"
   where
 	fallback p = unVerified $ feedprogressback $ \p' -> do
 		oh <- mkOutputHandlerQuiet
 		Ssh.rsyncHelper oh (Just (combineMeterUpdate p' p))
 			=<< Ssh.rsyncParamsRemote False r Download key dest file
+
 	{- Feed local rsync's progress info back to the remote,
 	 - by forking a feeder thread that runs
 	 - git-annex-shell transferinfo at the same time
@@ -575,7 +592,7 @@ copyFromRemote'' repo forcersync r st@(State connpool _ _ _) key file dest meter
 			repo "transferinfo" 
 			[Param $ serializeKey key] fields
 		v <- liftIO (newEmptySV :: IO (MSampleVar Integer))
-		pidv <- liftIO $ newEmptyMVar
+		pv <- liftIO $ newEmptyMVar
 		tid <- liftIO $ forkIO $ void $ tryIO $ do
 			bytes <- readSV v
 			p <- createProcess $
@@ -583,7 +600,7 @@ copyFromRemote'' repo forcersync r st@(State connpool _ _ _) key file dest meter
 					{ std_in = CreatePipe
 					, std_err = CreatePipe
 					}
-			putMVar pidv (processHandle p)
+			putMVar pv p
 			hClose $ stderrHandle p
 			let h = stdinHandle p
 			let send b = do
@@ -602,63 +619,62 @@ copyFromRemote'' repo forcersync r st@(State connpool _ _ _) key file dest meter
 		-- do it in the background.
 		let cleanup = forkIO $ do
 			void $ tryIO $ killThread tid
-			void $ tryNonAsync $
-				maybe noop (void . waitForProcess)
-					=<< tryTakeMVar pidv
+			void $ tryNonAsync $ 
+				maybe noop (void . waitForProcess . processHandle)
+					=<< tryTakeMVar pv
+
+		let forcestop = do
+			void $ tryIO $ killThread tid
+			void $ tryNonAsync $ 
+				maybe noop cleanupProcess
+					=<< tryTakeMVar pv
+
 		bracketIO noop (const cleanup) (const $ a feeder)
+			`onException` liftIO forcestop
 
-copyFromRemoteCheap :: Remote -> State -> Key -> AssociatedFile -> FilePath -> Annex Bool
-copyFromRemoteCheap r st key af file = do
-	repo <- getRepo r
-	copyFromRemoteCheap' repo r st key af file
-
-copyFromRemoteCheap' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> FilePath -> Annex Bool
+copyFromRemoteCheap :: Remote -> State -> Git.Repo -> Maybe (Key -> AssociatedFile -> FilePath -> Annex ())
 #ifndef mingw32_HOST_OS
-copyFromRemoteCheap' repo r st key af file
-	| not $ Git.repoIsUrl repo = guardUsable repo (return False) $ do
+copyFromRemoteCheap r st repo
+	| not $ Git.repoIsUrl repo = Just $ \key _af file -> guardUsable repo (giveup "cannot access remote") $ do
 		gc <- getGitConfigFromState st
 		loc <- liftIO $ gitAnnexLocation key repo gc
 		liftIO $ ifM (R.doesPathExist loc)
 			( do
 				absloc <- absPath (fromRawFilePath loc)
-				catchBoolIO $ do
-					createSymbolicLink absloc file
-					return True
-			, return False
+				createSymbolicLink absloc file
+			, giveup "remote does not contain key"
 			)
-	| Git.repoIsSsh repo =
+	| Git.repoIsSsh repo = Just $ \key af file ->
 		ifM (Annex.Content.preseedTmp key file)
-			( fst <$> copyFromRemote' True r st key af file nullMeterUpdate
-			, return False
+			( void $ copyFromRemote' True r st key af file nullMeterUpdate
+			, giveup "cannot preseed rsync with existing content"
 			)
-	| otherwise = return False
+	| otherwise = Nothing
 #else
-copyFromRemoteCheap' _ _ _ _ _ _ = return False
+copyFromRemoteCheap _ _ _ = Nothing
 #endif
 
 {- Tries to copy a key's content to a remote's annex. -}
-copyToRemote :: Remote -> State -> Key -> AssociatedFile -> MeterUpdate -> Annex Bool
+copyToRemote :: Remote -> State -> Key -> AssociatedFile -> MeterUpdate -> Annex ()
 copyToRemote r st key file meterupdate = do
 	repo <- getRepo r
 	copyToRemote' repo r st key file meterupdate
 
-copyToRemote' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> MeterUpdate -> Annex Bool
-copyToRemote' repo r st@(State connpool duc _ _) key file meterupdate
+copyToRemote' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> MeterUpdate -> Annex ()
+copyToRemote' repo r st@(State connpool duc _ _ _) key file meterupdate
 	| not $ Git.repoIsUrl repo = ifM duc
-		( guardUsable repo (return False) $ commitOnCleanup repo r $
+		( guardUsable repo (giveup "cannot access remote") $ commitOnCleanup repo r st $
 			copylocal =<< Annex.Content.prepSendAnnex key
-		, return False
+		, giveup "remote does not have expected annex.uuid value"
 		)
-	| Git.repoIsSsh repo = commitOnCleanup repo r $
+	| Git.repoIsSsh repo = commitOnCleanup repo r st $
 		P2PHelper.store
-			(\p -> Ssh.runProto r connpool (return False) (copyremotefallback p))
+			(Ssh.runProto r connpool (return False) . copyremotefallback)
 			key file meterupdate
 		
-	| otherwise = do
-		warning "copying to non-ssh repo not supported"
-		return False
+	| otherwise = giveup "copying to non-ssh repo not supported"
   where
-	copylocal Nothing = return False
+	copylocal Nothing = giveup "content not available"
 	copylocal (Just (object, checksuccess)) = do
 		-- The checksuccess action is going to be run in
 		-- the remote's Annex, but it needs access to the local
@@ -668,7 +684,7 @@ copyToRemote' repo r st@(State connpool duc _ _) key file meterupdate
 		u <- getUUID
 		hardlink <- wantHardLink
 		-- run copy from perspective of remote
-		onLocalFast repo r $ ifM (Annex.Content.inAnnex key)
+		res <- onLocalFast st $ ifM (Annex.Content.inAnnex key)
 			( return True
 			, runTransfer (Transfer Download u (fromKey id key)) file stdRetry $ \p -> do
 				copier <- mkCopier hardlink st params
@@ -680,7 +696,11 @@ copyToRemote' repo r st@(State connpool duc _ _) key file meterupdate
 				Annex.Content.saveState True
 				return res
 			)
-	copyremotefallback p = Annex.Content.sendAnnex key noop $ \object -> do
+		unless res $
+			giveup "failed to send content to remote"
+	copyremotefallback p = either (const False) id
+		<$> tryNonAsync (copyremotefallback' p)
+	copyremotefallback' p = Annex.Content.sendAnnex key noop $ \object -> do
 		-- This is too broad really, but recvkey normally
 		-- verifies content anyway, so avoid complicating
 		-- it with a local sendAnnex check and rollback.
@@ -715,6 +735,13 @@ repairRemote r a = return $ do
 		ensureInitialized
 		a `finally` stopCoProcesses
 
+data LocalRemoteAnnex = LocalRemoteAnnex Git.Repo (MVar (Maybe Annex.AnnexState))
+
+{- This can safely be called on a Repo that is not local, but of course
+ - onLocal will not work if used with the result. -}
+mkLocalRemoteAnnex :: Git.Repo -> Annex (LocalRemoteAnnex)
+mkLocalRemoteAnnex repo = LocalRemoteAnnex repo <$> liftIO (newMVar Nothing)
+
 {- Runs an action from the perspective of a local remote.
  -
  - The AnnexState is cached for speed and to avoid resource leaks.
@@ -724,23 +751,29 @@ repairRemote r a = return $ do
  - The remote will be automatically initialized/upgraded first,
  - when possible.
  -}
-onLocal :: Git.Repo -> Remote -> Annex a -> Annex a
-onLocal repo r a = do
-	m <- Annex.getState Annex.remoteannexstate
-	case M.lookup (uuid r) m of
-		Nothing -> do
-			st <- liftIO $ Annex.new repo
-			go (st, ensureInitialized >> a)
-		Just st -> go (st, a)
+onLocal :: State -> Annex a -> Annex a
+onLocal (State _ _ _ _ lra) = onLocal' lra
+
+onLocalRepo :: Git.Repo -> Annex a -> Annex a
+onLocalRepo repo a = do
+	lra <- mkLocalRemoteAnnex repo
+	onLocal' lra a
+
+onLocal' :: LocalRemoteAnnex -> Annex a -> Annex a
+onLocal' (LocalRemoteAnnex repo v) a = liftIO (takeMVar v) >>= \case
+	Nothing -> do
+		st <- liftIO $ Annex.new repo
+		go (st, ensureInitialized >> a)
+	Just st -> go (st, a)
   where
-	cache st = Annex.changeState $ \s -> s
-		{ Annex.remoteannexstate = M.insert (uuid r) st (Annex.remoteannexstate s) }
 	go (st, a') = do
 		curro <- Annex.getState Annex.output
-		(ret, st') <- liftIO $ Annex.run (st { Annex.output = curro }) $
+		let act = Annex.run (st { Annex.output = curro }) $
 			a' `finally` stopCoProcesses
-		cache st'
+		(ret, st') <- liftIO $ act `onException` cache st
+		liftIO $ cache st'
 		return ret
+	cache st = putMVar v (Just st)
 
 {- Faster variant of onLocal.
  -
@@ -749,8 +782,8 @@ onLocal repo r a = do
  - it gets the most current value. Caller of onLocalFast can make changes
  - to the branch, however.
  -}
-onLocalFast :: Git.Repo -> Remote -> Annex a -> Annex a
-onLocalFast repo r a = onLocal repo r $ Annex.BranchState.disableUpdate >> a
+onLocalFast :: State -> Annex a -> Annex a
+onLocalFast st a = onLocal st $ Annex.BranchState.disableUpdate >> a
 
 -- To avoid the overhead of trying copy-on-write every time, it's tried
 -- once and if it fails, is not tried again.
@@ -784,7 +817,7 @@ rsyncOrCopyFile st rsyncparams src dest p =
 		)
   where
 	copycowtried = case st of
-		State _ _ (CopyCoWTried v) _ -> v
+		State _ _ (CopyCoWTried v) _ _ -> v
 	dorsync = do
 		-- dest may already exist, so make sure rsync can write to it
 		void $ liftIO $ tryIO $ allowWrite dest
@@ -796,15 +829,15 @@ rsyncOrCopyFile st rsyncparams src dest p =
 	docopywith a = liftIO $ watchFileSize dest p $
 		a CopyTimeStamps src dest
 
-commitOnCleanup :: Git.Repo -> Remote -> Annex a -> Annex a
-commitOnCleanup repo r a = go `after` a
+commitOnCleanup :: Git.Repo -> Remote -> State -> Annex a -> Annex a
+commitOnCleanup repo r st a = go `after` a
   where
 	go = Annex.addCleanup (RemoteCleanup $ uuid r) cleanup
 	cleanup
-		| not $ Git.repoIsUrl repo = onLocalFast repo r $
+		| not $ Git.repoIsUrl repo = onLocalFast st $
 			doQuietSideAction $
 				Annex.Branch.commit =<< Annex.Branch.commitMessage
-		| otherwise = void $ do
+		| otherwise = do
 			Just (shellcmd, shellparams) <-
 				Ssh.git_annex_shell NoConsumeStdin
 					repo "commit" [] []
@@ -812,10 +845,13 @@ commitOnCleanup repo r a = go `after` a
 			-- Throw away stderr, since the remote may not
 			-- have a new enough git-annex shell to
 			-- support committing.
-			liftIO $ catchMaybeIO $
-				withQuietOutput createProcessSuccess $
-					proc shellcmd $
-						toCommand shellparams
+			liftIO $ void $ catchMaybeIO $ withNullHandle $ \nullh ->
+				let p = (proc shellcmd (toCommand shellparams))
+					{ std_out = UseHandle nullh
+					, std_err = UseHandle nullh
+					}
+				in withCreateProcess p $ \_ _ _ ->
+					forceSuccessProcess p
 
 wantHardLink :: Annex Bool
 wantHardLink = (annexHardLink <$> Annex.getGitConfig)
@@ -857,23 +893,24 @@ mkCopier remotewanthardlink st rsyncparams = do
  - This returns False when the repository UUID is not as expected. -}
 type DeferredUUIDCheck = Annex Bool
 
-data State = State Ssh.P2PSshConnectionPool DeferredUUIDCheck CopyCoWTried (Annex (Git.Repo, GitConfig))
+data State = State Ssh.P2PSshConnectionPool DeferredUUIDCheck CopyCoWTried (Annex (Git.Repo, GitConfig)) LocalRemoteAnnex
 
 getRepoFromState :: State -> Annex Git.Repo
-getRepoFromState (State _ _ _ a) = fst <$> a
+getRepoFromState (State _ _ _ a _) = fst <$> a
 
 #ifndef mingw32_HOST_OS
 {- The config of the remote git repository, cached for speed. -}
 getGitConfigFromState :: State -> Annex GitConfig
-getGitConfigFromState (State _ _ _ a) = snd <$> a
+getGitConfigFromState (State _ _ _ a _) = snd <$> a
 #endif
 
 mkState :: Git.Repo -> UUID -> RemoteGitConfig -> Annex State
 mkState r u gc = do
 	pool <- Ssh.mkP2PSshConnectionPool
 	copycowtried <- liftIO newCopyCoWTried
+	lra <- mkLocalRemoteAnnex r
 	(duc, getrepo) <- go
-	return $ State pool duc copycowtried getrepo
+	return $ State pool duc copycowtried getrepo lra
   where
 	go
 		| remoteAnnexCheckUUID gc = return
@@ -882,7 +919,7 @@ mkState r u gc = do
 			rv <- liftIO newEmptyMVar
 			let getrepo = ifM (liftIO $ isEmptyMVar rv)
 				( do
-					r' <- tryGitConfigRead False r
+					r' <- tryGitConfigRead False r True
 					let t = (r', extractGitConfig FromGitConfig r')
 					void $ liftIO $ tryPutMVar rv t
 					return t
