@@ -6,6 +6,7 @@
  -}
 
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE CPP #-}
 
 module Remote.GitLFS (remote, gen, configKnownUrl) where
 
@@ -15,8 +16,8 @@ import Annex.Url
 import Types.Key
 import Types.Creds
 import Types.ProposedAccepted
+import Types.NumCopies
 import qualified Annex
-import qualified Annex.SpecialRemote.Config
 import qualified Git
 import qualified Git.Types as Git
 import qualified Git.Url
@@ -40,9 +41,14 @@ import Utility.SshHost
 import Utility.Url
 import Logs.Remote
 import Logs.RemoteState
-import qualified Utility.GitLFS as LFS
 import qualified Git.Config
 
+#ifdef WITH_GIT_LFS
+import qualified Network.GitLFS as LFS
+#else
+import qualified Utility.GitLFS as LFS
+#endif
+
 import Control.Concurrent.STM
 import Data.String
 import Network.HTTP.Types
@@ -91,25 +97,25 @@ gen r u rc gc rs = do
 		{ chunkConfig = NoChunks
 		}
 	return $ Just $ specialRemote' specialcfg c
-		(simplyPrepare $ store rs h)
-		(simplyPrepare $ retrieve rs h)
-		(simplyPrepare $ remove h)
-		(simplyPrepare $ checkKey rs h)
-		(this c cst)
+		(store rs h)
+		(retrieve rs h)
+		(remove h)
+		(checkKey rs h)
+		(this c cst h)
   where
-	this c cst = Remote
+	this c cst h = Remote
 		{ uuid = u
 		, cost = cst
 		, name = Git.repoDescribe r
 		, storeKey = storeKeyDummy
-		, retrieveKeyFile = retreiveKeyFileDummy
-		, retrieveKeyFileCheap = retrieveCheap
+		, retrieveKeyFile = retrieveKeyFileDummy
+		, retrieveKeyFileCheap = Nothing
 		-- content stored on git-lfs is hashed with SHA256
 		-- no matter what git-annex key it's for, and the hash
 		-- is checked on download
 		, retrievalSecurityPolicy = RetrievalAllKeysSecure
 		, removeKey = removeKeyDummy
-		, lockContent = Nothing
+		, lockContent = Just $ lockKey (this c cst h) rs h
 		, checkPresent = checkPresentDummy
 		, checkPresentCheap = False
 		, exportActions = exportUnsupported
@@ -127,41 +133,42 @@ gen r u rc gc rs = do
 		-- content cannot be removed from a git-lfs repo
 		, appendonly = True
 		, mkUnavailable = return Nothing
-		, getInfo = gitRepoInfo (this c cst)
+		, getInfo = gitRepoInfo (this c cst h)
 		, claimUrl = Nothing
 		, checkUrl = Nothing
 		, remoteStateHandle = rs
 		}
 
 mySetup :: SetupStage -> Maybe UUID -> Maybe CredPair -> RemoteConfig -> RemoteGitConfig -> Annex (RemoteConfig, UUID)
-mySetup _ mu _ c gc = do
+mySetup ss mu _ c gc = do
 	u <- maybe (liftIO genUUID) return mu
 
 	(c', _encsetup) <- encryptionSetup c gc
 	pc <- either giveup return . parseRemoteConfig c' =<< configParser remote c'
+	let failinitunlessforced msg = case ss of
+		Init -> unlessM (Annex.getState Annex.force) (giveup msg)
+		Enable _ -> noop
 	case (isEncrypted pc, Git.GCrypt.urlPrefix `isPrefixOf` url) of
 		(False, False) -> noop
 		(True, True) -> Remote.GCrypt.setGcryptEncryption pc remotename
-		(True, False) -> unlessM (Annex.getState Annex.force) $
-			giveup $ unwords $
-				[ "Encryption is enabled for this remote,"
-				, "but only the files that git-annex stores on"
-				, "it would be encrypted; "
-				, "anything that git push sends to it would"
-				, "not be encrypted. Recommend prefixing the"
-				, "url with \"gcrypt::\" to also encrypt"
-				, "git pushes."
-				, "(Use --force if you want to use this"
-				, "likely insecure configuration.)"
-				]
-		(False, True) -> unlessM (Annex.getState Annex.force) $
-			giveup $ unwords $
-				[ "You used a \"gcrypt::\" url for this remote,"
-				, "but encryption=none prevents git-annex"
-				, "from encrypting files it stores there."
-				, "(Use --force if you want to use this"
-				, "likely insecure configuration.)"
-				]
+		(True, False) -> failinitunlessforced $ unwords $
+			[ "Encryption is enabled for this remote,"
+			, "but only the files that git-annex stores on"
+			, "it would be encrypted; "
+			, "anything that git push sends to it would"
+			, "not be encrypted. Recommend prefixing the"
+			, "url with \"gcrypt::\" to also encrypt"
+			, "git pushes."
+			, "(Use --force if you want to use this"
+			, "likely insecure configuration.)"
+			]
+		(False, True) -> failinitunlessforced $ unwords
+			[ "You used a \"gcrypt::\" url for this remote,"
+			, "but encryption=none prevents git-annex"
+			, "from encrypting files it stores there."
+			, "(Use --force if you want to use this"
+			, "likely insecure configuration.)"
+			]
 
 	-- Set up remote.name.url to point to the repo,
 	-- (so it's also usable by git as a non-special remote),
@@ -439,18 +446,15 @@ mkDownloadRequest rs k = case (extractKeySha256 k, extractKeySize k) of
 
 store :: RemoteStateHandle -> TVar LFSHandle -> Storer
 store rs h = fileStorer $ \k src p -> getLFSEndpoint LFS.RequestUpload h >>= \case
-	Nothing -> return False
-	Just endpoint -> flip catchNonAsync failederr $ do
+	Nothing -> giveup "unable to connect to git-lfs endpoint"
+	Just endpoint -> do
 		(req, sha256, size) <- mkUploadRequest rs k src
 		sendTransferRequest req endpoint >>= \case
-			Left err -> do
-				warning err
-				return False
 			Right resp -> do
 				body <- liftIO $ httpBodyStorer src p
 				forM_ (LFS.objects resp) $
 					send body sha256 size
-				return True
+			Left err -> giveup err
   where
 	send body sha256 size tro
 		| LFS.resp_oid tro /= sha256 || LFS.resp_size tro /= size =
@@ -465,9 +469,6 @@ store rs h = fileStorer $ \k src p -> getLFSEndpoint LFS.RequestUpload h >>= \ca
 					Just [] -> noop -- server already has it
 					Just reqs -> forM_ reqs $
 						makeSmallAPIRequest . setRequestCheckStatus
-	failederr e = do
-		warning (show e)
-		return False
 
 retrieve :: RemoteStateHandle -> TVar LFSHandle -> Retriever
 retrieve rs h = fileRetriever $ \dest k p -> getLFSEndpoint LFS.RequestDownload h >>= \case
@@ -493,6 +494,16 @@ retrieve rs h = fileRetriever $ \dest k p -> getLFSEndpoint LFS.RequestDownload
 					uo <- getUrlOptions
 					liftIO $ downloadConduit p req dest uo
 
+-- Since git-lfs does not support removing content, nothing needs to be
+-- done to lock content in the remote, except for checking that the content
+-- is actually present.
+lockKey :: Remote -> RemoteStateHandle -> TVar LFSHandle -> Key -> (VerifiedCopy -> Annex a) -> Annex a
+lockKey r rs h key callback = 
+	ifM (checkKey rs h key)
+		( withVerifiedCopy LockedCopy (uuid r) (return True) callback
+		, giveup $ "content seems to be missing from " ++ name r
+		)
+
 checkKey :: RemoteStateHandle -> TVar LFSHandle -> CheckPresent
 checkKey rs h key = getLFSEndpoint LFS.RequestDownload h >>= \case
 	Nothing -> giveup "unable to connect to git-lfs endpoint"
@@ -530,10 +541,5 @@ checkKey rs h key = getLFSEndpoint LFS.RequestDownload h >>= \case
 					giveup "git-lfs server replied with other object than the one we requested"
 				| otherwise -> return True
 
-retrieveCheap :: Key -> AssociatedFile -> FilePath -> Annex Bool
-retrieveCheap _ _ _ = return False
-
 remove :: TVar LFSHandle -> Remover
-remove _h _key = do
-	warning "git-lfs does not support removing content"
-	return False
+remove _h _key = giveup "git-lfs does not support removing content"
