@@ -29,7 +29,6 @@ import qualified P2P.Annex as P2P
 import Control.Concurrent.STM
 import Control.Concurrent.Async
 import qualified Data.ByteString as B
-import Data.Unique
 
 toRepo :: ConsumeStdin -> Git.Repo -> RemoteGitConfig -> SshCommand -> Annex (FilePath, [CommandParam])
 toRepo cs r gc remotecmd = do
@@ -106,8 +105,12 @@ inAnnex r k = do
 	dispatch _ = cantCheck r
 
 {- Removes a key from a remote. -}
-dropKey :: Git.Repo -> Key -> Annex Bool
-dropKey r key = onRemote NoConsumeStdin r (\f p -> liftIO (boolSystem f p), return False) "dropkey"
+dropKey :: Git.Repo -> Key -> Annex ()
+dropKey r key = unlessM (dropKey' r key) $
+	giveup "unable to remove key from remote"
+
+dropKey' :: Git.Repo -> Key -> Annex Bool
+dropKey' r key = onRemote NoConsumeStdin r (\f p -> liftIO (boolSystem f p), return False) "dropkey"
 	[ Param "--quiet", Param "--force"
 	, Param $ serializeKey key
 	]
@@ -195,11 +198,13 @@ data StderrHandlerState = DiscardStderr | DisplayStderr | EndStderrHandler
 
 closeP2PSshConnection :: P2PSshConnection -> IO (P2PSshConnection, Maybe ExitCode)
 closeP2PSshConnection P2P.ClosedConnection = return (P2P.ClosedConnection, Nothing)
-closeP2PSshConnection (P2P.OpenConnection (_st, conn, pid, stderrhandlerst)) = do
-	P2P.closeConnection conn
-	atomically $ writeTVar stderrhandlerst EndStderrHandler
-	exitcode <- waitForProcess pid
-	return (P2P.ClosedConnection, Just exitcode)
+closeP2PSshConnection (P2P.OpenConnection (_st, conn, pid, stderrhandlerst)) =
+	-- mask async exceptions, avoid cleanup being interrupted
+	uninterruptibleMask_ $ do
+		P2P.closeConnection conn
+		atomically $ writeTVar stderrhandlerst EndStderrHandler
+		exitcode <- waitForProcess pid
+		return (P2P.ClosedConnection, Just exitcode)
 
 -- Pool of connections over ssh to git-annex-shell p2pstdio.
 type P2PSshConnectionPool = TVar (Maybe P2PSshConnectionPoolState)
@@ -258,16 +263,14 @@ openP2PSshConnection r connpool = do
 				, std_out = CreatePipe
 				, std_err = CreatePipe
 				}
-		-- Could use getPid, but need to build with older versions
-		-- of process, so instead a unique connection number.
-		connnum <- hashUnique <$> newUnique
+		pidnum <- getPid pid
 		let conn = P2P.P2PConnection
 			{ P2P.connRepo = repo
 			, P2P.connCheckAuth = const False
 			, P2P.connIhdl = to
 			, P2P.connOhdl = from
 			, P2P.connIdent = P2P.ConnIdent $
-				Just $ "ssh connection " ++ show connnum
+				Just $ "ssh connection " ++ show pidnum
 			}
 		stderrhandlerst <- newStderrHandler err
 		runst <- P2P.mkRunState P2P.Client
