@@ -1,6 +1,6 @@
 {- git-annex test suite
  -
- - Copyright 2010-2019 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -19,11 +19,13 @@ import Test.Tasty.Runners
 import Test.Tasty.HUnit
 import Test.Tasty.QuickCheck
 import Test.Tasty.Ingredients.Rerun
+import Test.Tasty.Options
 import Options.Applicative (switch, long, help, internal)
 
 import qualified Data.Map as M
 import qualified Data.ByteString.Lazy.UTF8 as BU8
 import System.Environment
+import Control.Concurrent.STM hiding (check)
 
 import Common
 import CmdLine.GitAnnex.Options
@@ -65,6 +67,7 @@ import qualified Annex.AdjustedBranch
 import qualified Annex.View
 import qualified Annex.View.ViewedFile
 import qualified Logs.View
+import qualified Command.TestRemote
 import qualified Utility.Path
 import qualified Utility.FileMode
 import qualified BuildInfo
@@ -85,16 +88,16 @@ import qualified Utility.Tmp.Dir
 import qualified Utility.FileSystemEncoding
 import qualified Utility.Aeson
 import qualified Utility.CopyFile
+import qualified Types.Remote
 #ifndef mingw32_HOST_OS
 import qualified Remote.Helper.Encryptable
 import qualified Types.Crypto
 import qualified Utility.Gpg
-import qualified Types.Remote
 #endif
 
 optParser :: Parser TestOptions
 optParser = TestOptions
-	<$> suiteOptionParser ingredients (tests False True mempty)
+	<$> snd tastyParser
 	<*> switch
 		( long "keep-failures"
 		<> help "preserve repositories on test failure"
@@ -105,6 +108,15 @@ optParser = TestOptions
 		)
 	<*> cmdParams "non-options are for internal use only"
 
+tastyParser :: ([String], Parser Test.Tasty.Options.OptionSet)
+#if MIN_VERSION_tasty(1,3,0)
+tastyParser = go
+#else
+tastyParser = ([], go)
+#endif
+  where
+ 	go = suiteOptionParser ingredients (tests False True mempty)
+
 runner :: TestOptions -> IO ()
 runner opts
 	| fakeSsh opts = runFakeSsh (internalData opts)
@@ -116,11 +128,15 @@ runner opts
 	-- suite.
 	subenv = "GIT_ANNEX_TEST_SUBPROCESS"
 	runsubprocesstests Nothing = do
+		let warnings = fst tastyParser
+		unless (null warnings) $ do
+			hPutStrLn stderr "warnings from tasty:"
+			mapM_ (hPutStrLn stderr) warnings
 		pp <- Annex.Path.programPath
 		Utility.Env.Set.setEnv subenv "1" True
 		ps <- getArgs
-		(Nothing, Nothing, Nothing, pid) <- createProcess (proc pp ps)
-		exitcode <- waitForProcess pid
+		exitcode <- withCreateProcess (proc pp ps) $
+			\_ _ _ pid -> waitForProcess pid
 		unless (keepFailuresOption opts) finalCleanup
 		exitWith exitcode
 	runsubprocesstests (Just _) = isolateGitConfig $ do
@@ -145,14 +161,16 @@ ingredients =
 
 tests :: Bool -> Bool -> TestOptions -> TestTree
 tests crippledfilesystem adjustedbranchok opts = 
-	testGroup "Tests" $ properties :
-		map (\(d, te) -> withTestMode te initTests (unitTests d)) testmodes
+	testGroup "Tests" $ properties 
+		: withTestMode remotetestmode Nothing testRemotes
+		: map (\(d, te) -> withTestMode te (Just initTests) (unitTests d)) testmodes
   where
 	testmodes = catMaybes
 		[ canadjust ("v8 adjusted unlocked branch", (testMode opts (RepoVersion 8)) { adjustedUnlockedBranch = True })
 		, unlesscrippled ("v8 unlocked", (testMode opts (RepoVersion 8)) { unlockedFiles = True })
 		, unlesscrippled ("v8 locked", testMode opts (RepoVersion 8))
 		]
+	remotetestmode = testMode opts (RepoVersion 8)
 	unlesscrippled v
 		| crippledfilesystem = Nothing
 		| otherwise = Just v
@@ -203,6 +221,65 @@ properties = localOption (QuickCheckTests 1000) $ testGroup "QuickCheck" $
 		, Utility.Hash.props_macs_stable
 		]
 
+testRemotes :: TestTree
+testRemotes = testGroup "Remote Tests"
+	[ testRemote "directory"
+		[ "directory=remotedir"
+		, "encryption=none"
+		]
+		(createDirectory "remotedir")
+	]
+
+testRemote :: String -> [String] -> IO () -> TestTree
+testRemote remotetype config preinitremote = 
+	withResource newEmptyTMVarIO (const noop) $ \getv -> 
+		testGroup ("remote type " ++ remotetype) $ concat
+			[ [testCase "init" (prep getv)]
+			, go getv
+			]
+  where
+	reponame = "test repo"
+	remotename = "testremote"
+	basesz = 1024 * 1024
+	keysizes = Command.TestRemote.keySizes basesz False
+	prep getv = do
+		d <- newmainrepodir
+		setmainrepodir d
+		innewrepo $ do
+			git_annex "init" [reponame, "--quiet"]
+				@? "init failed"
+			preinitremote
+			git_annex "initremote"
+				([ remotename
+				, "type=" ++ remotetype
+				, "--quiet"
+				] ++ config)
+				@? "init failed"
+			r <- annexeval $ either error return 
+				=<< Remote.byName' remotename
+			cache <- Command.TestRemote.newRemoteVariantCache
+			unavailr <- annexeval $ Types.Remote.mkUnavailable r
+			exportr <- annexeval $ Command.TestRemote.exportTreeVariant cache r
+			ks <- annexeval $ mapM Command.TestRemote.randKey keysizes
+			v <- getv
+			cv <- annexeval cache
+			liftIO $ atomically $ putTMVar v
+				(r, (unavailr, (exportr, (ks, cv))))
+	go getv = Command.TestRemote.mkTestTrees runannex mkrs mkunavailr mkexportr mkks
+	  where
+		runannex = inmainrepo . annexeval
+		mkrs = Command.TestRemote.remoteVariants cache mkr basesz False
+		mkr = descas (remotetype ++ " remote") (fst <$> v)
+		mkunavailr = fst . snd <$> v
+		mkexportr = fst . snd . snd <$> v
+		mkks = map (\(sz, n) -> desckeysize sz (getk n))
+			(zip keysizes [0..])
+		getk n = fmap (!! n) (fst . snd . snd . snd <$> v)
+		cache = snd . snd . snd . snd <$> v
+		v = liftIO $ atomically . readTMVar =<< getv
+		descas = Command.TestRemote.Described
+		desckeysize sz = descas ("key size " ++ show sz)
+
 {- These tests set up the test environment, but also test some basic parts
  - of git-annex. They are always run before the unitTests. -}
 initTests :: TestTree
@@ -215,6 +292,8 @@ unitTests :: String -> TestTree
 unitTests note = testGroup ("Unit Tests " ++ note)
 	[ testCase "add dup" test_add_dup
 	, testCase "add extras" test_add_extras
+	, testCase "ignore deleted files" test_ignore_deleted_files
+	, testCase "metadata" test_metadata
 	, testCase "export_import" test_export_import
 	, testCase "export_import_subdir" test_export_import_subdir
 	, testCase "shared clone" test_shared_clone
@@ -281,7 +360,7 @@ unitTests note = testGroup ("Unit Tests " ++ note)
 	, testCase "addurl" test_addurl
 	]
 
--- this test case create the main repo
+-- this test case creates the main repo
 test_init :: Assertion
 test_init = innewrepo $ do
 	ver <- annexVersion <$> getTestMode
@@ -325,6 +404,33 @@ test_add_extras = intmpclonerepo $ do
 	annexed_present wormannexedfile
 	checkbackend wormannexedfile backendWORM
 
+test_ignore_deleted_files :: Assertion
+test_ignore_deleted_files = intmpclonerepo $ do
+	git_annex "get" [annexedfile] @? "get failed"
+	git_annex_expectoutput "find" [] [annexedfile]
+	nukeFile annexedfile
+	-- A file that has been deleted, but the deletion not staged,
+	-- is a special case; make sure git-annex skips these.
+	git_annex_expectoutput "find" [] []
+
+test_metadata :: Assertion
+test_metadata = intmpclonerepo $ do
+	git_annex "metadata" ["-s", "foo=bar", annexedfile] @? "set metadata"
+	git_annex_expectoutput "find" ["--metadata", "foo=bar"] [annexedfile]
+	git_annex_expectoutput "find" ["--metadata", "foo=other"] []
+	writecontent annexedfiledup $ content annexedfiledup
+	add_annex annexedfiledup @? "add of second file with same content failed"
+	annexed_present annexedfiledup
+	git_annex_expectoutput "find" ["--metadata", "foo=bar"]
+		[annexedfile, annexedfiledup]
+	git_annex_shouldfail "metadata" ["--remove", "foo", "."]
+		@? "removing metadata from dir with multiple files failed to fail"
+	git_annex "metadata" ["--remove", "foo", annexedfile]
+		@? "removing metadata failed"
+	git_annex_expectoutput "find" ["--metadata", "foo=bar"] []
+	git_annex "metadata" ["--force", "-s", "foo=bar", "."]
+		@? "recursively set metadata force"
+
 test_shared_clone :: Assertion
 test_shared_clone = intmpsharedclonerepo $ do
 	v <- catchMaybeIO $ Utility.Process.readProcess "git" 
@@ -627,7 +733,7 @@ test_lock_force = intmpclonerepo $ do
 	git_annex "get" [annexedfile] @? "get of file failed"
 	git_annex "unlock" [annexedfile] @? "unlock failed"
 	annexeval $ do
-		Just k <- Annex.WorkTree.lookupFile (toRawFilePath annexedfile)
+		Just k <- Annex.WorkTree.lookupKey (toRawFilePath annexedfile)
 		Database.Keys.removeInodeCaches k
 		Database.Keys.closeDb
 		liftIO . nukeFile =<< Annex.fromRepo Annex.Locations.gitAnnexKeysDbIndexCache
@@ -1425,6 +1531,7 @@ test_adjusted_branch_subtree_regression =
 	withtmpclonerepo $ \r -> whenM (adjustedbranchsupported r) $ do
 		indir r $ do
 			disconnectOrigin
+			origbranch <- annexeval origBranch
 			git_annex "upgrade" [] @? "upgrade failed"
 			git_annex "adjust" ["--unlock", "--force"] @? "adjust failed"
 			createDirectoryIfMissing True "a/b/c"
@@ -1435,7 +1542,7 @@ test_adjusted_branch_subtree_regression =
 			writecontent "a/b/x/y" "foo"
 			git_annex "add" ["a/b/x"] @? "add a/b/x failed"
 			git_annex "sync" [] @? "sync failed"
-			boolSystem "git" [Param "checkout", Param "master"] @? "git checkout master failed"
+			boolSystem "git" [Param "checkout", Param origbranch] @? "git checkout failed"
 			doesFileExist "a/b/x/y" @? ("a/b/x/y missing from master after adjusted branch sync")
 
 {- Set up repos as remotes of each other. -}
@@ -1576,14 +1683,12 @@ test_crypto = do
 	testscheme scheme = do
 		abstmp <- absPath tmpdir
 		testscheme' scheme abstmp
-	testscheme' scheme abstmp = intmpclonerepo $ whenM (Utility.Path.inPath (Utility.Gpg.unGpgCmd gpgcmd)) $ do
-		-- Use a relative path to avoid too long path to gpg's
-		-- agent socket.
+	testscheme' scheme abstmp = intmpclonerepo $ do
 		gpgtmp <- (</> "gpgtmp") <$> relPathCwdToFile abstmp
 		createDirectoryIfMissing False gpgtmp
-		Utility.Gpg.testTestHarness gpgtmp gpgcmd 
+		Utility.Gpg.testTestHarness gpgtmp gpgcmd
 			@? "test harness self-test failed"
-		Utility.Gpg.testHarness gpgtmp gpgcmd $ do
+		void $ Utility.Gpg.testHarness gpgtmp gpgcmd $ do
 			createDirectory "dir"
 			let a cmd = git_annex cmd $
 				[ "foo"
@@ -1604,7 +1709,7 @@ test_crypto = do
 			(c,k) <- annexeval $ do
 				uuid <- Remote.nameToUUID "foo"
 				rs <- Logs.Remote.readRemoteLog
-				Just k <- Annex.WorkTree.lookupFile (toRawFilePath annexedfile)
+				Just k <- Annex.WorkTree.lookupKey (toRawFilePath annexedfile)
 				return (fromJust $ M.lookup uuid rs, k)
 			let key = if scheme `elem` ["hybrid","pubkey"]
 					then Just $ Utility.Gpg.KeyIds [Utility.Gpg.testKeyId]
@@ -1664,7 +1769,7 @@ test_add_subdirs = intmpclonerepo $ do
 	unlessM (hasUnlockedFiles <$> getTestMode) $ do
 		git_annex "sync" [] @? "sync failed"
 		l <- annexeval $ Utility.FileSystemEncoding.decodeBL
-			<$> Annex.CatFile.catObject (Git.Types.Ref "HEAD:dir/foo")
+			<$> Annex.CatFile.catObject (Git.Types.Ref (encodeBS "HEAD:dir/foo"))
 		"../.git/annex/" `isPrefixOf` l @? ("symlink from subdir to .git/annex is wrong: " ++ l)
 
 	createDirectory "dir2"
@@ -1696,19 +1801,20 @@ test_export_import = intmpclonerepo $ do
 	-- is in sync with the adjusted branch, which it may not be
 	-- depending on how the repository was set up.
 	commitchanges
-	git_annex "export" ["master", "--to", "foo"] @? "export to dir failed"
+	origbranch <- annexeval origBranch
+	git_annex "export" [origbranch, "--to", "foo"] @? "export to dir failed"
 	dircontains annexedfile (content annexedfile)
 
 	writedir "import" (content "import")
-	git_annex "import" ["master", "--from", "foo"] @? "import from dir failed"
-	git_annex "merge" ["foo/master"] @? "git annex merge foo/master failed"
+	git_annex "import" [origbranch, "--from", "foo"] @? "import from dir failed"
+	git_annex "merge" ["foo/" ++ origbranch] @? "git annex merge failed"
 	annexed_present_imported "import"
 
 	nukeFile "import"
 	writecontent "import" (content "newimport1")
 	git_annex "add" ["import"] @? "add of import failed"
 	commitchanges
-	git_annex "export" ["master", "--to", "foo"] @? "export modified file to dir failed"
+	git_annex "export" [origbranch, "--to", "foo"] @? "export modified file to dir failed"
 	dircontains "import" (content "newimport1")
 
 	-- verify that export refuses to overwrite modified file
@@ -1717,17 +1823,17 @@ test_export_import = intmpclonerepo $ do
 	writecontent "import" (content "newimport3")
 	git_annex "add" ["import"] @? "add of import failed"
 	commitchanges
-	git_annex_shouldfail "export" ["master", "--to", "foo"] @? "export failed to fail in conflict"
+	git_annex_shouldfail "export" [origbranch, "--to", "foo"] @? "export failed to fail in conflict"
 	dircontains "import" (content "newimport2")
 
 	-- resolving import conflict
-	git_annex "import" ["master", "--from", "foo"] @? "import from dir failed"
+	git_annex "import" [origbranch, "--from", "foo"] @? "import from dir failed"
 	not <$> boolSystem "git" [Param "merge", Param "foo/master", Param "-mmerge"] @? "git merge of conflict failed to exit nonzero"
 	nukeFile "import"
 	writecontent "import" (content "newimport3")
 	git_annex "add" ["import"] @? "add of import failed"
 	commitchanges
-	git_annex "export" ["master", "--to", "foo"] @? "export failed after import conflict"
+	git_annex "export" [origbranch, "--to", "foo"] @? "export failed after import conflict"
 	dircontains "import" (content "newimport3")
   where
 	dircontains f v = 
@@ -1775,11 +1881,13 @@ test_export_import_subdir = intmpclonerepo $ do
 	subannexedfile = "subdir" </> annexedfile
 	
 	testexport = do
-		git_annex "export" ["master:"++subdir, "--to", "foo"] @? "export of subdir failed"
+		origbranch <- annexeval origBranch
+		git_annex "export" [origbranch++":"++subdir, "--to", "foo"] @? "export of subdir failed"
 		dircontains annexedfile (content annexedfile)
 	
 	testimport = do
-		git_annex "import" ["master:"++subdir, "--from", "foo"] @? "import of subdir failed"
+		origbranch <- annexeval origBranch
+		git_annex "import" [origbranch++":"++subdir, "--from", "foo"] @? "import of subdir failed"
 		git_annex "merge" ["foo/master"] @? "git annex merge foo/master failed"
 
 		-- Make sure that import did not import the file to the top
