@@ -1,6 +1,6 @@
 {- git-annex test suite framework
  -
- - Copyright 2010-2019 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -24,6 +24,8 @@ import qualified Backend
 import qualified Git.CurrentRepo
 import qualified Git.Construct
 import qualified Git.Types
+import qualified Git.Branch
+import qualified Git.Ref
 import qualified Types.KeySource
 import qualified Types.Backend
 import qualified Types
@@ -79,10 +81,10 @@ annexeval a = do
 		Annex.setOutput Types.Messages.QuietOutput
 		a `finally` Annex.Action.stopCoProcesses
 
-innewrepo :: Assertion -> Assertion
+innewrepo :: IO () -> IO ()
 innewrepo a = withgitrepo $ \r -> indir r a
 
-inmainrepo :: Assertion -> Assertion
+inmainrepo :: IO a -> IO a
 inmainrepo a = do
 	d <- mainrepodir
 	indir d a
@@ -314,7 +316,7 @@ checkdangling f = ifM (annexeval Config.crippledFileSystem)
 checklocationlog :: FilePath -> Bool -> Assertion
 checklocationlog f expected = do
 	thisuuid <- annexeval Annex.UUID.getUUID
-	r <- annexeval $ Annex.WorkTree.lookupFile (toRawFilePath f)
+	r <- annexeval $ Annex.WorkTree.lookupKey (toRawFilePath f)
 	case r of
 		Just k -> do
 			uuids <- annexeval $ Remote.keyLocations k
@@ -325,7 +327,7 @@ checklocationlog f expected = do
 checkbackend :: FilePath -> Types.Backend -> Assertion
 checkbackend file expected = do
 	b <- annexeval $ maybe (return Nothing) (Backend.getBackend file) 
-		=<< Annex.WorkTree.lookupFile (toRawFilePath file)
+		=<< Annex.WorkTree.lookupKey (toRawFilePath file)
 	assertEqual ("backend for " ++ file) (Just expected) b
 
 checkispointerfile :: FilePath -> Assertion
@@ -417,17 +419,19 @@ testMode opts v = TestMode
 hasUnlockedFiles :: TestMode -> Bool
 hasUnlockedFiles m = unlockedFiles m || adjustedUnlockedBranch m
 
-withTestMode :: TestMode -> TestTree -> TestTree -> TestTree
-withTestMode testmode inittests = withResource prepare release . const
+withTestMode :: TestMode -> Maybe TestTree -> TestTree -> TestTree
+withTestMode testmode minittests = withResource prepare release . const
   where
 	prepare = do
 		setTestMode testmode
 		setmainrepodir =<< newmainrepodir
-		case tryIngredients [consoleTestReporter] mempty inittests of
-			Nothing -> error "No tests found!?"
-			Just act -> unlessM act $
-				error "init tests failed! cannot continue"
-		return ()
+		case minittests of
+			Just inittests ->
+				case tryIngredients [consoleTestReporter] mempty inittests of
+					Nothing -> error "No tests found!?"
+					Just act -> unlessM act $
+						error "init tests failed! cannot continue"
+			Nothing -> return ()
 	release _ = noop
 
 setTestMode :: TestMode -> IO ()
@@ -456,9 +460,9 @@ setTestMode testmode = do
 		]
 runFakeSsh :: [String] -> IO ()
 runFakeSsh ("-n":ps) = runFakeSsh ps
-runFakeSsh (_host:cmd:[]) = do
-	(_, _, _, pid) <- createProcess (shell cmd)
-	exitWith =<< waitForProcess pid
+runFakeSsh (_host:cmd:[]) =
+	withCreateProcess (shell cmd) $
+		\_ _ _ pid -> exitWith =<< waitForProcess pid
 runFakeSsh ps = error $ "fake ssh option parse error: " ++ show ps
 
 getTestMode :: IO TestMode
@@ -577,14 +581,22 @@ backendWORM :: Types.Backend
 backendWORM = backend_ "WORM"
 
 backend_ :: String -> Types.Backend
-backend_ = Backend.lookupBackendVariety . Types.Key.parseKeyVariety . encodeBS
+backend_ = Backend.lookupBuiltinBackendVariety . Types.Key.parseKeyVariety . encodeBS
 
 getKey :: Types.Backend -> FilePath -> IO Types.Key
-getKey b f = fromJust <$> annexeval go
+getKey b f = case Types.Backend.genKey b of
+	Just a -> annexeval $ a ks Utility.Metered.nullMeterUpdate
+	Nothing -> error "internal"
   where
-	go = Types.Backend.getKey b ks Utility.Metered.nullMeterUpdate
 	ks = Types.KeySource.KeySource
 		{ Types.KeySource.keyFilename = toRawFilePath f
 		, Types.KeySource.contentLocation = toRawFilePath f
 		, Types.KeySource.inodeCache = Nothing
 		}
+
+{- Get the name of the original branch, eg the current branch, or
+ - if in an adjusted branch, the parent branch. -}
+origBranch :: Types.Annex String
+origBranch = maybe "foo"
+	(Git.Types.fromRef . Git.Ref.base . Annex.AdjustedBranch.fromAdjustedBranch)
+	<$> Annex.inRepo Git.Branch.current
