@@ -114,6 +114,7 @@ data GitConfig = GitConfig
 	, annexAddUnlocked :: Configurable (Maybe String)
 	, annexSecureHashesOnly :: Bool
 	, annexRetry :: Maybe Integer
+	, annexForwardRetry :: Maybe Integer
 	, annexRetryDelay :: Maybe Seconds
 	, annexAllowedUrlSchemes :: S.Set Scheme
 	, annexAllowedIPAddresses :: String
@@ -123,11 +124,13 @@ data GitConfig = GitConfig
 	, annexCacheCreds :: Bool
 	, annexAutoUpgradeRepository :: Bool
 	, annexCommitMode :: CommitMode
+	, annexSkipUnknown :: Bool
 	, coreSymlinks :: Bool
 	, coreSharedRepository :: SharedRepository
 	, receiveDenyCurrentBranch :: DenyCurrentBranch
 	, gcryptId :: Maybe String
 	, gpgCmd :: GpgCmd
+	, mergeDirectoryRenames :: Maybe String
 	}
 
 extractGitConfig :: ConfigSource -> Git.Repo -> GitConfig
@@ -176,7 +179,7 @@ extractGitConfig configsource r = GitConfig
 	, annexFsckNudge = getbool (annexConfig "fscknudge") True
 	, annexAutoUpgrade = toAutoUpgrade $
 		getmaybe (annexConfig "autoupgrade")
-	, annexExpireUnused = maybe Nothing Just . parseDuration
+	, annexExpireUnused = either (const Nothing) Just . parseDuration
 		<$> getmaybe (annexConfig "expireunused")
 	, annexSecureEraseCommand = getmaybe (annexConfig "secure-erase-command")
 	, annexGenMetaData = getbool (annexConfig "genmetadata") False
@@ -195,6 +198,7 @@ extractGitConfig configsource r = GitConfig
 		fmap Just $ getmaybe (annexConfig "addunlocked")
 	, annexSecureHashesOnly = getbool (annexConfig "securehashesonly") False
 	, annexRetry = getmayberead (annexConfig "retry")
+	, annexForwardRetry = getmayberead (annexConfig "forward-retry")
 	, annexRetryDelay = Seconds
 		<$> getmayberead (annexConfig "retrydelay")
 	, annexAllowedUrlSchemes = S.fromList $ map mkScheme $
@@ -214,11 +218,13 @@ extractGitConfig configsource r = GitConfig
 	, annexCommitMode = if getbool (annexConfig "allowsign") False
 		then ManualCommit
 		else AutomaticCommit
+	, annexSkipUnknown = getbool (annexConfig "skipunknown") True
 	, coreSymlinks = getbool "core.symlinks" True
 	, coreSharedRepository = getSharedRepository r
 	, receiveDenyCurrentBranch = getDenyCurrentBranch r
 	, gcryptId = getmaybe "core.gcrypt-id"
 	, gpgCmd = mkGpgCmd (getmaybe "gpg.program")
+	, mergeDirectoryRenames = getmaybe "directoryrenames"
 	}
   where
 	getbool k d = fromMaybe d $ getmaybebool k
@@ -293,6 +299,7 @@ data RemoteGitConfig = RemoteGitConfig
 	, remoteAnnexSpeculatePresent :: Bool
 	, remoteAnnexBare :: Maybe Bool
 	, remoteAnnexRetry :: Maybe Integer
+	, remoteAnnexForwardRetry :: Maybe Integer
 	, remoteAnnexRetryDelay :: Maybe Seconds
 	, remoteAnnexAllowUnverifiedDownloads :: Bool
 	, remoteAnnexConfigUUID :: Maybe UUID
@@ -343,7 +350,7 @@ extractRemoteGitConfig r remotename = do
 		, remoteAnnexReadOnly = getbool "readonly" False
 		, remoteAnnexCheckUUID = getbool "checkuuid" True
 		, remoteAnnexVerify = getbool "verify" True
-		, remoteAnnexTrackingBranch = Git.Ref <$>
+		, remoteAnnexTrackingBranch = Git.Ref . encodeBS <$>
 			( notempty (getmaybe "tracking-branch")
 			<|> notempty (getmaybe "export-tracking") -- old name
 			)
@@ -354,6 +361,7 @@ extractRemoteGitConfig r remotename = do
 		, remoteAnnexSpeculatePresent = getbool "speculate-present" False
 		, remoteAnnexBare = getmaybebool "bare"
 		, remoteAnnexRetry = getmayberead "retry"
+		, remoteAnnexForwardRetry = getmayberead "forward-retry"
 		, remoteAnnexRetryDelay = Seconds
 			<$> getmayberead "retrydelay"
 		, remoteAnnexAllowUnverifiedDownloads = (== Just "ACKTHPPT") $
