@@ -13,7 +13,7 @@ import Annex.Common
 import qualified Annex
 import qualified Git
 import Config
-import Config.Files
+import Annex.Path
 import Annex.Version
 import Types.RepoVersion
 #ifndef mingw32_HOST_OS
@@ -103,13 +103,16 @@ upgrade automatic destversion = do
 	-- upgrading a git repo other than the current repo.
 	upgraderemote = do
 		rp <- fromRawFilePath <$> fromRepo Git.repoPath
-		cmd <- liftIO readProgramFile
-		liftIO $ boolSystem' cmd
-			[ Param "upgrade"
-			, Param "--quiet"
-			, Param "--autoonly"
+		gitAnnexChildProcess
+			[ "upgrade"
+			, "--quiet"
+			, "--autoonly"
 			]
 			(\p -> p { cwd = Just rp })
+			(\_ _ _ pid -> waitForProcess pid >>= return . \case
+				ExitSuccess -> True
+				_ -> False
+			)
 
 upgradingRemote :: Annex Bool
 upgradingRemote = isJust <$> fromRepo Git.remoteName
