@@ -31,8 +31,8 @@ keyFile0 :: Key -> FilePath
 keyFile0 = Upgrade.V1.keyFile1
 fileKey0 :: FilePath -> Key
 fileKey0 = Upgrade.V1.fileKey1
-lookupFile0 :: FilePath -> Annex (Maybe (Key, Backend))
-lookupFile0 = Upgrade.V1.lookupFile1
+lookupKey0 :: FilePath -> Annex (Maybe (Key, Backend))
+lookupKey0 = Upgrade.V1.lookupKey1
 
 getKeysPresent0 :: FilePath -> Annex [Key]
 getKeysPresent0 dir = ifM (liftIO $ doesDirectoryExist dir)
