@@ -110,16 +110,16 @@ moveFile src dest = tryIO (rename src dest) >>= onrename
 	onrename (Left e)
 		| isPermissionError e = rethrow
 		| isDoesNotExistError e = rethrow
-		| otherwise = viaTmp mv dest ""
+		| otherwise = viaTmp mv dest ()
 	  where
 		rethrow = throwM e
 
-		mv tmp _ = do
-		-- copyFile is likely not as optimised as
-		-- the mv command, so we'll use the command.
-		--
-		-- But, while Windows has a "mv", it does not seem very
-		-- reliable, so use copyFile there.
+		mv tmp () = do
+			-- copyFile is likely not as optimised as
+			-- the mv command, so we'll use the command.
+			--
+			-- But, while Windows has a "mv", it does not seem very
+			-- reliable, so use copyFile there.
 #ifndef mingw32_HOST_OS	
 			-- If dest is a directory, mv would move the file
 			-- into it, which is not desired.
@@ -145,11 +145,10 @@ moveFile src dest = tryIO (rename src dest) >>= onrename
 			(Right s) -> return $ isDirectory s
 #endif
 
-{- Removes a file, which may or may not exist, and does not have to
- - be a regular file.
+{- Removes a file (or symlink), which may or may not exist.
  -
  - Note that an exception is thrown if the file exists but
- - cannot be removed. -}
+ - cannot be removed, or if its a directory. -}
 nukeFile :: FilePath -> IO ()
 nukeFile file = void $ tryWhenExists go
   where
