@@ -17,7 +17,7 @@ module Utility.Gpg (
 	stdEncryptionParams,
 	pipeStrict,
 	feedRead,
-	pipeLazy,
+	feedRead',
 	findPubKeys,
 	UserId,
 	secretKeys,
@@ -47,7 +47,7 @@ import Utility.Tmp
 #endif
 import Utility.Format (decode_c)
 
-import Control.Concurrent
+import Control.Concurrent.Async
 import Control.Monad.IO.Class
 import qualified Data.Map as M
 import Data.Char
@@ -112,21 +112,33 @@ stdEncryptionParams symmetric = enc symmetric ++
 readStrict :: GpgCmd -> [CommandParam] -> IO String
 readStrict (GpgCmd cmd) params = do
 	params' <- stdParams params
-	withHandle StdoutHandle createProcessSuccess (proc cmd params') $ \h -> do
-		hSetBinaryMode h True
-		hGetContentsStrict h
+	let p = (proc cmd params')
+		{ std_out = CreatePipe }
+	withCreateProcess p (go p)
+  where
+	go p _ (Just hout) _ pid = do
+		hSetBinaryMode hout True
+		forceSuccessProcess p pid `after` hGetContentsStrict hout
+	go _ _ _ _ _ = error "internal"
 
 {- Runs gpg, piping an input value to it, and returning its stdout,
  - strictly. -}
 pipeStrict :: GpgCmd -> [CommandParam] -> String -> IO String
 pipeStrict (GpgCmd cmd) params input = do
 	params' <- stdParams params
-	withIOHandles createProcessSuccess (proc cmd params') $ \(to, from) -> do
+	let p = (proc cmd params')
+		{ std_in = CreatePipe
+		, std_out = CreatePipe
+		}
+	withCreateProcess p (go p)
+  where
+	go p (Just to) (Just from) _ pid = do
 		hSetBinaryMode to True
 		hSetBinaryMode from True
 		hPutStr to input
 		hClose to
-		hGetContentsStrict from
+		forceSuccessProcess p pid `after` hGetContentsStrict from
+	go _ _ _ _ _ = error "internal"
 
 {- Runs gpg with some parameters. First sends it a passphrase (unless it
  - is empty) via '--passphrase-fd'. Then runs a feeder action that is
@@ -137,20 +149,26 @@ pipeStrict (GpgCmd cmd) params input = do
  - Runs gpg in batch mode; this is necessary to avoid gpg 2.x prompting for
  - the passphrase.
  -
- - Note that to avoid deadlock with the cleanup stage,
- - the reader must fully consume gpg's input before returning. -}
+ - Note that the reader must fully consume gpg's input before returning. -}
 feedRead :: (MonadIO m, MonadMask m) => GpgCmd -> [CommandParam] -> String -> (Handle -> IO ()) -> (Handle -> m a) -> m a
 feedRead cmd params passphrase feeder reader = do
 #ifndef mingw32_HOST_OS
-	-- pipe the passphrase into gpg on a fd
-	(frompipe, topipe) <- liftIO System.Posix.IO.createPipe
-	liftIO $ void $ forkIO $ do
+	let setup = liftIO $ do
+		-- pipe the passphrase into gpg on a fd
+		(frompipe, topipe) <- System.Posix.IO.createPipe
 		toh <- fdToHandle topipe
-		hPutStrLn toh passphrase
+		t <- async $ do
+			hPutStrLn toh passphrase
+			hClose toh
+		let Fd pfd = frompipe
+		let passphrasefd = [Param "--passphrase-fd", Param $ show pfd]
+		return (passphrasefd, frompipe, toh, t)
+	let cleanup (_, frompipe, toh, t) = liftIO $ do
+		closeFd frompipe
 		hClose toh
-	let Fd pfd = frompipe
-	let passphrasefd = [Param "--passphrase-fd", Param $ show pfd]
-	liftIO (closeFd frompipe) `after` go (passphrasefd ++ params)
+		cancel t
+	bracket setup cleanup $ \(passphrasefd, _, _, _) -> 
+		go (passphrasefd ++ params)
 #else
 	-- store the passphrase in a temp file for gpg
 	withTmpFile "gpg" $ \tmpfile h -> do
@@ -160,27 +178,31 @@ feedRead cmd params passphrase feeder reader = do
 		go $ passphrasefile ++ params
 #endif
   where
-	go params' = pipeLazy cmd params' feeder reader
+	go params' = feedRead' cmd params' feeder reader
 
 {- Like feedRead, but without passphrase. -}
-pipeLazy :: (MonadIO m, MonadMask m) => GpgCmd -> [CommandParam] -> (Handle -> IO ()) -> (Handle -> m a) -> m a
-pipeLazy (GpgCmd cmd) params feeder reader = do
+feedRead' :: (MonadIO m, MonadMask m) => GpgCmd -> [CommandParam] -> (Handle -> IO ()) -> (Handle -> m a) -> m a
+feedRead' (GpgCmd cmd) params feeder reader = do
 	params' <- liftIO $ stdParams $ Param "--batch" : params
 	let p = (proc cmd params')
 		{ std_in = CreatePipe
 		, std_out = CreatePipe
 		, std_err = Inherit
 		}
-	bracket (setup p) (cleanup p) go
+	bracket (setup p) cleanup (go p)
   where
 	setup = liftIO . createProcess
-	cleanup p (_, _, _, pid) = liftIO $ forceSuccessProcess p pid
-	go p = do
-		let (to, from) = ioHandles p
-		liftIO $ void $ forkIO $ do
+	cleanup = liftIO . cleanupProcess
+
+	go p (Just to, Just from, _, pid) =
+		let runfeeder = do
 			feeder to
 			hClose to
-		reader from
+		in bracketIO (async runfeeder) cancel $ const $ do
+			r <- reader from
+			liftIO $ forceSuccessProcess p pid
+			return r
+	go _ _ = error "internal"
 
 {- Finds gpg public keys matching some string. (Could be an email address,
  - a key id, or a name; See the section 'HOW TO SPECIFY A USER ID' of
@@ -241,10 +263,13 @@ maxRecommendedKeySize = 4096
  -}
 genSecretKey :: GpgCmd -> KeyType -> Passphrase -> UserId -> Size -> IO ()
 genSecretKey (GpgCmd cmd) keytype passphrase userid keysize =
-	withHandle StdinHandle createProcessSuccess (proc cmd params) feeder
+	let p = (proc cmd params)
+		{ std_in = CreatePipe }
+	in withCreateProcess p (go p)
   where
 	params = ["--batch", "--gen-key"]
-	feeder h = do
+
+	go p (Just h) _ _ pid = do
 		hPutStr h $ unlines $ catMaybes
 			[ Just $  "Key-Type: " ++ 
 				case keytype of
@@ -259,6 +284,8 @@ genSecretKey (GpgCmd cmd) keytype passphrase userid keysize =
 				else Just $ "Passphrase: " ++ passphrase
 			]
 		hClose h
+		forceSuccessProcess p pid
+	go _ _ _ _ _ = error "internal"
 
 {- Creates a block of high-quality random data suitable to use as a cipher.
  - It is armored, to avoid newlines, since gpg only reads ciphers up to the
@@ -376,11 +403,20 @@ keyBlock public ls = unlines
 #ifndef mingw32_HOST_OS
 {- Runs an action using gpg in a test harness, in which gpg does
  - not use ~/.gpg/, but sets up the test key in a subdirectory of 
- - the passed directory and uses it. -}
-testHarness :: FilePath -> GpgCmd -> IO a -> IO a
-testHarness tmpdir cmd a = bracket setup cleanup (const a)
+ - the passed directory and uses it.
+ -
+ - If the test harness is not able to be set up (eg, because gpg is not
+ - installed or because there is some problem importing the test key,
+ - perhaps related to the agent socket), the action is not run, and Nothing
+ - is returned.
+ -}
+testHarness :: FilePath -> GpgCmd -> IO a -> IO (Maybe a)
+testHarness tmpdir cmd a = ifM (inPath (unGpgCmd cmd))
+	( bracket (eitherToMaybe <$> tryNonAsync setup) cleanup go
+	, return Nothing
+	)
   where
-	var = "GNUPGHOME"		
+	var = "GNUPGHOME"
 
 	setup = do
 		orig <- getEnv var
@@ -395,8 +431,19 @@ testHarness tmpdir cmd a = bracket setup cleanup (const a)
 			[testSecretKey, testKey]
 		return orig
 		
-	cleanup (Just v) = setEnv var v True
-	cleanup Nothing = unsetEnv var
+	cleanup (Just (Just v)) = stopgpgagent >> setEnv var v True
+	cleanup (Just Nothing) = stopgpgagent >> unsetEnv var
+	cleanup Nothing = stopgpgagent
+
+	-- Recent versions of gpg automatically start gpg-agent, or perhaps
+	-- other daemons. Stop them when done. This only affects
+	-- daemons started for the GNUPGHOME that was used.
+	-- Older gpg may not support this, so ignore failure.
+	stopgpgagent = whenM (inPath "gpgconf") $
+		void $ boolSystem "gpgconf" [Param "--kill", Param "all"]
+
+	go (Just _) = Just <$> a
+	go Nothing = return Nothing
 
         makenewdir n = do
 		let subdir = tmpdir </> show n
@@ -406,9 +453,12 @@ testHarness tmpdir cmd a = bracket setup cleanup (const a)
 
 {- Tests the test harness. -}
 testTestHarness :: FilePath -> GpgCmd -> IO Bool
-testTestHarness tmpdir cmd = do
-	keys <- testHarness tmpdir cmd $ findPubKeys cmd testKeyId
-	return $ KeyIds [testKeyId] == keys
+testTestHarness tmpdir cmd =
+	testHarness tmpdir cmd (findPubKeys cmd testKeyId) >>= \case
+		Nothing -> do
+			hPutStrLn stderr "unable to test gpg, setting up the test harness did not succeed"
+			return True
+		Just keys -> return $ KeyIds [testKeyId] == keys
 
 checkEncryptionFile :: GpgCmd -> FilePath -> Maybe KeyIds -> IO Bool
 checkEncryptionFile cmd filename keys =
